const char luarocks_cmd_build_lua[] =
"\n"
"--- Module implementing the LuaRocks \"build\" command.\n"
"-- Builds a rock, compiling its C parts if any.\n"
"local cmd_build = {}\n"
"\n"
"local dir = require(\"luarocks.dir\")\n"
"local pack = require(\"luarocks.pack\")\n"
"local path = require(\"luarocks.path\")\n"
"local util = require(\"luarocks.util\")\n"
"local fetch = require(\"luarocks.fetch\")\n"
"local fs = require(\"luarocks.fs\")\n"
"local deps = require(\"luarocks.deps\")\n"
"local remove = require(\"luarocks.remove\")\n"
"local cfg = require(\"luarocks.core.cfg\")\n"
"local build = require(\"luarocks.build\")\n"
"local writer = require(\"luarocks.manif.writer\")\n"
"local search = require(\"luarocks.search\")\n"
"local make = require(\"luarocks.cmd.make\")\n"
"local cmd = require(\"luarocks.cmd\")\n"
"\n"
"cmd_build.help_summary = \"build/compile a rock.\"\n"
"cmd_build.help_arguments = \"[<flags...>] {<rockspec>|<rock>|<name> [<version>]}\"\n"
"cmd_build.help = [[\n"
"Build and install a rock, compiling its C parts if any.\n"
"Argument may be a rockspec file, a source rock file\n"
"or the name of a rock to be fetched from a repository.\n"
"\n"
"--pack-binary-rock  Do not install rock. Instead, produce a .rock file\n"
"                    with the contents of compilation in the current\n"
"                    directory.\n"
"\n"
"--keep              Do not remove previously installed versions of the\n"
"                    rock after building a new one. This behavior can\n"
"                    be made permanent by setting keep_other_versions=true\n"
"                    in the configuration file.\n"
"\n"
"--branch=<name>     Override the `source.branch` field in the loaded\n"
"                    rockspec. Allows to specify a different branch to \n"
"                    fetch. Particularly for \"dev\" rocks.\n"
"\n"
"--only-deps         Installs only the dependencies of the rock.\n"
"\n"
"--verify            Verify signature of the rockspec or src.rock being\n"
"                    built. If the rockspec or src.rock is being downloaded,\n"
"                    LuaRocks will attempt to download the signature as well.\n"
"                    Otherwise, the signature file should be already\n"
"                    available locally in the same directory.\n"
"                    You need the signer’s public key in your local\n"
"                    keyring for this option to work properly.\n"
"\n"
"--sign              To be used with --pack-binary-rock. Also produce\n"
"                    a signature file for the generated .rock file.\n"
"\n"
"]]..util.deps_mode_help()\n"
"\n"
"--- Build and install a rock.\n"
"-- @param rock_filename string: local or remote filename of a rock.\n"
"-- @param opts table: build options\n"
"-- @return boolean or (nil, string, [string]): True if build was successful,\n"
"-- or false and an error message and an optional error code.\n"
"local function build_rock(rock_filename, opts)\n"
"   assert(type(rock_filename) == \"string\")\n"
"   assert(opts:type() == \"build.opts\")\n"
"\n"
"   local ok, err, errcode\n"
"\n"
"   local unpack_dir\n"
"   unpack_dir, err, errcode = fetch.fetch_and_unpack_rock(rock_filename, nil, opts.verify)\n"
"   if not unpack_dir then\n"
"      return nil, err, errcode\n"
"   end\n"
"\n"
"   local rockspec_filename = path.rockspec_name_from_rock(rock_filename)\n"
"\n"
"   ok, err = fs.change_dir(unpack_dir)\n"
"   if not ok then return nil, err end\n"
"\n"
"   local rockspec\n"
"   rockspec, err, errcode = fetch.load_rockspec(rockspec_filename)\n"
"   if not rockspec then\n"
"      return nil, err, errcode\n"
"   end\n"
"\n"
"   ok, err, errcode = build.build_rockspec(rockspec, opts)\n"
"\n"
"   fs.pop_dir()\n"
"   return ok, err, errcode\n"
"end\n"
"\n"
"local function do_build(ns_name, version, opts)\n"
"   assert(type(ns_name) == \"string\")\n"
"   assert(version == nil or type(version) == \"string\")\n"
"   assert(opts:type() == \"build.opts\")\n"
"\n"
"   local url, err\n"
"   if ns_name:match(\"%.rockspec$\") or ns_name:match(\"%.rock$\") then\n"
"      url = ns_name\n"
"   else\n"
"      url, err = search.find_src_or_rockspec(ns_name, version, true)\n"
"      if not url then\n"
"         return nil, err\n"
"      end\n"
"      local _, namespace = util.split_namespace(ns_name)\n"
"      opts.namespace = namespace\n"
"   end\n"
"\n"
"   if url:match(\"%.rockspec$\") then\n"
"      local rockspec, err, errcode = fetch.load_rockspec(url, nil, opts.verify)\n"
"      if not rockspec then\n"
"         return nil, err, errcode\n"
"      end\n"
"      return build.build_rockspec(rockspec, opts)\n"
"   end\n"
"\n"
"   if url:match(\"%.src%.rock$\") then\n"
"      opts.need_to_fetch = false\n"
"   end\n"
"\n"
"   return build_rock(url, opts)\n"
"end\n"
"\n"
"local function remove_doc_dir(name, version)\n"
"   local install_dir = path.install_dir(name, version)\n"
"   for _, f in ipairs(fs.list_dir(install_dir)) do\n"
"      local doc_dirs = { \"doc\", \"docs\" }\n"
"      for _, d in ipairs(doc_dirs) do\n"
"         if f == d then\n"
"            fs.delete(dir.path(install_dir, f))\n"
"         end\n"
"      end\n"
"   end\n"
"end\n"
"\n"
"--- Driver function for \"build\" command.\n"
"-- @param name string: A local or remote rockspec or rock file.\n"
"-- If a package name is given, forwards the request to \"search\" and,\n"
"-- if returned a result, installs the matching rock.\n"
"-- @param version string: When passing a package name, a version number may\n"
"-- also be given.\n"
"-- @return boolean or (nil, string, exitcode): True if build was successful; nil and an\n"
"-- error message otherwise. exitcode is optionally returned.\n"
"function cmd_build.command(flags, name, version)\n"
"   assert(type(name) == \"string\" or not name)\n"
"   assert(type(version) == \"string\" or not version)\n"
"   \n"
"   if not name then\n"
"      return make.command(flags)\n"
"   end\n"
"\n"
"   name = util.adjust_name_and_namespace(name, flags)\n"
"\n"
"   local opts = build.opts({\n"
"      need_to_fetch = true,\n"
"      minimal_mode = false,\n"
"      deps_mode = deps.get_deps_mode(flags),\n"
"      build_only_deps = not not flags[\"only-deps\"],\n"
"      namespace = flags[\"namespace\"],\n"
"      branch = not not flags[\"branch\"],\n"
"      verify = not not flags[\"verify\"],\n"
"   })\n"
"\n"
"   if flags[\"sign\"] and not flags[\"pack-binary-rock\"] then\n"
"      return nil, \"In the build command, --sign is meant to be used only with --pack-binary-rock\"\n"
"   end\n"
"\n"
"   if flags[\"pack-binary-rock\"] then\n"
"      return pack.pack_binary_rock(name, version, flags[\"sign\"], function()\n"
"         opts.build_only_deps = false\n"
"         local status, err, errcode = do_build(name, version, opts)\n"
"         if status and flags[\"no-doc\"] then\n"
"            remove_doc_dir(name, version)\n"
"         end\n"
"         return status, err, errcode\n"
"      end)\n"
"   end\n"
"   \n"
"   local ok, err = fs.check_command_permissions(flags)\n"
"   if not ok then\n"
"      return nil, err, cmd.errorcodes.PERMISSIONDENIED\n"
"   end\n"
"\n"
"   ok, err = do_build(name, version, opts)\n"
"   if not ok then return nil, err end\n"
"   name, version = ok, err\n"
"\n"
"   if flags[\"no-doc\"] then\n"
"      remove_doc_dir(name, version)\n"
"   end\n"
"\n"
"   if opts.build_only_deps then\n"
"      util.printout(\"Stopping after installing dependencies for \" ..name..\" \"..version)\n"
"      util.printout()\n"
"   else\n"
"      if (not flags[\"keep\"]) and not cfg.keep_other_versions then\n"
"         local ok, err = remove.remove_other_versions(name, version, flags[\"force\"], flags[\"force-fast\"])\n"
"         if not ok then\n"
"            util.printerr(err)\n"
"         end\n"
"      end\n"
"   end\n"
"\n"
"   writer.check_dependencies(nil, deps.get_deps_mode(flags))\n"
"   return name, version\n"
"end\n"
"\n"
"return cmd_build\n"
""
;
