const char luarocks_cmd_config_lua[] =
"--- Module implementing the LuaRocks \"config\" command.\n"
"-- Queries information about the LuaRocks configuration.\n"
"local config_cmd = {}\n"
"\n"
"local persist = require(\"luarocks.persist\")\n"
"local cfg = require(\"luarocks.core.cfg\")\n"
"local util = require(\"luarocks.util\")\n"
"local deps = require(\"luarocks.deps\")\n"
"local dir = require(\"luarocks.dir\")\n"
"local fs = require(\"luarocks.fs\")\n"
"\n"
"config_cmd.help_summary = \"Query information about the LuaRocks configuration.\"\n"
"config_cmd.help_arguments = \"(<key> | <key> <value> --scope=<scope> | <key> --unset --scope=<scope> | )\"\n"
"config_cmd.help = [[\n"
"* When given a configuration key, it prints the value of that key\n"
"  according to the currently active configuration (taking into account\n"
"  all config files and any command-line flags passed)\n"
"\n"
"  Examples:\n"
"     luarocks config lua_interpreter\n"
"     luarocks config variables.LUA_INCDIR\n"
"     luarocks config lua_version\n"
"\n"
"* When given a configuration key and a value,\n"
"  it overwrites the config file (see the --scope option below to determine which)\n"
"  and replaces the value of the given key with the given value.\n"
"\n"
"  * `lua_dir` is a special key as it checks for a valid Lua installation\n"
"    (equivalent to --lua-dir) and sets several keys at once.\n"
"  * `lua_version` is a special key as it changes the default Lua version\n"
"    used by LuaRocks commands (eqivalent to passing --lua-version). \n"
"\n"
"  Examples:\n"
"     luarocks config variables.OPENSSL_DIR /usr/local/openssl\n"
"     luarocks config lua_dir /usr/local\n"
"     luarocks config lua_version 5.3\n"
"\n"
"* When given a configuration key and --unset,\n"
"  it overwrites the config file (see the --scope option below to determine which)\n"
"  and deletes that key from the file.\n"
"\n"
"  Example: luarocks config variables.OPENSSL_DIR --unset\n"
"\n"
"* When given no arguments, it prints the entire currently active\n"
"  configuration, resulting from reading the config files from\n"
"  all scopes.\n"
"\n"
"  Example: luarocks config\n"
"\n"
"OPTIONS\n"
"--scope=<scope>   The scope indicates which config file should be rewritten.\n"
"                  Accepted values are \"system\", \"user\" or \"project\".\n"
"                  * Using a wrapper created with `luarocks init`,\n"
"                    the default is \"project\".\n"
"                  * Using --local (or when `local_by_default` is `true`),\n"
"                    the default is \"user\".\n"
"                  * Otherwise, the default is \"system\".\n"
"\n"
"--json           Output as JSON\n"
"]]\n"
"config_cmd.help_see_also = [[\n"
"\thttps://github.com/luarocks/luarocks/wiki/Config-file-format\n"
"\tfor detailed information on the LuaRocks config file format.\n"
"]]\n"
"\n"
"local function config_file(conf)\n"
"   print(dir.normalize(conf.file))\n"
"   if conf.found then\n"
"      return true\n"
"   else\n"
"      return nil, \"file not found\"\n"
"   end\n"
"end\n"
"\n"
"local cfg_skip = {\n"
"   errorcodes = true,\n"
"   flags = true,\n"
"   platforms = true,\n"
"   root_dir = true,\n"
"   upload_servers = true,\n"
"}\n"
"\n"
"local function should_skip(k, v)\n"
"   return type(v) == \"function\" or cfg_skip[k]\n"
"end\n"
"\n"
"local function cleanup(tbl)\n"
"   local copy = {}\n"
"   for k, v in pairs(tbl) do\n"
"      if not should_skip(k, v) then\n"
"         copy[k] = v\n"
"      end\n"
"   end\n"
"   return copy\n"
"end\n"
"\n"
"local function traverse_varstring(var, tbl, fn, missing_parent)\n"
"   local k, r = var:match(\"^%[([0-9]+)%]%.(.*)$\")\n"
"   if k then\n"
"      k = tonumber(k)\n"
"   else\n"
"      k, r = var:match(\"^([^.[]+)%.(.*)$\")\n"
"      if not k then\n"
"         k, r = var:match(\"^([^[]+)(%[.*)$\")\n"
"      end\n"
"   end\n"
"   \n"
"   if k then\n"
"      if not tbl[k] and missing_parent then\n"
"         missing_parent(tbl, k)\n"
"      end\n"
"\n"
"      if tbl[k] then\n"
"         return traverse_varstring(r, tbl[k], fn, missing_parent)\n"
"      else\n"
"         return nil, \"Unknown entry \" .. k\n"
"      end\n"
"   end\n"
"\n"
"   local i = var:match(\"^%[([0-9]+)%]$\")\n"
"   if i then\n"
"      var = tonumber(i)\n"
"   end\n"
"   \n"
"   return fn(tbl, var)\n"
"end\n"
"\n"
"local function print_json(value)\n"
"   local json_ok, json = util.require_json()\n"
"   if not json_ok then\n"
"      return nil, \"A JSON library is required for this command. \"..json\n"
"   end\n"
"\n"
"   print(json.encode(value))\n"
"   return true\n"
"end\n"
"\n"
"local function print_entry(var, tbl, is_json)\n"
"   return traverse_varstring(var, tbl, function(t, k)\n"
"      if not t[k] then\n"
"         return nil, \"Unknown entry \" .. k\n"
"      end\n"
"      local val = t[k]\n"
"\n"
"      if not should_skip(var, val) then\n"
"         if is_json then\n"
"            return print_json(val)\n"
"         elseif type(val) == \"string\" then\n"
"            print(val)\n"
"         else\n"
"            persist.write_value(io.stdout, val)\n"
"         end\n"
"      end\n"
"      return true\n"
"   end)\n"
"end\n"
"\n"
"local function infer_type(var)\n"
"   local typ\n"
"   traverse_varstring(var, cfg, function(t, k)\n"
"      if t[k] ~= nil then\n"
"         typ = type(t[k])\n"
"      end\n"
"   end)\n"
"   return typ\n"
"end\n"
"\n"
"local function write_entries(keys, scope, do_unset)\n"
"   if scope == \"project\" and not cfg.config_files.project then\n"
"      return nil, \"Current directory is not part of a project. You may want to run `luarocks init`.\"\n"
"   end\n"
"   \n"
"   local tbl, err = persist.load_config_file_if_basic(cfg.config_files[scope].file, cfg)\n"
"   if not tbl then\n"
"      return nil, err\n"
"   end\n"
"   \n"
"   for var, val in util.sortedpairs(keys) do\n"
"      traverse_varstring(var, tbl, function(t, k)\n"
"         if do_unset then\n"
"            t[k] = nil\n"
"         else\n"
"            local typ = infer_type(var)\n"
"            local v\n"
"            if typ == \"number\" and tonumber(val) then\n"
"               v = tonumber(val)\n"
"            elseif typ == \"boolean\" and val == \"true\" then\n"
"               v = true\n"
"            elseif typ == \"boolean\" and val == \"false\" then\n"
"               v = false\n"
"            else\n"
"               v = val\n"
"            end\n"
"            t[k] = v\n"
"            keys[var] = v\n"
"         end\n"
"         return true\n"
"      end, function(p, k)\n"
"         p[k] = {}\n"
"      end)\n"
"   end\n"
"\n"
"   local ok, err = persist.save_from_table(cfg.config_files[scope].file, tbl)\n"
"   if ok then\n"
"      print(do_unset and \"Removed\" or \"Wrote\")\n"
"      for var, val in util.sortedpairs(keys) do\n"
"         if do_unset then\n"
"            print((\"\\t%s\"):format(var))\n"
"         else\n"
"            print((\"\\t%s = %q\"):format(var, val))\n"
"         end\n"
"      end\n"
"      print(do_unset and \"from\" or \"to\")\n"
"      print(\"\\t\" .. cfg.config_files[scope].file)\n"
"      return true\n"
"   else\n"
"      return nil, err\n"
"   end\n"
"end\n"
"\n"
"local function check_scope(flags)\n"
"   local scope = flags[\"scope\"]\n"
"                 or (flags[\"local\"] and \"user\")\n"
"                 or (flags[\"project-tree\"] and \"project\")\n"
"                 or (cfg.local_by_default and \"user\")\n"
"                 or \"system\"\n"
"   if scope ~= \"system\" and scope ~= \"user\" and scope ~= \"project\" then\n"
"      return nil, \"Valid values for scope are: system, user, project\"\n"
"   end\n"
"\n"
"   return scope\n"
"end\n"
"\n"
"--- Driver function for \"config\" command.\n"
"-- @return boolean: True if succeeded, nil on errors.\n"
"function config_cmd.command(flags, var, val)\n"
"   deps.check_lua(cfg.variables)\n"
"   \n"
"   -- deprecated flags\n"
"   if flags[\"lua-incdir\"] then\n"
"      print(cfg.variables.LUA_INCDIR)\n"
"      return true\n"
"   end\n"
"   if flags[\"lua-libdir\"] then\n"
"      print(cfg.variables.LUA_LIBDIR)\n"
"      return true\n"
"   end\n"
"   if flags[\"lua-ver\"] then\n"
"      print(cfg.lua_version)\n"
"      return true\n"
"   end\n"
"   if flags[\"system-config\"] then\n"
"      return config_file(cfg.config_files.system)\n"
"   end\n"
"   if flags[\"user-config\"] then\n"
"      return config_file(cfg.config_files.user)\n"
"   end\n"
"   if flags[\"rock-trees\"] then\n"
"      for _, tree in ipairs(cfg.rocks_trees) do\n"
"      \tif type(tree) == \"string\" then\n"
"      \t   util.printout(dir.normalize(tree))\n"
"      \telse\n"
"      \t   local name = tree.name and \"\\t\"..tree.name or \"\"\n"
"      \t   util.printout(dir.normalize(tree.root)..name)\n"
"      \tend\n"
"      end\n"
"      return true\n"
"   end\n"
"\n"
"   if var == \"lua_version\" and val then\n"
"      local scope, err = check_scope(flags)\n"
"      if not scope then\n"
"         return nil, err\n"
"      end\n"
"\n"
"      if scope == \"project\" and not cfg.config_files.project then\n"
"         return nil, \"Current directory is not part of a project. You may want to run `luarocks init`.\"\n"
"      end\n"
"\n"
"      local prefix = dir.dir_name(cfg.config_files[scope].file)\n"
"      local ok, err = persist.save_default_lua_version(prefix, val)\n"
"      if not ok then\n"
"         return nil, \"could not set default Lua version: \" .. err\n"
"      end\n"
"      print(\"Lua version will default to \" .. val .. \" in \" .. prefix)\n"
"   end\n"
"   \n"
"   if var == \"lua_dir\" and val then\n"
"      local scope, err = check_scope(flags)\n"
"      if not scope then\n"
"         return nil, err\n"
"      end\n"
"      local keys = {\n"
"         [\"variables.LUA_DIR\"] = cfg.variables.LUA_DIR,\n"
"         [\"variables.LUA_BINDIR\"] = cfg.variables.LUA_BINDIR,\n"
"         [\"variables.LUA_INCDIR\"] = cfg.variables.LUA_INCDIR,\n"
"         [\"variables.LUA_LIBDIR\"] = cfg.variables.LUA_LIBDIR,\n"
"         [\"lua_interpreter\"] = cfg.lua_interpreter,\n"
"      }\n"
"      return write_entries(keys, scope, flags[\"unset\"])\n"
"   end\n"
"\n"
"   if var then\n"
"      if val or flags[\"unset\"] then\n"
"         local scope, err = check_scope(flags)\n"
"         if not scope then\n"
"            return nil, err\n"
"         end\n"
"   \n"
"         return write_entries({ [var] = val }, scope, flags[\"unset\"])\n"
"      else\n"
"         return print_entry(var, cfg, flags[\"json\"])\n"
"      end\n"
"   end\n"
"\n"
"   local cleancfg = cleanup(cfg)\n"
"\n"
"   if flags[\"json\"] then\n"
"      return print_json(cleancfg)\n"
"   else\n"
"      print(persist.save_from_table_to_string(cleancfg))\n"
"      return true\n"
"   end\n"
"end\n"
"\n"
"return config_cmd\n"
""
;
