const char luarocks_cmd_install_lua[] =
"--- Module implementing the LuaRocks \"install\" command.\n"
"-- Installs binary rocks.\n"
"local install = {}\n"
"\n"
"local path = require(\"luarocks.path\")\n"
"local repos = require(\"luarocks.repos\")\n"
"local fetch = require(\"luarocks.fetch\")\n"
"local util = require(\"luarocks.util\")\n"
"local fs = require(\"luarocks.fs\")\n"
"local deps = require(\"luarocks.deps\")\n"
"local writer = require(\"luarocks.manif.writer\")\n"
"local remove = require(\"luarocks.remove\")\n"
"local search = require(\"luarocks.search\")\n"
"local queries = require(\"luarocks.queries\")\n"
"local cfg = require(\"luarocks.core.cfg\")\n"
"local cmd = require(\"luarocks.cmd\")\n"
"local dir = require(\"luarocks.dir\")\n"
"\n"
"install.help_summary = \"Install a rock.\"\n"
"\n"
"install.help_arguments = \"{<rock>|<name> [<version>]}\"\n"
"\n"
"install.help = [[\n"
"Argument may be the name of a rock to be fetched from a repository\n"
"or a filename of a locally available rock.\n"
"\n"
"--keep              Do not remove previously installed versions of the\n"
"                    rock after installing a new one. This behavior can\n"
"                    be made permanent by setting keep_other_versions=true\n"
"                    in the configuration file.\n"
"\n"
"--only-deps         Installs only the dependencies of the rock.\n"
"\n"
"--no-doc            Installs the rock without its documentation.\n"
"\n"
"--verify            Verify signature of the rock being installed.\n"
"                    If rock is being downloaded, LuaRocks will attempt\n"
"                    to download the signature as well. If the rock is\n"
"                    local, the signature file should be in the same\n"
"                    directory.\n"
"                    You need the signer’s public key in your local\n"
"                    keyring for this option to work properly.\n"
"\n"
"]]..util.deps_mode_help()\n"
"\n"
"install.opts = util.opts_table(\"install.opts\", {\n"
"   namespace = \"string\?\",\n"
"   keep = \"boolean\",\n"
"   force = \"boolean\",\n"
"   force_fast = \"boolean\",\n"
"   no_doc = \"boolean\",\n"
"   deps_mode = \"string\",\n"
"   verify = \"boolean\",\n"
"})\n"
"\n"
"--- Install a binary rock.\n"
"-- @param rock_file string: local or remote filename of a rock.\n"
"-- @param opts table: installation options\n"
"-- @return (string, string) or (nil, string, [string]): Name and version of\n"
"-- installed rock if succeeded or nil and an error message followed by an error code.\n"
"function install.install_binary_rock(rock_file, opts)\n"
"   assert(type(rock_file) == \"string\")\n"
"   assert(opts:type() == \"install.opts\")\n"
"\n"
"   local namespace = opts.namespace\n"
"   local deps_mode = opts.deps_mode\n"
"\n"
"   local name, version, arch = path.parse_name(rock_file)\n"
"   if not name then\n"
"      return nil, \"Filename \"..rock_file..\" does not match format 'name-version-revision.arch.rock'.\"\n"
"   end\n"
"   \n"
"   if arch ~= \"all\" and arch ~= cfg.arch then\n"
"      return nil, \"Incompatible architecture \"..arch, \"arch\"\n"
"   end\n"
"   if repos.is_installed(name, version) then\n"
"      repos.delete_version(name, version, opts.deps_mode)\n"
"   end\n"
"   \n"
"   local install_dir = path.install_dir(name, version)\n"
"   \n"
"   local rollback = util.schedule_function(function()\n"
"      fs.delete(install_dir)\n"
"      fs.remove_dir_if_empty(path.versions_dir(name))\n"
"   end)\n"
"\n"
"   local ok, err, errcode = fetch.fetch_and_unpack_rock(rock_file, install_dir, opts.verify)\n"
"   if not ok then return nil, err, errcode end\n"
"\n"
"   local rockspec, err = fetch.load_rockspec(path.rockspec_file(name, version))\n"
"   if err then\n"
"      return nil, \"Failed loading rockspec for installed package: \"..err, errcode\n"
"   end\n"
"\n"
"   if opts.deps_mode == \"none\" then\n"
"      util.warning(\"skipping dependency checks.\")\n"
"   else\n"
"      ok, err, errcode = deps.check_external_deps(rockspec, \"install\")\n"
"      if err then return nil, err, errcode end\n"
"   end\n"
"\n"
"   -- For compatibility with .rock files built with LuaRocks 1\n"
"   if not fs.exists(path.rock_manifest_file(name, version)) then\n"
"      ok, err = writer.make_rock_manifest(name, version)\n"
"      if err then return nil, err end\n"
"   end\n"
"\n"
"   if namespace then\n"
"      ok, err = writer.make_namespace_file(name, version, namespace)\n"
"      if err then return nil, err end\n"
"   end\n"
"\n"
"   if deps_mode ~= \"none\" then\n"
"      ok, err, errcode = deps.fulfill_dependencies(rockspec, \"dependencies\", deps_mode, opts.verify)\n"
"      if err then return nil, err, errcode end\n"
"   end\n"
"\n"
"   ok, err = repos.deploy_files(name, version, repos.should_wrap_bin_scripts(rockspec), deps_mode)\n"
"   if err then return nil, err end\n"
"\n"
"   util.remove_scheduled_function(rollback)\n"
"   rollback = util.schedule_function(function()\n"
"      repos.delete_version(name, version, deps_mode)\n"
"   end)\n"
"\n"
"   ok, err = repos.run_hook(rockspec, \"post_install\")\n"
"   if err then return nil, err end\n"
"\n"
"   util.announce_install(rockspec)\n"
"   util.remove_scheduled_function(rollback)\n"
"   return name, version\n"
"end\n"
"\n"
"--- Installs the dependencies of a binary rock.\n"
"-- @param rock_file string: local or remote filename of a rock.\n"
"-- @param opts table: installation options\n"
"-- @return (string, string) or (nil, string, [string]): Name and version of\n"
"-- the rock whose dependencies were installed if succeeded or nil and an error message \n"
"-- followed by an error code.\n"
"function install.install_binary_rock_deps(rock_file, opts)\n"
"   assert(type(rock_file) == \"string\")\n"
"   assert(opts:type() == \"install.opts\")\n"
"\n"
"   local name, version, arch = path.parse_name(rock_file)\n"
"   if not name then\n"
"      return nil, \"Filename \"..rock_file..\" does not match format 'name-version-revision.arch.rock'.\"\n"
"   end\n"
"   \n"
"   if arch ~= \"all\" and arch ~= cfg.arch then\n"
"      return nil, \"Incompatible architecture \"..arch, \"arch\"\n"
"   end\n"
"\n"
"   local install_dir = path.install_dir(name, version)\n"
"\n"
"   local ok, err, errcode = fetch.fetch_and_unpack_rock(rock_file, install_dir, opts.verify)\n"
"   if not ok then return nil, err, errcode end\n"
"   \n"
"   local rockspec, err = fetch.load_rockspec(path.rockspec_file(name, version))\n"
"   if err then\n"
"      return nil, \"Failed loading rockspec for installed package: \"..err, errcode\n"
"   end\n"
"\n"
"   ok, err, errcode = deps.fulfill_dependencies(rockspec, \"dependencies\", opts.deps_mode, opts.verify)\n"
"   if err then return nil, err, errcode end\n"
"\n"
"   util.printout()\n"
"   util.printout(\"Successfully installed dependencies for \" ..name..\" \"..version)\n"
"\n"
"   return name, version\n"
"end\n"
"\n"
"local function install_rock_file_deps(filename, opts)\n"
"   assert(opts:type() == \"install.opts\")\n"
"\n"
"   local name, version = install.install_binary_rock_deps(filename, opts)\n"
"   if not name then return nil, version end\n"
"\n"
"   writer.check_dependencies(nil, opts.deps_mode)\n"
"   return name, version\n"
"end\n"
"\n"
"local function install_rock_file(filename, opts)\n"
"   assert(type(filename) == \"string\")\n"
"   assert(opts:type() == \"install.opts\")\n"
"\n"
"   local name, version = install.install_binary_rock(filename, opts)\n"
"   if not name then return nil, version end\n"
"\n"
"   if opts.no_doc then\n"
"      local install_dir = path.install_dir(name, version)\n"
"      for _, f in ipairs(fs.list_dir(install_dir)) do\n"
"         local doc_dirs = { \"doc\", \"docs\" }\n"
"         for _, d in ipairs(doc_dirs) do\n"
"            if f == d then\n"
"               fs.delete(dir.path(install_dir, f))\n"
"            end\n"
"         end\n"
"      end\n"
"   end\n"
"\n"
"   if (not opts.keep) and not cfg.keep_other_versions then\n"
"      local ok, err = remove.remove_other_versions(name, version, opts.force, opts.force_fast)\n"
"      if not ok then util.printerr(err) end\n"
"   end\n"
"\n"
"   writer.check_dependencies(nil, opts.deps_mode)\n"
"   return name, version\n"
"end\n"
"\n"
"--- Driver function for the \"install\" command.\n"
"-- @param name string: name of a binary rock. If an URL or pathname\n"
"-- to a binary rock is given, fetches and installs it. If a rockspec or a\n"
"-- source rock is given, forwards the request to the \"build\" command.\n"
"-- If a package name is given, forwards the request to \"search\" and,\n"
"-- if returned a result, installs the matching rock.\n"
"-- @param version string: When passing a package name, a version number\n"
"-- may also be given.\n"
"-- @return boolean or (nil, string, exitcode): True if installation was\n"
"-- successful, nil and an error message otherwise. exitcode is optionally returned.\n"
"function install.command(flags, name, version)\n"
"   if type(name) ~= \"string\" then\n"
"      return nil, \"Argument missing. \"..util.see_help(\"install\")\n"
"   end\n"
"\n"
"   name = util.adjust_name_and_namespace(name, flags)\n"
"\n"
"   local ok, err = fs.check_command_permissions(flags)\n"
"   if not ok then return nil, err, cmd.errorcodes.PERMISSIONDENIED end\n"
"\n"
"   if name:match(\"%.rockspec$\") or name:match(\"%.src%.rock$\") then\n"
"      local build = require(\"luarocks.cmd.build\")\n"
"      return build.command(flags, name)\n"
"   elseif name:match(\"%.rock$\") then\n"
"      local deps_mode = deps.get_deps_mode(flags)\n"
"      local opts = install.opts({\n"
"         namespace = flags[\"namespace\"],\n"
"         keep = not not flags[\"keep\"],\n"
"         force = not not flags[\"force\"],\n"
"         force_fast = not not flags[\"force-fast\"],\n"
"         no_doc = not not flags[\"no-doc\"],\n"
"         deps_mode = deps_mode,\n"
"         verify = not not flags[\"verify\"],\n"
"      })\n"
"      if flags[\"only-deps\"] then\n"
"         return install_rock_file_deps(name, opts)\n"
"      else\n"
"         return install_rock_file(name, opts)\n"
"      end\n"
"   else\n"
"      local url, err = search.find_suitable_rock(queries.new(name:lower(), version), true)\n"
"      if not url then\n"
"         return nil, err\n"
"      end\n"
"      util.printout(\"Installing \"..url)\n"
"      return install.command(flags, url)\n"
"   end\n"
"end\n"
"\n"
"return install\n"
""
;
