const char luarocks_cmd_make_lua[] =
"\n"
"--- Module implementing the LuaRocks \"make\" command.\n"
"-- Builds sources in the current directory, but unlike \"build\",\n"
"-- it does not fetch sources, etc., assuming everything is \n"
"-- available in the current directory.\n"
"local make = {}\n"
"\n"
"local build = require(\"luarocks.build\")\n"
"local fs = require(\"luarocks.fs\")\n"
"local util = require(\"luarocks.util\")\n"
"local cfg = require(\"luarocks.core.cfg\")\n"
"local fetch = require(\"luarocks.fetch\")\n"
"local pack = require(\"luarocks.pack\")\n"
"local remove = require(\"luarocks.remove\")\n"
"local deps = require(\"luarocks.deps\")\n"
"local writer = require(\"luarocks.manif.writer\")\n"
"local cmd = require(\"luarocks.cmd\")\n"
"\n"
"make.help_summary = \"Compile package in current directory using a rockspec.\"\n"
"make.help_arguments = \"[--pack-binary-rock] [<rockspec>]\"\n"
"make.help = [[\n"
"Builds sources in the current directory, but unlike \"build\",\n"
"it does not fetch sources, etc., assuming everything is \n"
"available in the current directory. If no argument is given,\n"
"it looks for a rockspec in the current directory and in \"rockspec/\"\n"
"and \"rockspecs/\" subdirectories, picking the rockspec with newest version\n"
"or without version name. If rockspecs for different rocks are found\n"
"or there are several rockspecs without version, you must specify which to use,\n"
"through the command-line.\n"
"\n"
"This command is useful as a tool for debugging rockspecs. \n"
"To install rocks, you'll normally want to use the \"install\" and\n"
"\"build\" commands. See the help on those for details.\n"
"\n"
"NB: Use `luarocks install` with the `--only-deps` flag if you want to install\n"
"only dependencies of the rockspec (see `luarocks help install`).\n"
"\n"
"--pack-binary-rock  Do not install rock. Instead, produce a .rock file\n"
"                    with the contents of compilation in the current\n"
"                    directory.\n"
"\n"
"--keep              Do not remove previously installed versions of the\n"
"                    rock after installing a new one. This behavior can\n"
"                    be made permanent by setting keep_other_versions=true\n"
"                    in the configuration file.\n"
"\n"
"--branch=<name>     Override the `source.branch` field in the loaded\n"
"                    rockspec. Allows to specify a different branch to \n"
"                    fetch. Particularly for \"dev\" rocks.\n"
"\n"
"--verify            Verify signature of the rockspec or src.rock being\n"
"                    built. If the rockspec or src.rock is being downloaded,\n"
"                    LuaRocks will attempt to download the signature as well.\n"
"                    Otherwise, the signature file should be already\n"
"                    available locally in the same directory.\n"
"                    You need the signer’s public key in your local\n"
"                    keyring for this option to work properly.\n"
"\n"
"--sign              To be used with --pack-binary-rock. Also produce\n"
"                    a signature file for the generated .rock file.\n"
"\n"
"--chdir=<path>      Specify a source directory of the rock.\n"
"\n"
"]]\n"
"\n"
"--- Driver function for \"make\" command.\n"
"-- @param name string: A local rockspec.\n"
"-- @return boolean or (nil, string, exitcode): True if build was successful; nil and an\n"
"-- error message otherwise. exitcode is optionally returned.\n"
"function make.command(flags, rockspec_filename)\n"
"   assert(type(rockspec_filename) == \"string\" or not rockspec_filename)\n"
"   \n"
"   if flags[\"chdir\"] then\n"
"      local ok, err = fs.change_dir(flags[\"chdir\"])\n"
"      if not ok then\n"
"         return nil, err\n"
"      end\n"
"   end\n"
"   if not rockspec_filename then\n"
"      local err\n"
"      rockspec_filename, err = util.get_default_rockspec()\n"
"      if not rockspec_filename then\n"
"         return nil, err\n"
"      end\n"
"   end\n"
"   if not rockspec_filename:match(\"rockspec$\") then\n"
"      return nil, \"Invalid argument: 'make' takes a rockspec as a parameter. \"..util.see_help(\"make\")\n"
"   end\n"
"   \n"
"   local rockspec, err, errcode = fetch.load_rockspec(rockspec_filename)\n"
"   if not rockspec then\n"
"      return nil, err\n"
"   end\n"
"\n"
"   local name = util.adjust_name_and_namespace(rockspec.name, flags)\n"
"\n"
"   local opts = build.opts({\n"
"      need_to_fetch = false,\n"
"      minimal_mode = true,\n"
"      deps_mode = deps.get_deps_mode(flags),\n"
"      build_only_deps = false,\n"
"      namespace = flags[\"namespace\"],\n"
"      branch = not not flags[\"branch\"],\n"
"      verify = not not flags[\"verify\"],\n"
"   })\n"
"\n"
"   if flags[\"sign\"] and not flags[\"pack-binary-rock\"] then\n"
"      return nil, \"In the make command, --sign is meant to be used only with --pack-binary-rock\"\n"
"   end\n"
"\n"
"   if flags[\"pack-binary-rock\"] then\n"
"      return pack.pack_binary_rock(name, rockspec.version, flags[\"sign\"], function()\n"
"         return build.build_rockspec(rockspec, opts)\n"
"      end)\n"
"   else\n"
"      local ok, err = fs.check_command_permissions(flags)\n"
"      if not ok then return nil, err, cmd.errorcodes.PERMISSIONDENIED end\n"
"      ok, err = build.build_rockspec(rockspec, opts)\n"
"      if not ok then return nil, err end\n"
"      local name, version = ok, err\n"
"\n"
"      if (not flags[\"keep\"]) and not cfg.keep_other_versions then\n"
"         local ok, err = remove.remove_other_versions(name, version, flags[\"force\"], flags[\"force-fast\"])\n"
"         if not ok then util.printerr(err) end\n"
"      end\n"
"\n"
"      writer.check_dependencies(nil, deps.get_deps_mode(flags))\n"
"      return name, version\n"
"   end\n"
"end\n"
"\n"
"return make\n"
""
;
