const char luarocks_cmd_unpack_lua[] =
"\n"
"--- Module implementing the LuaRocks \"unpack\" command.\n"
"-- Unpack the contents of a rock.\n"
"local unpack = {}\n"
"\n"
"local fetch = require(\"luarocks.fetch\")\n"
"local fs = require(\"luarocks.fs\")\n"
"local util = require(\"luarocks.util\")\n"
"local build = require(\"luarocks.build\")\n"
"local dir = require(\"luarocks.dir\")\n"
"local search = require(\"luarocks.search\")\n"
"\n"
"unpack.help_summary = \"Unpack the contents of a rock.\"\n"
"unpack.help_arguments = \"[--force] {<rock>|<name> [<version>]}\"\n"
"unpack.help = [[\n"
"Unpacks the contents of a rock in a newly created directory.\n"
"Argument may be a rock file, or the name of a rock in a rocks server.\n"
"In the latter case, the app version may be given as a second argument.\n"
"\n"
"--force   Unpack files even if the output directory already exists.\n"
"]]\n"
"\n"
"--- Load a rockspec file to the given directory, fetches the source\n"
"-- files specified in the rockspec, and unpack them inside the directory.\n"
"-- @param rockspec_file string: The URL for a rockspec file.\n"
"-- @param dir_name string: The directory where to store and unpack files.\n"
"-- @return table or (nil, string): the loaded rockspec table or \n"
"-- nil and an error message.\n"
"local function unpack_rockspec(rockspec_file, dir_name)\n"
"   assert(type(rockspec_file) == \"string\")\n"
"   assert(type(dir_name) == \"string\")\n"
"\n"
"   local rockspec, err = fetch.load_rockspec(rockspec_file)\n"
"   if not rockspec then\n"
"      return nil, \"Failed loading rockspec \"..rockspec_file..\": \"..err\n"
"   end\n"
"   local ok, err = fs.change_dir(dir_name)\n"
"   if not ok then return nil, err end\n"
"   local ok, sources_dir = fetch.fetch_sources(rockspec, true, \".\")\n"
"   if not ok then\n"
"      return nil, sources_dir\n"
"   end\n"
"   ok, err = fs.change_dir(sources_dir)\n"
"   if not ok then return nil, err end\n"
"   ok, err = build.apply_patches(rockspec)\n"
"   fs.pop_dir()\n"
"   if not ok then return nil, err end\n"
"   return rockspec\n"
"end\n"
"\n"
"--- Load a .rock file to the given directory and unpack it inside it.\n"
"-- @param rock_file string: The URL for a .rock file.\n"
"-- @param dir_name string: The directory where to unpack.\n"
"-- @param kind string: the kind of rock file, as in the second-level\n"
"-- extension in the rock filename (eg. \"src\", \"all\", \"linux-x86\")\n"
"-- @return table or (nil, string): the loaded rockspec table or \n"
"-- nil and an error message.\n"
"local function unpack_rock(rock_file, dir_name, kind)\n"
"   assert(type(rock_file) == \"string\")\n"
"   assert(type(dir_name) == \"string\")\n"
"\n"
"   local ok, err, errcode = fetch.fetch_and_unpack_rock(rock_file, dir_name)\n"
"   if not ok then\n"
"      return nil, err, errcode\n"
"   end\n"
"   ok, err = fs.change_dir(dir_name)\n"
"   if not ok then return nil, err end\n"
"   local rockspec_file = dir_name..\".rockspec\"\n"
"   local rockspec, err = fetch.load_rockspec(rockspec_file)\n"
"   if not rockspec then\n"
"      return nil, \"Failed loading rockspec \"..rockspec_file..\": \"..err\n"
"   end\n"
"   if kind == \"src\" then\n"
"      if rockspec.source.file then\n"
"         local ok, err = fs.unpack_archive(rockspec.source.file)\n"
"         if not ok then\n"
"            return nil, err\n"
"         end\n"
"         ok, err = fs.change_dir(rockspec.source.dir)\n"
"         if not ok then return nil, err end\n"
"         ok, err = build.apply_patches(rockspec)\n"
"         fs.pop_dir()\n"
"         if not ok then return nil, err end\n"
"      end\n"
"   end\n"
"   return rockspec\n"
"end\n"
"\n"
"--- Create a directory and perform the necessary actions so that\n"
"-- the sources for the rock and its rockspec are unpacked inside it,\n"
"-- laid out properly so that the 'make' command is able to build the module.\n"
"-- @param file string: A rockspec or .rock URL.\n"
"-- @return boolean or (nil, string): true if successful or nil followed\n"
"-- by an error message.\n"
"local function run_unpacker(file, force)\n"
"   assert(type(file) == \"string\")\n"
"   \n"
"   local base_name = dir.base_name(file)\n"
"   local dir_name, kind, extension = base_name:match(\"(.*)%.([^.]+)%.(rock)$\")\n"
"   if not extension then\n"
"      dir_name, extension = base_name:match(\"(.*)%.(rockspec)$\")\n"
"      kind = \"rockspec\"\n"
"   end\n"
"   if not extension then\n"
"      return nil, file..\" does not seem to be a valid filename.\"\n"
"   end\n"
"\n"
"   local exists = fs.exists(dir_name)\n"
"   if exists and not force then\n"
"      return nil, \"Directory \"..dir_name..\" already exists.\"\n"
"   end\n"
"   if not exists then\n"
"      local ok, err = fs.make_dir(dir_name)\n"
"      if not ok then return nil, err end\n"
"   end\n"
"   local rollback = util.schedule_function(fs.delete, fs.absolute_name(dir_name))\n"
"\n"
"   local rockspec, err\n"
"   if extension == \"rock\" then\n"
"      rockspec, err = unpack_rock(file, dir_name, kind)\n"
"   elseif extension == \"rockspec\" then\n"
"      rockspec, err = unpack_rockspec(file, dir_name)\n"
"   end\n"
"   if not rockspec then\n"
"      return nil, err\n"
"   end\n"
"   if kind == \"src\" or kind == \"rockspec\" then\n"
"      if rockspec.source.dir ~= \".\" then\n"
"         local ok = fs.copy(rockspec.local_abs_filename, rockspec.source.dir, \"read\")\n"
"         if not ok then\n"
"            return nil, \"Failed copying unpacked rockspec into unpacked source directory.\"\n"
"         end\n"
"      end\n"
"      util.printout()   \n"
"      util.printout(\"Done. You may now enter directory \")\n"
"      util.printout(dir.path(dir_name, rockspec.source.dir))\n"
"      util.printout(\"and type 'luarocks make' to build.\")\n"
"   end\n"
"   util.remove_scheduled_function(rollback)\n"
"   return true\n"
"end\n"
"\n"
"--- Driver function for the \"unpack\" command.\n"
"-- @param ns_name string: may be a rock filename, for unpacking a \n"
"-- rock file or the name of a rock to be fetched and unpacked.\n"
"-- @param version string or nil: if the name of a package is given, a\n"
"-- version may also be passed.\n"
"-- @return boolean or (nil, string): true if successful or nil followed\n"
"-- by an error message.\n"
"function unpack.command(flags, ns_name, version)\n"
"   assert(type(version) == \"string\" or not version)\n"
"   if type(ns_name) ~= \"string\" then\n"
"      return nil, \"Argument missing. \"..util.see_help(\"unpack\")\n"
"   end\n"
"\n"
"   ns_name = util.adjust_name_and_namespace(ns_name, flags)\n"
"\n"
"   local url, err\n"
"   if ns_name:match(\".*%.rock\") or ns_name:match(\".*%.rockspec\") then\n"
"      url = ns_name\n"
"   else\n"
"      url, err = search.find_src_or_rockspec(ns_name, version, true)\n"
"      if not url then\n"
"         return nil, err\n"
"      end\n"
"   end\n"
"\n"
"   return run_unpacker(url, flags[\"force\"])\n"
"end\n"
"\n"
"return unpack\n"
""
;
