const char luarocks_core_util_lua[] =
"\n"
"local util = {}\n"
"\n"
"local require = nil\n"
"--------------------------------------------------------------------------------\n"
"\n"
"--- Run a process and read a its output.\n"
"-- Equivalent to io.popen(cmd):read(\"*l\"), except that it \n"
"-- closes the fd right away.\n"
"-- @param cmd string: The command to execute\n"
"-- @param spec string: \"*l\" by default, to read a single line.\n"
"-- May be used to read more, passing, for instance, \"*a\".\n"
"-- @return string: the output of the program.\n"
"function util.popen_read(cmd, spec)\n"
"   local tmpfile = os.tmpname()\n"
"   os.execute(cmd .. \" > \" .. tmpfile)\n"
"   local fd = io.open(tmpfile, \"rb\")\n"
"   if not fd then\n"
"      os.remove(tmpfile)\n"
"      return \"\"\n"
"   end\n"
"   local out = fd:read(spec or \"*l\")\n"
"   fd:close()\n"
"   os.remove(tmpfile)\n"
"   return out or \"\"\n"
"end\n"
"\n"
"--- Create a new shallow copy of a table: a new table with\n"
"-- the same keys and values. Keys point to the same objects as\n"
"-- the original table (ie, does not copy recursively).\n"
"-- @param tbl table: the input table\n"
"-- @return table: a new table with the same contents.\n"
"function util.make_shallow_copy(tbl)\n"
"   local copy = {}\n"
"   for k,v in pairs(tbl) do\n"
"      copy[k] = v\n"
"   end\n"
"   return copy\n"
"end\n"
"\n"
"---\n"
"-- Formats tables with cycles recursively to any depth.\n"
"-- References to other tables are shown as values.\n"
"-- Self references are indicated.\n"
"-- The string returned is \"Lua code\", which can be processed\n"
"-- (in the case in which indent is composed by spaces or \"--\").\n"
"-- Userdata and function keys and values are shown as strings,\n"
"-- which logically are exactly not equivalent to the original code.\n"
"-- This routine can serve for pretty formating tables with\n"
"-- proper indentations, apart from printing them:\n"
"-- io.write(table.show(t, \"t\"))   -- a typical use\n"
"-- Written by Julio Manuel Fernandez-Diaz,\n"
"-- Heavily based on \"Saving tables with cycles\", PIL2, p. 113.\n"
"-- @param t table: is the table.\n"
"-- @param tname string: is the name of the table (optional)\n"
"-- @param top_indent string: is a first indentation (optional).\n"
"-- @return string: the pretty-printed table\n"
"function util.show_table(t, tname, top_indent)\n"
"   local cart     -- a container\n"
"   local autoref  -- for self references\n"
"\n"
"   local function is_empty_table(tbl) return next(tbl) == nil end\n"
"   \n"
"   local function basic_serialize(o)\n"
"      local so = tostring(o)\n"
"      if type(o) == \"function\" then\n"
"         local info = debug.getinfo(o, \"S\")\n"
"         -- info.name is nil because o is not a calling level\n"
"         if info.what == \"C\" then\n"
"            return (\"%q\"):format(so .. \", C function\")\n"
"         else \n"
"            -- the information is defined through lines\n"
"            return (\"%q\"):format(so .. \", defined in (\" .. info.linedefined .. \"-\" .. info.lastlinedefined .. \")\" .. info.source)\n"
"         end\n"
"      elseif type(o) == \"number\" then\n"
"         return so\n"
"      else\n"
"         return (\"%q\"):format(so)\n"
"      end\n"
"   end\n"
"   \n"
"   local function add_to_cart(value, name, indent, saved, field)\n"
"      indent = indent or \"\"\n"
"      saved = saved or {}\n"
"      field = field or name\n"
"      \n"
"      cart = cart .. indent .. field\n"
"      \n"
"      if type(value) ~= \"table\" then\n"
"         cart = cart .. \" = \" .. basic_serialize(value) .. \";\\n\"\n"
"      else\n"
"         if saved[value] then\n"
"            cart = cart .. \" = {}; -- \" .. saved[value] .. \" (self reference)\\n\"\n"
"            autoref = autoref ..  name .. \" = \" .. saved[value] .. \";\\n\"\n"
"         else\n"
"            saved[value] = name\n"
"            if is_empty_table(value) then\n"
"               cart = cart .. \" = {};\\n\"\n"
"            else\n"
"               cart = cart .. \" = {\\n\"\n"
"               for k, v in pairs(value) do\n"
"                  k = basic_serialize(k)\n"
"                  local fname = (\"%s[%s]\"):format(name, k)\n"
"                  field = (\"[%s]\"):format(k)\n"
"                  -- three spaces between levels\n"
"                  add_to_cart(v, fname, indent .. \"   \", saved, field)\n"
"               end\n"
"               cart = cart .. indent .. \"};\\n\"\n"
"            end\n"
"         end\n"
"      end\n"
"   end\n"
"   \n"
"   tname = tname or \"__unnamed__\"\n"
"   if type(t) ~= \"table\" then\n"
"      return tname .. \" = \" .. basic_serialize(t)\n"
"   end\n"
"   cart, autoref = \"\", \"\"\n"
"   add_to_cart(t, tname, top_indent)\n"
"   return cart .. autoref\n"
"end\n"
"\n"
"--- Merges contents of src on top of dst's contents.\n"
"-- @param dst Destination table, which will receive src's contents.\n"
"-- @param src Table which provides new contents to dst.\n"
"function util.deep_merge(dst, src)\n"
"   for k, v in pairs(src) do\n"
"      if type(v) == \"table\" then\n"
"         if not dst[k] then\n"
"            dst[k] = {}\n"
"         end\n"
"         if type(dst[k]) == \"table\" then\n"
"            util.deep_merge(dst[k], v)\n"
"         else\n"
"            dst[k] = v\n"
"         end\n"
"      else\n"
"         dst[k] = v\n"
"      end\n"
"   end\n"
"end\n"
"\n"
"--- Merges contents of src below those of dst's contents.\n"
"-- @param dst Destination table, which will receive src's contents.\n"
"-- @param src Table which provides new contents to dst.\n"
"function util.deep_merge_under(dst, src)\n"
"   for k, v in pairs(src) do\n"
"      if type(v) == \"table\" then\n"
"         if not dst[k] then\n"
"            dst[k] = {}\n"
"         end\n"
"         if type(dst[k]) == \"table\" then\n"
"            util.deep_merge_under(dst[k], v)\n"
"         end\n"
"      elseif dst[k] == nil then\n"
"         dst[k] = v\n"
"      end\n"
"   end\n"
"end\n"
"\n"
"--- Clean up a path-style string ($PATH, $LUA_PATH/package.path, etc.),\n"
"-- removing repeated entries and making sure only the relevant\n"
"-- Lua version is used.\n"
"-- Example: given (\"a;b;c;a;b;d\", \";\"), returns \"a;b;c;d\".\n"
"-- @param list string: A path string (from $PATH or package.path)\n"
"-- @param sep string: The separator\n"
"-- @param lua_version (optional) string: The Lua version to use.\n"
"function util.cleanup_path(list, sep, lua_version)\n"
"   assert(type(list) == \"string\")\n"
"   assert(type(sep) == \"string\")\n"
"   local parts = util.split_string(list, sep)\n"
"   local final, entries = {}, {}\n"
"   for _, part in ipairs(parts) do\n"
"      part = part:gsub(\"//\", \"/\")\n"
"      if lua_version then\n"
"         part = part:gsub(\"/lua/([%d.]+)/\", function(part_version)\n"
"            if part_version:sub(1, #lua_version) ~= lua_version then            \n"
"               return \"/lua/\"..lua_version..\"/\"\n"
"            end\n"
"         end)\n"
"      end\n"
"      if not entries[part] then\n"
"         table.insert(final, part)\n"
"         entries[part] = true\n"
"      end\n"
"   end\n"
"   return table.concat(final, sep)\n"
"end\n"
"\n"
"-- from http://lua-users.org/wiki/SplitJoin\n"
"-- by Philippe Lhoste\n"
"function util.split_string(str, delim, maxNb)\n"
"   -- Eliminate bad cases...\n"
"   if string.find(str, delim) == nil then\n"
"      return { str }\n"
"   end\n"
"   if maxNb == nil or maxNb < 1 then\n"
"      maxNb = 0    -- No limit\n"
"   end\n"
"   local result = {}\n"
"   local pat = \"(.-)\" .. delim .. \"()\"\n"
"   local nb = 0\n"
"   local lastPos\n"
"   for part, pos in string.gmatch(str, pat) do\n"
"      nb = nb + 1\n"
"      result[nb] = part\n"
"      lastPos = pos\n"
"      if nb == maxNb then break end\n"
"   end\n"
"   -- Handle the last field\n"
"   if nb ~= maxNb then\n"
"      result[nb + 1] = string.sub(str, lastPos)\n"
"   end\n"
"   return result\n"
"end\n"
"\n"
"--- Return an array of keys of a table.\n"
"-- @param tbl table: The input table.\n"
"-- @return table: The array of keys.\n"
"function util.keys(tbl)\n"
"   local ks = {}\n"
"   for k,_ in pairs(tbl) do\n"
"      table.insert(ks, k)\n"
"   end\n"
"   return ks\n"
"end\n"
"\n"
"--- Print a line to standard error\n"
"function util.printerr(...)\n"
"   io.stderr:write(table.concat({...},\"\\t\"))\n"
"   io.stderr:write(\"\\n\")\n"
"end\n"
"\n"
"--- Display a warning message.\n"
"-- @param msg string: the warning message\n"
"function util.warning(msg)\n"
"   util.printerr(\"Warning: \"..msg)\n"
"end\n"
"\n"
"--- Simple sort function used as a default for util.sortedpairs.\n"
"local function default_sort(a, b)\n"
"   local ta = type(a)\n"
"   local tb = type(b)\n"
"   if ta == \"number\" and tb == \"number\" then\n"
"      return a < b\n"
"   elseif ta == \"number\" then\n"
"      return true\n"
"   elseif tb == \"number\" then\n"
"      return false\n"
"   else\n"
"      return tostring(a) < tostring(b)\n"
"   end\n"
"end\n"
"\n"
"--- A table iterator generator that returns elements sorted by key,\n"
"-- to be used in \"for\" loops.\n"
"-- @param tbl table: The table to be iterated.\n"
"-- @param sort_function function or table or nil: An optional comparison function\n"
"-- to be used by table.sort when sorting keys, or an array listing an explicit order\n"
"-- for keys. If a value itself is an array, it is taken so that the first element\n"
"-- is a string representing the field name, and the second element is a priority table\n"
"-- for that key, which is returned by the iterator as the third value after the key\n"
"-- and the value.\n"
"-- @return function: the iterator function.\n"
"function util.sortedpairs(tbl, sort_function)\n"
"   sort_function = sort_function or default_sort\n"
"   local keys = util.keys(tbl)\n"
"   local sub_orders = {}\n"
"\n"
"   if type(sort_function) == \"function\" then\n"
"      table.sort(keys, sort_function)\n"
"   else\n"
"      local order = sort_function\n"
"      local ordered_keys = {}\n"
"      local all_keys = keys\n"
"      keys = {}\n"
"\n"
"      for _, order_entry in ipairs(order) do\n"
"         local key, sub_order\n"
"         if type(order_entry) == \"table\" then\n"
"            key = order_entry[1]\n"
"            sub_order = order_entry[2]\n"
"         else\n"
"            key = order_entry\n"
"         end\n"
"\n"
"         if tbl[key] then\n"
"            ordered_keys[key] = true\n"
"            sub_orders[key] = sub_order\n"
"            table.insert(keys, key)\n"
"         end\n"
"      end\n"
"\n"
"      table.sort(all_keys, default_sort)\n"
"      for _, key in ipairs(all_keys) do\n"
"         if not ordered_keys[key] then\n"
"            table.insert(keys, key)\n"
"         end\n"
"      end\n"
"   end\n"
"\n"
"   local i = 1\n"
"   return function()\n"
"      local key = keys[i]\n"
"      i = i + 1\n"
"      return key, tbl[key], sub_orders[key]\n"
"   end\n"
"end\n"
"\n"
"return util\n"
"\n"
""
;
