const char luarocks_deps_lua[] =
"\n"
"--- High-level dependency related functions.\n"
"local deps = {}\n"
"\n"
"local cfg = require(\"luarocks.core.cfg\")\n"
"local manif = require(\"luarocks.manif\")\n"
"local path = require(\"luarocks.path\")\n"
"local dir = require(\"luarocks.dir\")\n"
"local fun = require(\"luarocks.fun\")\n"
"local util = require(\"luarocks.util\")\n"
"local vers = require(\"luarocks.core.vers\")\n"
"local queries = require(\"luarocks.queries\")\n"
"local builtin = require(\"luarocks.build.builtin\")\n"
"\n"
"--- Attempt to match a dependency to an installed rock.\n"
"-- @param dep table: A dependency parsed in table format.\n"
"-- @param blacklist table: Versions that can't be accepted. Table where keys\n"
"-- are program versions and values are 'true'.\n"
"-- @param rocks_provided table: A table of auto-provided dependencies.\n"
"-- by this Lua implementation for the given dependency.\n"
"-- @return string or nil: latest installed version of the rock matching the dependency\n"
"-- or nil if it could not be matched.\n"
"local function match_dep(dep, blacklist, deps_mode, rocks_provided)\n"
"   assert(type(dep) == \"table\")\n"
"   assert(type(rocks_provided) == \"table\")\n"
"  \n"
"   local versions, locations\n"
"   local provided = rocks_provided[dep.name]\n"
"   if provided then\n"
"      -- Provided rocks have higher priority than manifest's rocks.\n"
"      versions, locations = { provided }, {}\n"
"   else\n"
"      versions, locations = manif.get_versions(dep, deps_mode)\n"
"   end\n"
"\n"
"   local latest_version\n"
"   local latest_vstring\n"
"   for _, vstring in ipairs(versions) do\n"
"      if not blacklist or not blacklist[vstring] then\n"
"         local version = vers.parse_version(vstring)\n"
"         if vers.match_constraints(version, dep.constraints) then\n"
"            if not latest_version or version > latest_version then\n"
"               latest_version = version\n"
"               latest_vstring = vstring\n"
"            end\n"
"         end\n"
"      end\n"
"   end\n"
"   return latest_vstring, locations[latest_vstring]\n"
"end\n"
"\n"
"--- Attempt to match dependencies of a rockspec to installed rocks.\n"
"-- @param dependencies table: The table of dependencies.\n"
"-- @param rocks_provided table: The table of auto-provided dependencies.\n"
"-- @param blacklist table or nil: Program versions to not use as valid matches.\n"
"-- Table where keys are program names and values are tables where keys\n"
"-- are program versions and values are 'true'.\n"
"-- @return table, table, table: A table where keys are dependencies parsed\n"
"-- in table format and values are tables containing fields 'name' and\n"
"-- version' representing matches; a table of missing dependencies\n"
"-- parsed as tables; and a table of \"no-upgrade\" missing dependencies\n"
"-- (to be used in plugin modules so that a plugin does not force upgrade of\n"
"-- its parent application).\n"
"function deps.match_deps(dependencies, rocks_provided, blacklist, deps_mode)\n"
"   assert(type(blacklist) == \"table\" or not blacklist)\n"
"   local matched, missing, no_upgrade = {}, {}, {}\n"
"   \n"
"   for _, dep in ipairs(dependencies) do\n"
"      local found = match_dep(dep, blacklist and blacklist[dep.name] or nil, deps_mode, rocks_provided)\n"
"      if found then\n"
"         if not rocks_provided[dep.name] then\n"
"            matched[dep] = {name = dep.name, version = found}\n"
"         end\n"
"      else\n"
"         if dep.constraints[1] and dep.constraints[1].no_upgrade then\n"
"            no_upgrade[dep.name] = dep\n"
"         else\n"
"            missing[dep.name] = dep\n"
"         end\n"
"      end\n"
"   end\n"
"   return matched, missing, no_upgrade\n"
"end\n"
"\n"
"--- Return a set of values of a table.\n"
"-- @param tbl table: The input table.\n"
"-- @return table: The array of keys.\n"
"local function values_set(tbl)\n"
"   local set = {}\n"
"   for _, v in pairs(tbl) do\n"
"      set[v] = true\n"
"   end\n"
"   return set\n"
"end\n"
"\n"
"local function rock_status(name, deps_mode, rocks_provided)\n"
"   local installed = match_dep(queries.new(name), nil, deps_mode, rocks_provided)\n"
"   local installation_type = rocks_provided[name] and \"provided by VM\" or \"installed\"\n"
"   return installed and installed..\" \"..installation_type or \"not installed\"\n"
"end\n"
"\n"
"--- Check depenendencies of a package and report any missing ones.\n"
"-- @param name string: package name.\n"
"-- @param version string: package version.\n"
"-- @param dependencies table: array of dependencies.\n"
"-- @param deps_mode string: Which trees to check dependencies for:\n"
"-- @param rocks_provided table: A table of auto-dependencies provided \n"
"-- by this Lua implementation for the given dependency.\n"
"-- \"one\" for the current default tree, \"all\" for all trees,\n"
"-- \"order\" for all trees with priority >= the current default, \"none\" for no trees.\n"
"function deps.report_missing_dependencies(name, version, dependencies, deps_mode, rocks_provided)\n"
"   assert(type(name) == \"string\")\n"
"   assert(type(version) == \"string\")\n"
"   assert(type(dependencies) == \"table\")\n"
"   assert(type(deps_mode) == \"string\")\n"
"   assert(type(rocks_provided) == \"table\")\n"
"\n"
"   local first_missing_dep = true\n"
"\n"
"   for _, dep in ipairs(dependencies) do\n"
"      if not match_dep(dep, nil, deps_mode, rocks_provided) then\n"
"         if first_missing_dep then\n"
"            util.printout((\"Missing dependencies for %s %s:\"):format(name, version))\n"
"            first_missing_dep = false\n"
"         end\n"
"\n"
"         util.printout((\"   %s (%s)\"):format(tostring(dep), rock_status(dep.name, deps_mode, rocks_provided)))\n"
"      end\n"
"   end\n"
"end\n"
"\n"
"function deps.fulfill_dependency(dep, deps_mode, name, version, rocks_provided, verify)\n"
"   assert(dep:type() == \"query\")\n"
"   assert(type(deps_mode) == \"string\" or deps_mode == nil)\n"
"   assert(type(name) == \"string\" or name == nil)\n"
"   assert(type(version) == \"string\" or version == nil)\n"
"   assert(type(rocks_provided) == \"table\" or rocks_provided == nil)\n"
"   assert(type(verify) == \"boolean\" or verify == nil)\n"
"   deps_mode = deps_mode or \"all\"\n"
"   rocks_provided = rocks_provided or {}\n"
"\n"
"   local found, where = match_dep(dep, nil, deps_mode, rocks_provided)\n"
"   if found then\n"
"      return true, found, where\n"
"   end\n"
"\n"
"   local search = require(\"luarocks.search\")\n"
"   local install = require(\"luarocks.cmd.install\")\n"
"\n"
"   if name and version then\n"
"      util.printout((\"%s %s depends on %s (%s)\"):format(\n"
"         name, version, tostring(dep), rock_status(dep.name, deps_mode, rocks_provided)))\n"
"   else\n"
"      util.printout((\"Fulfilling dependency on %s (%s)\"):format(\n"
"         tostring(dep), rock_status(dep.name, deps_mode, rocks_provided)))\n"
"   end\n"
"   \n"
"   if dep.constraints[1] and dep.constraints[1].no_upgrade then\n"
"      util.printerr(\"This version of \"..name..\" is designed for use with\")\n"
"      util.printerr(tostring(dep)..\", but is configured to avoid upgrading it\")\n"
"      util.printerr(\"automatically. Please upgrade \"..dep.name..\" with\")\n"
"      util.printerr(\"   luarocks install \"..dep.name)\n"
"      util.printerr(\"or choose an older version of \"..name..\" with\")\n"
"      util.printerr(\"   luarocks search \"..name)\n"
"      return nil, \"Failed matching dependencies\"\n"
"   end\n"
"\n"
"   local url, search_err = search.find_suitable_rock(dep, true)\n"
"   if not url then\n"
"      return nil, \"Could not satisfy dependency \"..tostring(dep)..\": \"..search_err\n"
"   end\n"
"   util.printout(\"Installing \"..url)\n"
"   local install_flags = {\n"
"      deps_mode = deps_mode,\n"
"      namespace = dep.namespace,\n"
"      verify = verify,\n"
"   }\n"
"   local ok, install_err, errcode = install.command(install_flags, url)\n"
"   if not ok then\n"
"      return nil, \"Failed installing dependency: \"..url..\" - \"..install_err, errcode\n"
"   end\n"
"\n"
"   found, where = match_dep(dep, nil, deps_mode, rocks_provided)\n"
"   assert(found)\n"
"   return true, found, where\n"
"end\n"
"\n"
"--- Check dependencies of a rock and attempt to install any missing ones.\n"
"-- Packages are installed using the LuaRocks \"install\" command.\n"
"-- Aborts the program if a dependency could not be fulfilled.\n"
"-- @param rockspec table: A rockspec in table format.\n"
"-- @param depskey string: Rockspec key to fetch to get dependency table.\n"
"-- @param deps_mode string\n"
"-- @param verify boolean\n"
"-- @return boolean or (nil, string, [string]): True if no errors occurred, or\n"
"-- nil and an error message if any test failed, followed by an optional\n"
"-- error code.\n"
"function deps.fulfill_dependencies(rockspec, depskey, deps_mode, verify)\n"
"   assert(type(rockspec) == \"table\")\n"
"   assert(type(depskey) == \"string\")\n"
"   assert(type(deps_mode) == \"string\")\n"
"   assert(type(verify) == \"boolean\" or verify == nil)\n"
"\n"
"   if rockspec.supported_platforms and next(rockspec.supported_platforms) then\n"
"      local all_negative = true\n"
"      local supported = false\n"
"      for _, plat in pairs(rockspec.supported_platforms) do\n"
"         local neg\n"
"         neg, plat = plat:match(\"^(!\?)(.*)\")\n"
"         if neg == \"!\" then\n"
"            if cfg.is_platform(plat) then\n"
"               return nil, \"This rockspec for \"..rockspec.package..\" does not support \"..plat..\" platforms.\"\n"
"            end\n"
"         else\n"
"            all_negative = false\n"
"            if cfg.is_platform(plat) then\n"
"               supported = true\n"
"               break\n"
"            end\n"
"         end\n"
"      end\n"
"      if supported == false and not all_negative then\n"
"         local plats = cfg.print_platforms()\n"
"         return nil, \"This rockspec for \"..rockspec.package..\" does not support \"..plats..\" platforms.\"\n"
"      end\n"
"   end\n"
"\n"
"   deps.report_missing_dependencies(rockspec.name, rockspec.version, rockspec[depskey], deps_mode, rockspec.rocks_provided)\n"
"\n"
"   util.printout()\n"
"   for _, dep in ipairs(rockspec[depskey]) do\n"
"      local ok, err = deps.fulfill_dependency(dep, deps_mode, rockspec.name, rockspec.version, rockspec.rocks_provided, verify)\n"
"      if not ok then\n"
"         return nil, err\n"
"      end\n"
"   end\n"
"\n"
"   return true\n"
"end\n"
"\n"
"--- If filename matches a pattern, return the capture.\n"
"-- For example, given \"libfoo.so\" and \"lib\?.so\" is a pattern,\n"
"-- returns \"foo\" (which can then be used to build names\n"
"-- based on other patterns.\n"
"-- @param file string: a filename\n"
"-- @param pattern string: a pattern, where \? is to be matched by the filename.\n"
"-- @return string The pattern, if found, or nil.\n"
"local function deconstruct_pattern(file, pattern)\n"
"   local depattern = \"^\"..(pattern:gsub(\"%.\", \"%%.\"):gsub(\"%*\", \".*\"):gsub(\"\?\", \"(.*)\"))..\"$\"\n"
"   return (file:match(depattern))\n"
"end\n"
"\n"
"--- Construct all possible patterns for a name and add to the files array.\n"
"-- Run through the patterns array replacing all occurrences of \"\?\"\n"
"-- with the given file name and store them in the files array.\n"
"-- @param file string A raw name (e.g. \"foo\")\n"
"-- @param array of string An array of patterns with \"\?\" as the wildcard\n"
"-- (e.g. {\"\?.so\", \"lib\?.so\"})\n"
"-- @param files The array of constructed names\n"
"local function add_all_patterns(file, patterns, files)\n"
"   for _, pattern in ipairs(patterns) do\n"
"      table.insert(files, (pattern:gsub(\"\?\", file)))\n"
"   end\n"
"end\n"
"\n"
"local function get_external_deps_dirs(mode)\n"
"   local patterns = cfg.external_deps_patterns\n"
"   local subdirs = cfg.external_deps_subdirs\n"
"   if mode == \"install\" then\n"
"      patterns = cfg.runtime_external_deps_patterns\n"
"      subdirs = cfg.runtime_external_deps_subdirs\n"
"   end\n"
"   local dirs = {\n"
"      BINDIR = { subdir = subdirs.bin, testfile = \"program\", pattern = patterns.bin },\n"
"      INCDIR = { subdir = subdirs.include, testfile = \"header\", pattern = patterns.include },\n"
"      LIBDIR = { subdir = subdirs.lib, testfile = \"library\", pattern = patterns.lib }\n"
"   }\n"
"   if mode == \"install\" then\n"
"      dirs.INCDIR = nil\n"
"   end\n"
"   return dirs\n"
"end\n"
"\n"
"local function resolve_prefix(prefix, dirs)\n"
"   if type(prefix) == \"string\" then\n"
"      return prefix\n"
"   elseif type(prefix) == \"table\" then\n"
"      if prefix.bin then\n"
"         dirs.BINDIR.subdir = prefix.bin\n"
"      end\n"
"      if prefix.include then\n"
"         if dirs.INCDIR then\n"
"            dirs.INCDIR.subdir = prefix.include\n"
"         end\n"
"      end\n"
"      if prefix.lib then\n"
"         dirs.LIBDIR.subdir = prefix.lib\n"
"      end\n"
"      return prefix.prefix\n"
"   end\n"
"end\n"
"\n"
"local function check_external_dependency_at(prefix, name, ext_files, vars, dirs, err_files, cache)\n"
"   local fs = require(\"luarocks.fs\")\n"
"   cache = cache or {}\n"
"\n"
"   for dirname, dirdata in util.sortedpairs(dirs) do\n"
"      local paths\n"
"      local path_var_value = vars[name..\"_\"..dirname]\n"
"      if path_var_value then\n"
"         paths = { path_var_value }\n"
"      elseif type(dirdata.subdir) == \"table\" then\n"
"         paths = {}\n"
"         for i,v in ipairs(dirdata.subdir) do\n"
"            paths[i] = dir.path(prefix, v)\n"
"         end\n"
"      else\n"
"         paths = { dir.path(prefix, dirdata.subdir) }\n"
"      end\n"
"      dirdata.dir = paths[1]\n"
"      local file = ext_files[dirdata.testfile]\n"
"      if file then\n"
"         local files = {}\n"
"         -- If it doesn't look like it contains a filename extension\n"
"         if not (file:match(\"%.[a-z]+$\") or file:match(\"%.[a-z]+%.\")) then\n"
"            add_all_patterns(file, dirdata.pattern, files)\n"
"         else\n"
"            for _, pattern in ipairs(dirdata.pattern) do\n"
"               local matched = deconstruct_pattern(file, pattern)\n"
"               if matched then\n"
"                  add_all_patterns(matched, dirdata.pattern, files)\n"
"               end\n"
"            end\n"
"            table.insert(files, file)\n"
"         end\n"
"         local found = false\n"
"         for _, f in ipairs(files) do\n"
"\n"
"            -- small convenience hack\n"
"            if f:match(\"%.so$\") or f:match(\"%.dylib$\") or f:match(\"%.dll$\") then\n"
"               f = f:gsub(\"%.[^.]+$\", \".\"..cfg.external_lib_extension)\n"
"            end\n"
"\n"
"            local pattern\n"
"            if f:match(\"%*\") then\n"
"               pattern = f:gsub(\"([-.+])\", \"%%%1\"):gsub(\"%*\", \".*\")\n"
"               f = \"matching \"..f\n"
"            end\n"
"\n"
"            for _, d in ipairs(paths) do\n"
"               if pattern then\n"
"                  if not cache[d] then\n"
"                     cache[d] = fs.list_dir(d)\n"
"                  end\n"
"                  for _, entry in ipairs(cache[d]) do\n"
"                     if entry:match(pattern) then\n"
"                        found = true\n"
"                        break\n"
"                     end\n"
"                  end\n"
"               else\n"
"                  found = fs.is_file(dir.path(d, f))\n"
"               end\n"
"               if found then\n"
"                  dirdata.dir = d\n"
"                  dirdata.file = f\n"
"                  break\n"
"               else\n"
"                  table.insert(err_files[dirdata.testfile], f..\" in \"..d)\n"
"               end\n"
"            end\n"
"            if found then\n"
"               break\n"
"            end\n"
"         end\n"
"         if not found then\n"
"            return nil, dirname, dirdata.testfile\n"
"         end\n"
"      end\n"
"   end\n"
"\n"
"   for dirname, dirdata in pairs(dirs) do\n"
"      vars[name..\"_\"..dirname] = dirdata.dir\n"
"      vars[name..\"_\"..dirname..\"_FILE\"] = dirdata.file\n"
"   end\n"
"   vars[name..\"_DIR\"] = prefix\n"
"   return true\n"
"end\n"
"\n"
"local function check_external_dependency(name, ext_files, vars, mode, cache)\n"
"   local ok\n"
"   local err_dirname\n"
"   local err_testfile\n"
"   local err_files = {program = {}, header = {}, library = {}}\n"
"\n"
"   local dirs = get_external_deps_dirs(mode)\n"
"   \n"
"   local prefixes\n"
"   if vars[name .. \"_DIR\"] then\n"
"      prefixes = { vars[name .. \"_DIR\"] }\n"
"   else\n"
"      prefixes = cfg.external_deps_dirs\n"
"   end\n"
"   \n"
"   for _, prefix in ipairs(prefixes) do\n"
"      prefix = resolve_prefix(prefix, dirs)\n"
"      if cfg.is_platform(\"mingw32\") and name == \"LUA\" then\n"
"         dirs.LIBDIR.pattern = fun.filter(util.deep_copy(dirs.LIBDIR.pattern), function(s)\n"
"            return not s:match(\"%.a$\")\n"
"         end)\n"
"      end\n"
"      ok, err_dirname, err_testfile = check_external_dependency_at(prefix, name, ext_files, vars, dirs, err_files, cache)\n"
"      if ok then\n"
"         return true\n"
"      end\n"
"   end\n"
"   \n"
"   return nil, err_dirname, err_testfile, err_files\n"
"end\n"
"\n"
"--- Set up path-related variables for external dependencies.\n"
"-- For each key in the external_dependencies table in the\n"
"-- rockspec file, four variables are created: <key>_DIR, <key>_BINDIR,\n"
"-- <key>_INCDIR and <key>_LIBDIR. These are not overwritten\n"
"-- if already set (e.g. by the LuaRocks config file or through the\n"
"-- command-line). Values in the external_dependencies table\n"
"-- are tables that may contain a \"header\" or a \"library\" field,\n"
"-- with filenames to be tested for existence.\n"
"-- @param rockspec table: The rockspec table.\n"
"-- @param mode string: if \"build\" is given, checks all files;\n"
"-- if \"install\" is given, do not scan for headers.\n"
"-- @return boolean or (nil, string): True if no errors occurred, or\n"
"-- nil and an error message if any test failed.\n"
"function deps.check_external_deps(rockspec, mode)\n"
"   assert(rockspec:type() == \"rockspec\")\n"
"   \n"
"   if not rockspec.external_dependencies then\n"
"      rockspec.external_dependencies = builtin.autodetect_external_dependencies(rockspec.build)\n"
"   end\n"
"   if not rockspec.external_dependencies then\n"
"      return true\n"
"   end\n"
"\n"
"   for name, ext_files in util.sortedpairs(rockspec.external_dependencies) do\n"
"      local ok, err_dirname, err_testfile, err_files = check_external_dependency(name, ext_files, rockspec.variables, mode)\n"
"      if not ok then\n"
"         local lines = {\"Could not find \"..err_testfile..\" file for \"..name}\n"
"      \n"
"         local err_paths = {}\n"
"         for _, err_file in ipairs(err_files[err_testfile]) do\n"
"            if not err_paths[err_file] then\n"
"               err_paths[err_file] = true\n"
"               table.insert(lines, \"  No file \"..err_file)\n"
"            end\n"
"         end\n"
"      \n"
"         table.insert(lines, \"You may have to install \"..name..\" in your system and/or pass \"..name..\"_DIR or \"..name..\"_\"..err_dirname..\" to the luarocks command.\")\n"
"         table.insert(lines, \"Example: luarocks install \"..rockspec.name..\" \"..name..\"_DIR=/usr/local\")\n"
"      \n"
"         return nil, table.concat(lines, \"\\n\"), \"dependency\"\n"
"      end\n"
"   end\n"
"   return true\n"
"end\n"
"\n"
"--- Recursively add satisfied dependencies of a package to a table,\n"
"-- to build a transitive closure of all dependent packages.\n"
"-- Additionally ensures that `dependencies` table of the manifest is up-to-date.\n"
"-- @param results table: The results table being built, maps package names to versions.\n"
"-- @param manifest table: The manifest table containing dependencies.\n"
"-- @param name string: Package name.\n"
"-- @param version string: Package version.\n"
"function deps.scan_deps(results, manifest, name, version, deps_mode)\n"
"   assert(type(results) == \"table\")\n"
"   assert(type(manifest) == \"table\")\n"
"   assert(type(name) == \"string\" and not name:match(\"/\"))\n"
"   assert(type(version) == \"string\")\n"
"\n"
"   local fetch = require(\"luarocks.fetch\")\n"
"\n"
"   if results[name] then\n"
"      return\n"
"   end\n"
"   if not manifest.dependencies then manifest.dependencies = {} end\n"
"   local md = manifest.dependencies\n"
"   if not md[name] then md[name] = {} end\n"
"   local mdn = md[name]\n"
"   local dependencies = mdn[version]\n"
"   local rocks_provided\n"
"   if not dependencies then\n"
"      local rockspec, err = fetch.load_local_rockspec(path.rockspec_file(name, version), false)\n"
"      if not rockspec then\n"
"         util.printerr(\"Couldn't load rockspec for \"..name..\" \"..version..\": \"..err)\n"
"         return\n"
"      end\n"
"      dependencies = rockspec.dependencies\n"
"      rocks_provided = rockspec.rocks_provided\n"
"      mdn[version] = dependencies\n"
"   else\n"
"      rocks_provided = setmetatable({}, { __index = cfg.rocks_provided_3_0 })\n"
"   end\n"
"   local matched = deps.match_deps(dependencies, rocks_provided, nil, deps_mode)\n"
"   results[name] = version\n"
"   for _, match in pairs(matched) do\n"
"      deps.scan_deps(results, manifest, match.name, match.version, deps_mode)\n"
"   end\n"
"end\n"
"\n"
"local function find_lua_incdir(prefix, luaver, luajitver)\n"
"   luajitver = luajitver and luajitver:gsub(\"%-.*\", \"\")\n"
"   local shortv = luaver:gsub(\"%.\", \"\")\n"
"   local incdirs = {\n"
"      prefix .. \"/include/lua/\" .. luaver,\n"
"      prefix .. \"/include/lua\" .. luaver,\n"
"      prefix .. \"/include/lua\" .. shortv,\n"
"      prefix .. \"/include\",\n"
"      prefix,\n"
"      luajitver and prefix .. \"/include/luajit-\" .. luajitver:match(\"^(%d+%.%d+)\"),\n"
"   }\n"
"   for _, d in ipairs(incdirs) do\n"
"      local lua_h = dir.path(d, \"lua.h\")\n"
"      local fd = io.open(lua_h)\n"
"      if fd then\n"
"         -- TODO check that LUA_VERSION_MAJOR and LUA_VERSION_MINOR match luaver\n"
"         fd:close()\n"
"         return d\n"
"      end\n"
"   end\n"
"\n"
"   -- not found, will fallback to a default\n"
"   return nil\n"
"end\n"
"\n"
"function deps.check_lua(vars)\n"
"   local incdir_found = true\n"
"   if (not vars.LUA_INCDIR) and vars.LUA_DIR then\n"
"      vars.LUA_INCDIR = find_lua_incdir(vars.LUA_DIR, cfg.lua_version, cfg.luajit_version)\n"
"      incdir_found = (vars.LUA_INCDIR ~= nil)\n"
"   end\n"
"\n"
"   -- Tarantool has no separate dynamic library. All necessary\n"
"   -- symbols are exposed from its executable file.\n"
"   if cfg.lua_interpreter == \"tarantool\" then\n"
"      return true\n"
"   end\n"
"\n"
"   local shortv = cfg.lua_version:gsub(\"%.\", \"\")\n"
"   local libnames = {\n"
"      \"lua\" .. cfg.lua_version,\n"
"      \"lua\" .. shortv,\n"
"      \"lua-\" .. cfg.lua_version,\n"
"      \"lua-\" .. shortv,\n"
"      \"lua\",\n"
"   }\n"
"   if cfg.luajit_version then\n"
"      table.insert(libnames, 1, \"luajit-\" .. cfg.lua_version)\n"
"   end\n"
"   local cache = {}\n"
"   for _, libname in ipairs(libnames) do\n"
"      local ok = check_external_dependency(\"LUA\", { library = libname }, vars, \"build\", cache)\n"
"      if ok then\n"
"         vars.LUALIB = vars.LUA_LIBDIR_FILE\n"
"         return true\n"
"      end\n"
"   end\n"
"   if not incdir_found then\n"
"      return nil, \"Failed finding Lua header files. You may need to install them or configure LUA_INCDIR.\", \"dependency\"\n"
"   end\n"
"   return nil, \"Failed finding Lua library. You may need to configure LUA_LIBDIR.\", \"dependency\"\n"
"end\n"
"\n"
"local valid_deps_modes = {\n"
"   one = true,\n"
"   order = true,\n"
"   all = true,\n"
"   none = true,\n"
"}\n"
"\n"
"function deps.check_deps_mode_flag(flag)\n"
"   return valid_deps_modes[flag]\n"
"end\n"
"\n"
"function deps.get_deps_mode(flags)\n"
"   if flags[\"deps-mode\"] then\n"
"      return flags[\"deps-mode\"]\n"
"   else\n"
"      return cfg.deps_mode\n"
"   end\n"
"end\n"
"\n"
"return deps\n"
""
;
