const char luarocks_fetch_lua[] =
"\n"
"--- Functions related to fetching and loading local and remote files.\n"
"local fetch = {}\n"
"\n"
"local fs = require(\"luarocks.fs\")\n"
"local dir = require(\"luarocks.dir\")\n"
"local rockspecs = require(\"luarocks.rockspecs\")\n"
"local signing = require(\"luarocks.signing\")\n"
"local persist = require(\"luarocks.persist\")\n"
"local util = require(\"luarocks.util\")\n"
"local cfg = require(\"luarocks.core.cfg\")\n"
"\n"
"--- Fetch a local or remote file.\n"
"-- Make a remote or local URL/pathname local, fetching the file if necessary.\n"
"-- Other \"fetch\" and \"load\" functions use this function to obtain files.\n"
"-- If a local pathname is given, it is returned as a result.\n"
"-- @param url string: a local pathname or a remote URL.\n"
"-- @param filename string or nil: this function attempts to detect the\n"
"-- resulting local filename of the remote file as the basename of the URL;\n"
"-- if that is not correct (due to a redirection, for example), the local\n"
"-- filename can be given explicitly as this second argument.\n"
"-- @param cache boolean: compare remote timestamps via HTTP HEAD prior to\n"
"-- re-downloading the file.\n"
"-- @return (string, nil, nil, boolean) or (nil, string, [string]):\n"
"-- in case of success:\n"
"-- * the absolute local pathname for the fetched file\n"
"-- * nil\n"
"-- * nil\n"
"-- * `true` if the file was fetched from cache\n"
"-- in case of failure:\n"
"-- * nil\n"
"-- * an error message\n"
"-- * an optional error code.\n"
"function fetch.fetch_url(url, filename, cache)\n"
"   assert(type(url) == \"string\")\n"
"   assert(type(filename) == \"string\" or not filename)\n"
"\n"
"   local protocol, pathname = dir.split_url(url)\n"
"   if protocol == \"file\" then\n"
"      return fs.absolute_name(pathname)\n"
"   elseif dir.is_basic_protocol(protocol) then\n"
"      local ok, name, from_cache = fs.download(url, filename, cache)\n"
"      if not ok then\n"
"         return nil, \"Failed downloading \"..url..(filename and \" - \"..filename or \"\"), \"network\"\n"
"      end\n"
"      return name, nil, nil, from_cache\n"
"   else\n"
"      return nil, \"Unsupported protocol \"..protocol\n"
"   end\n"
"end\n"
"\n"
"--- For remote URLs, create a temporary directory and download URL inside it.\n"
"-- This temporary directory will be deleted on program termination.\n"
"-- For local URLs, just return the local pathname and its directory.\n"
"-- @param url string: URL to be downloaded\n"
"-- @param tmpname string: name pattern to use for avoiding conflicts\n"
"-- when creating temporary directory.\n"
"-- @param filename string or nil: local filename of URL to be downloaded,\n"
"-- in case it can't be inferred from the URL.\n"
"-- @return (string, string) or (nil, string, [string]): absolute local pathname of\n"
"-- the fetched file and temporary directory name; or nil and an error message\n"
"-- followed by an optional error code\n"
"function fetch.fetch_url_at_temp_dir(url, tmpname, filename)\n"
"   assert(type(url) == \"string\")\n"
"   assert(type(tmpname) == \"string\")\n"
"   assert(type(filename) == \"string\" or not filename)\n"
"   filename = filename or dir.base_name(url)\n"
"\n"
"   local protocol, pathname = dir.split_url(url)\n"
"   if protocol == \"file\" then\n"
"      if fs.exists(pathname) then\n"
"         return pathname, dir.dir_name(fs.absolute_name(pathname))\n"
"      else\n"
"         return nil, \"File not found: \"..pathname\n"
"      end\n"
"   else\n"
"      local temp_dir, err = fs.make_temp_dir(tmpname)\n"
"      if not temp_dir then\n"
"         return nil, \"Failed creating temporary directory \"..tmpname..\": \"..err\n"
"      end\n"
"      util.schedule_function(fs.delete, temp_dir)\n"
"      local ok, err = fs.change_dir(temp_dir)\n"
"      if not ok then return nil, err end\n"
"      local file, err, errcode = fetch.fetch_url(url, filename)\n"
"      fs.pop_dir()\n"
"      if not file then\n"
"         return nil, \"Error fetching file: \"..err, errcode\n"
"      end\n"
"      return file, temp_dir\n"
"   end\n"
"end\n"
"\n"
"-- Determine base directory of a fetched URL by extracting its\n"
"-- archive and looking for a directory in the root.\n"
"-- @param file string: absolute local pathname of the fetched file\n"
"-- @param temp_dir string: temporary directory in which URL was fetched.\n"
"-- @param src_url string: URL to use when inferring base directory.\n"
"-- @param src_dir string or nil: expected base directory (inferred\n"
"-- from src_url if not given).\n"
"-- @return (string, string) or (string, nil) or (nil, string):\n"
"-- The inferred base directory and the one actually found (which may\n"
"-- be nil if not found), or nil followed by an error message.\n"
"-- The inferred dir is returned first to avoid confusion with errors,\n"
"-- because it is never nil.\n"
"function fetch.find_base_dir(file, temp_dir, src_url, src_dir)\n"
"   local ok, err = fs.change_dir(temp_dir)\n"
"   if not ok then return nil, err end\n"
"   fs.unpack_archive(file)\n"
"   local inferred_dir = src_dir or dir.deduce_base_dir(src_url)\n"
"   local found_dir = nil\n"
"   if fs.exists(inferred_dir) then\n"
"      found_dir = inferred_dir\n"
"   else\n"
"      util.printerr(\"Directory \"..inferred_dir..\" not found\")\n"
"      local files = fs.list_dir()\n"
"      if files then\n"
"         table.sort(files)\n"
"         for i,filename in ipairs(files) do\n"
"            if fs.is_dir(filename) then\n"
"               util.printerr(\"Found \"..filename)\n"
"               found_dir = filename\n"
"               break\n"
"            end\n"
"         end\n"
"      end\n"
"   end\n"
"   fs.pop_dir()\n"
"   return inferred_dir, found_dir\n"
"end\n"
"\n"
"local function fetch_and_verify_signature_for(url, filename, tmpdir)\n"
"   local sig_url = signing.signature_url(url)\n"
"   local sig_file, err, errcode = fetch.fetch_url_at_temp_dir(sig_url, tmpdir)\n"
"   if not sig_file then\n"
"      return nil, \"Could not fetch signature file for verification: \" .. err, errcode\n"
"   end\n"
"   \n"
"   local ok, err = signing.verify_signature(filename, sig_file)\n"
"   if not ok then\n"
"      return nil, \"Failed signature verification: \" .. err\n"
"   end\n"
"\n"
"   return fs.absolute_name(sig_file)\n"
"end\n"
"\n"
"--- Obtain a rock and unpack it.\n"
"-- If a directory is not given, a temporary directory will be created,\n"
"-- which will be deleted on program termination.\n"
"-- @param rock_file string: URL or filename of the rock.\n"
"-- @param dest string or nil: if given, directory will be used as\n"
"-- a permanent destination.\n"
"-- @param verify boolean: if true, download and verify signature for rockspec\n"
"-- @return string or (nil, string, [string]): the directory containing the contents\n"
"-- of the unpacked rock.\n"
"function fetch.fetch_and_unpack_rock(url, dest, verify)\n"
"   assert(type(url) == \"string\")\n"
"   assert(type(dest) == \"string\" or not dest)\n"
"\n"
"   local name = dir.base_name(url):match(\"(.*)%.[^.]*%.rock\")\n"
"   local tmpname = \"luarocks-rock-\" .. name\n"
"\n"
"   local rock_file, err, errcode = fetch.fetch_url_at_temp_dir(url, tmpname)\n"
"   if not rock_file then\n"
"      return nil, \"Could not fetch rock file: \" .. err, errcode\n"
"   end\n"
"\n"
"   local sig_file\n"
"   if verify then\n"
"      sig_file, err = fetch_and_verify_signature_for(url, rock_file, tmpname)\n"
"      if err then\n"
"         return nil, err\n"
"      end\n"
"   end\n"
"\n"
"   rock_file = fs.absolute_name(rock_file)\n"
"\n"
"   local unpack_dir\n"
"   if dest then\n"
"      unpack_dir = dest\n"
"      local ok, err = fs.make_dir(unpack_dir)\n"
"      if not ok then\n"
"         return nil, \"Failed unpacking rock file: \" .. err\n"
"      end\n"
"   else\n"
"      unpack_dir, err = fs.make_temp_dir(name)\n"
"      if not unpack_dir then\n"
"         return nil, \"Failed creating temporary dir: \" .. err\n"
"      end\n"
"   end\n"
"   if not dest then\n"
"      util.schedule_function(fs.delete, unpack_dir)\n"
"   end\n"
"   local ok, err = fs.change_dir(unpack_dir)\n"
"   if not ok then return nil, err end\n"
"   ok, err = fs.unzip(rock_file)\n"
"   if not ok then\n"
"      return nil, \"Failed unpacking rock file: \" .. rock_file .. \": \" .. err\n"
"   end\n"
"   if sig_file then\n"
"      ok, err = fs.copy(sig_file, \".\")\n"
"      if not ok then\n"
"         return nil, \"Failed copying signature file\"\n"
"      end\n"
"   end\n"
"   fs.pop_dir()\n"
"   return unpack_dir\n"
"end\n"
"\n"
"--- Back-end function that actually loads the local rockspec.\n"
"-- Performs some validation and postprocessing of the rockspec contents.\n"
"-- @param rel_filename string: The local filename of the rockspec file.\n"
"-- @param quick boolean: if true, skips some steps when loading\n"
"-- rockspec.\n"
"-- @return table or (nil, string): A table representing the rockspec\n"
"-- or nil followed by an error message.\n"
"function fetch.load_local_rockspec(rel_filename, quick)\n"
"   assert(type(rel_filename) == \"string\")\n"
"   local abs_filename = fs.absolute_name(rel_filename)\n"
"\n"
"   local basename = dir.base_name(abs_filename)\n"
"   if basename ~= \"rockspec\" then\n"
"      if not basename:match(\"(.*)%-[^-]*%-[0-9]*\") then\n"
"         return nil, \"Expected filename in format 'name-version-revision.rockspec'.\"\n"
"      end\n"
"   end\n"
"\n"
"   local tbl, err = persist.load_into_table(abs_filename)\n"
"   if not tbl then\n"
"      return nil, \"Could not load rockspec file \"..abs_filename..\" (\"..err..\")\"\n"
"   end\n"
"   \n"
"   local rockspec, err = rockspecs.from_persisted_table(abs_filename, tbl, err, quick)\n"
"   if not rockspec then\n"
"      return nil, abs_filename .. \": \" .. err\n"
"   end\n"
"\n"
"   local name_version = rockspec.package:lower() .. \"-\" .. rockspec.version\n"
"   if basename ~= \"rockspec\" and basename ~= name_version .. \".rockspec\" then\n"
"      return nil, \"Inconsistency between rockspec filename (\"..basename..\") and its contents (\"..name_version..\".rockspec).\"\n"
"   end\n"
"   \n"
"   return rockspec\n"
"end\n"
"\n"
"--- Load a local or remote rockspec into a table.\n"
"-- This is the entry point for the LuaRocks tools. \n"
"-- Only the LuaRocks runtime loader should use\n"
"-- load_local_rockspec directly.\n"
"-- @param filename string: Local or remote filename of a rockspec.\n"
"-- @param location string or nil: Where to download. If not given,\n"
"-- a temporary dir is created.\n"
"-- @param verify boolean: if true, download and verify signature for rockspec\n"
"-- @return table or (nil, string, [string]): A table representing the rockspec\n"
"-- or nil followed by an error message and optional error code.\n"
"function fetch.load_rockspec(url, location, verify)\n"
"   assert(type(url) == \"string\")\n"
"\n"
"   local name\n"
"   local basename = dir.base_name(url)\n"
"   if basename == \"rockspec\" then\n"
"      name = \"rockspec\"\n"
"   else\n"
"      name = basename:match(\"(.*)%.rockspec\")\n"
"      if not name then\n"
"         return nil, \"Filename '\"..url..\"' does not look like a rockspec.\"\n"
"      end\n"
"   end\n"
"\n"
"   local tmpname = \"luarocks-rockspec-\"..name\n"
"   local filename, err, errcode\n"
"   if location then\n"
"      local ok, err = fs.change_dir(location)\n"
"      if not ok then return nil, err end\n"
"      filename, err = fetch.fetch_url(url)\n"
"      fs.pop_dir()\n"
"   else\n"
"      filename, err, errcode = fetch.fetch_url_at_temp_dir(url, tmpname)\n"
"   end\n"
"   if not filename then\n"
"      return nil, err, errcode\n"
"   end\n"
"\n"
"   if verify then\n"
"      local _, err = fetch_and_verify_signature_for(url, filename, tmpname)\n"
"      if err then\n"
"         return nil, err\n"
"      end\n"
"   end\n"
"\n"
"   return fetch.load_local_rockspec(filename)\n"
"end\n"
"\n"
"--- Download sources for building a rock using the basic URL downloader.\n"
"-- @param rockspec table: The rockspec table\n"
"-- @param extract boolean: Whether to extract the sources from\n"
"-- the fetched source tarball or not.\n"
"-- @param dest_dir string or nil: If set, will extract to the given directory;\n"
"-- if not given, will extract to a temporary directory.\n"
"-- @return (string, string) or (nil, string, [string]): The absolute pathname of\n"
"-- the fetched source tarball and the temporary directory created to\n"
"-- store it; or nil and an error message and optional error code.\n"
"function fetch.get_sources(rockspec, extract, dest_dir)\n"
"   assert(rockspec:type() == \"rockspec\")\n"
"   assert(type(extract) == \"boolean\")\n"
"   assert(type(dest_dir) == \"string\" or not dest_dir)\n"
"\n"
"   local url = rockspec.source.url\n"
"   local name = rockspec.name..\"-\"..rockspec.version\n"
"   local filename = rockspec.source.file\n"
"   local source_file, store_dir\n"
"   local ok, err, errcode\n"
"   if dest_dir then\n"
"      ok, err = fs.change_dir(dest_dir)\n"
"      if not ok then return nil, err, \"dest_dir\" end\n"
"      source_file, err, errcode = fetch.fetch_url(url, filename)\n"
"      fs.pop_dir()\n"
"      store_dir = dest_dir\n"
"   else\n"
"      source_file, store_dir, errcode = fetch.fetch_url_at_temp_dir(url, \"luarocks-source-\"..name, filename)\n"
"   end\n"
"   if not source_file then\n"
"      return nil, err or store_dir, errcode\n"
"   end\n"
"   if rockspec.source.md5 then\n"
"      if not fs.check_md5(source_file, rockspec.source.md5) then\n"
"         return nil, \"MD5 check for \"..filename..\" has failed.\", \"md5\"\n"
"      end\n"
"   end\n"
"   if extract then\n"
"      local ok, err = fs.change_dir(store_dir)\n"
"      if not ok then return nil, err end\n"
"      ok, err = fs.unpack_archive(rockspec.source.file)\n"
"      if not ok then return nil, err end\n"
"      if not fs.exists(rockspec.source.dir) then\n"
"\n"
"         -- If rockspec.source.dir can't be found, see if we only have one\n"
"         -- directory in store_dir.  If that's the case, assume it's what\n"
"         -- we're looking for.\n"
"         -- We only do this if the rockspec source.dir was not set, and only\n"
"         -- with rockspecs newer than 3.0.\n"
"         local file_count, found_dir = 0\n"
"\n"
"         if not rockspec.source.dir_set and rockspec:format_is_at_least(\"3.0\") then\n"
"            for file in fs.dir() do\n"
"               file_count = file_count + 1\n"
"               if fs.is_dir(file) then\n"
"                  found_dir = file\n"
"               end\n"
"            end\n"
"         end\n"
"\n"
"         if file_count == 1 and found_dir then\n"
"            rockspec.source.dir = found_dir\n"
"         else\n"
"            return nil, \"Directory \"..rockspec.source.dir..\" not found inside archive \"..rockspec.source.file, \"source.dir\", source_file, store_dir\n"
"         end\n"
"      end\n"
"      fs.pop_dir()\n"
"   end\n"
"   return source_file, store_dir\n"
"end\n"
"\n"
"--- Download sources for building a rock, calling the appropriate protocol method.\n"
"-- @param rockspec table: The rockspec table\n"
"-- @param extract boolean: When downloading compressed formats, whether to extract\n"
"-- the sources from the fetched archive or not.\n"
"-- @param dest_dir string or nil: If set, will extract to the given directory.\n"
"-- if not given, will extract to a temporary directory.\n"
"-- @return (string, string) or (nil, string): The absolute pathname of\n"
"-- the fetched source tarball and the temporary directory created to\n"
"-- store it; or nil and an error message.\n"
"function fetch.fetch_sources(rockspec, extract, dest_dir)\n"
"   assert(rockspec:type() == \"rockspec\")\n"
"   assert(type(extract) == \"boolean\")\n"
"   assert(type(dest_dir) == \"string\" or not dest_dir)\n"
"\n"
"   local protocol = rockspec.source.protocol\n"
"   local ok, proto\n"
"   if dir.is_basic_protocol(protocol) then\n"
"      proto = fetch\n"
"   else\n"
"      ok, proto = pcall(require, \"luarocks.fetch.\"..protocol:gsub(\"[+-]\", \"_\"))\n"
"      if not ok then\n"
"         return nil, \"Unknown protocol \"..protocol\n"
"      end\n"
"   end\n"
"   \n"
"   if cfg.only_sources_from\n"
"   and rockspec.source.pathname\n"
"   and #rockspec.source.pathname > 0 then\n"
"      if #cfg.only_sources_from == 0 then\n"
"         return nil, \"Can't download \"..rockspec.source.url..\" -- download from remote servers disabled\"\n"
"      elseif rockspec.source.pathname:find(cfg.only_sources_from, 1, true) ~= 1 then\n"
"         return nil, \"Can't download \"..rockspec.source.url..\" -- only downloading from \"..cfg.only_sources_from\n"
"      end\n"
"   end\n"
"   return proto.get_sources(rockspec, extract, dest_dir)\n"
"end\n"
"\n"
"return fetch\n"
""
;
