const char luarocks_fetch_git_lua[] =
"\n"
"--- Fetch back-end for retrieving sources from GIT.\n"
"local git = {}\n"
"\n"
"local unpack = unpack or table.unpack\n"
"\n"
"local fs = require(\"luarocks.fs\")\n"
"local dir = require(\"luarocks.dir\")\n"
"local vers = require(\"luarocks.core.vers\")\n"
"local util = require(\"luarocks.util\")\n"
"\n"
"local cached_git_version\n"
"\n"
"--- Get git version.\n"
"-- @param git_cmd string: name of git command.\n"
"-- @return table: git version as returned by luarocks.core.vers.parse_version.\n"
"local function git_version(git_cmd)\n"
"   if not cached_git_version then\n"
"      local version_line = io.popen(fs.Q(git_cmd)..' --version'):read()\n"
"      local version_string = version_line:match('%d-%.%d+%.\?%d*')\n"
"      cached_git_version = vers.parse_version(version_string)\n"
"   end\n"
"\n"
"   return cached_git_version\n"
"end\n"
"\n"
"--- Check if git satisfies version requirement.\n"
"-- @param git_cmd string: name of git command.\n"
"-- @param version string: required version.\n"
"-- @return boolean: true if git matches version or is newer, false otherwise.\n"
"local function git_is_at_least(git_cmd, version)\n"
"   return git_version(git_cmd) >= vers.parse_version(version)\n"
"end\n"
"\n"
"--- Git >= 1.7.10 can clone a branch **or tag**, < 1.7.10 by branch only. We\n"
"-- need to know this in order to build the appropriate command; if we can't\n"
"-- clone by tag then we'll have to issue a subsequent command to check out the\n"
"-- given tag.\n"
"-- @param git_cmd string: name of git command.\n"
"-- @return boolean: Whether Git can clone by tag.\n"
"local function git_can_clone_by_tag(git_cmd)\n"
"   return git_is_at_least(git_cmd, \"1.7.10\")\n"
"end\n"
"\n"
"--- Git >= 1.8.4 can fetch submodules shallowly, saving bandwidth and time for\n"
"-- submodules with large history.\n"
"-- @param git_cmd string: name of git command.\n"
"-- @return boolean: Whether Git can fetch submodules shallowly.\n"
"local function git_supports_shallow_submodules(git_cmd)\n"
"   return git_is_at_least(git_cmd, \"1.8.4\")\n"
"end\n"
"\n"
"local function git_identifier(git_cmd, ver)\n"
"   if not (ver:match(\"^dev%-%d+$\") or ver:match(\"^scm%-%d+$\")) then\n"
"      return nil\n"
"   end\n"
"   local pd = io.popen(fs.command_at(fs.current_dir(), fs.Q(git_cmd)..\" log --pretty=format:%ai_%h -n 1\"))\n"
"   if not pd then\n"
"      return nil\n"
"   end\n"
"   local date_hash = pd:read(\"*l\")\n"
"   pd:close()\n"
"   if not date_hash then\n"
"      return nil\n"
"   end\n"
"   local date, time, tz, hash = date_hash:match(\"([^%s]+) ([^%s]+) ([^%s]+)_([^%s]+)\")\n"
"   date = date:gsub(\"%-\", \"\")\n"
"   time = time:gsub(\":\", \"\")\n"
"   return date .. \".\" .. time .. \".\" .. hash\n"
"end\n"
"\n"
"--- Download sources for building a rock, using git.\n"
"-- @param rockspec table: The rockspec table\n"
"-- @param extract boolean: Unused in this module (required for API purposes.)\n"
"-- @param dest_dir string or nil: If set, will extract to the given directory.\n"
"-- @return (string, string) or (nil, string): The absolute pathname of\n"
"-- the fetched source tarball and the temporary directory created to\n"
"-- store it; or nil and an error message.\n"
"function git.get_sources(rockspec, extract, dest_dir, depth)\n"
"   assert(rockspec:type() == \"rockspec\")\n"
"   assert(type(dest_dir) == \"string\" or not dest_dir)\n"
"\n"
"   local git_cmd = rockspec.variables.GIT\n"
"   local name_version = rockspec.name .. \"-\" .. rockspec.version\n"
"   local module = dir.base_name(rockspec.source.url)\n"
"   -- Strip off .git from base name if present\n"
"   module = module:gsub(\"%.git$\", \"\")\n"
"\n"
"   local ok, err_msg = fs.is_tool_available(git_cmd, \"Git\")\n"
"   if not ok then\n"
"      return nil, err_msg\n"
"   end\n"
"\n"
"   local store_dir\n"
"   if not dest_dir then\n"
"      store_dir = fs.make_temp_dir(name_version)\n"
"      if not store_dir then\n"
"         return nil, \"Failed creating temporary directory.\"\n"
"      end\n"
"      util.schedule_function(fs.delete, store_dir)\n"
"   else\n"
"      store_dir = dest_dir\n"
"   end\n"
"   store_dir = fs.absolute_name(store_dir)\n"
"   local ok, err = fs.change_dir(store_dir)\n"
"   if not ok then return nil, err end\n"
"\n"
"   local command = {fs.Q(git_cmd), \"clone\", \"--recursive\", depth or \"--depth=1\", rockspec.source.url, module}\n"
"   local tag_or_branch = rockspec.source.tag or rockspec.source.branch\n"
"   -- If the tag or branch is explicitly set to \"master\" in the rockspec, then\n"
"   -- we can avoid passing it to Git since it's the default.\n"
"   if tag_or_branch == \"master\" then tag_or_branch = nil end\n"
"   if tag_or_branch then\n"
"      if git_can_clone_by_tag(git_cmd) then\n"
"         -- The argument to `--branch` can actually be a branch or a tag as of\n"
"         -- Git 1.7.10.\n"
"         table.insert(command, 3, \"--branch=\" .. tag_or_branch)\n"
"      end\n"
"   end\n"
"   if not fs.execute(unpack(command)) then\n"
"      return nil, \"Failed cloning git repository.\"\n"
"   end\n"
"   ok, err = fs.change_dir(module)\n"
"   if not ok then return nil, err end\n"
"   if tag_or_branch and not git_can_clone_by_tag() then\n"
"      if not fs.execute(fs.Q(git_cmd), \"checkout\", tag_or_branch) then\n"
"         return nil, 'Failed to check out the \"' .. tag_or_branch ..'\" tag or branch.'\n"
"      end\n"
"   end\n"
"\n"
"   -- Fetching git submodules is supported only when rockspec format is >= 3.0.\n"
"   if rockspec:format_is_at_least(\"3.0\") then\n"
"      command = {fs.Q(git_cmd), \"submodule\", \"update\", \"--init\", \"--recursive\"}\n"
"\n"
"      if git_supports_shallow_submodules(git_cmd) then\n"
"         -- Fetch only the last commit of each submodule.\n"
"         table.insert(command, 5, \"--depth=1\")\n"
"      end\n"
"\n"
"      if not fs.execute(unpack(command)) then\n"
"         return nil, 'Failed to fetch submodules.'\n"
"      end\n"
"   end\n"
"   \n"
"   if not rockspec.source.tag then\n"
"      rockspec.source.identifier = git_identifier(git_cmd, rockspec.version)\n"
"   end\n"
"\n"
"   fs.delete(dir.path(store_dir, module, \".git\"))\n"
"   fs.delete(dir.path(store_dir, module, \".gitignore\"))\n"
"   fs.pop_dir()\n"
"   fs.pop_dir()\n"
"   return module, store_dir\n"
"end\n"
"\n"
"return git\n"
""
;
