const char luarocks_fs_lua_lua[] =
"\n"
"--- Native Lua implementation of filesystem and platform abstractions,\n"
"-- using LuaFileSystem, LuaSocket, LuaSec, lua-zlib, LuaPosix, MD5.\n"
"-- module(\"luarocks.fs.lua\")\n"
"local fs_lua = {}\n"
"\n"
"local fs = require(\"luarocks.fs\")\n"
"\n"
"local cfg = require(\"luarocks.core.cfg\")\n"
"local dir = require(\"luarocks.dir\")\n"
"local util = require(\"luarocks.util\")\n"
"\n"
"local socket_ok, zip_ok, lfs_ok, md5_ok, digest_ok, posix_ok, bz2_ok, _\n"
"local http, ftp, zip, lfs, md5, digest, posix, bz2\n"
"\n"
"if cfg.fs_use_modules then\n"
"   socket_ok, http = pcall(require, \"socket.http\")\n"
"   _, ftp = pcall(require, \"socket.ftp\")\n"
"   zip_ok, zip = pcall(require, \"luarocks.tools.zip\")\n"
"   bz2_ok, bz2 = pcall(require, \"luarocks.tools.bzip2\")\n"
"   lfs_ok, lfs = pcall(require, \"lfs\")\n"
"   md5_ok, md5 = pcall(require, \"md5\")\n"
"   digest_ok, digest = pcall(require, \"digest\")\n"
"   posix_ok, posix = pcall(require, \"posix\")\n"
"end\n"
"\n"
"local patch = require(\"luarocks.tools.patch\")\n"
"local tar = require(\"luarocks.tools.tar\")\n"
"\n"
"local dir_stack = {}\n"
"\n"
"--- Test is file/dir is writable.\n"
"-- Warning: testing if a file/dir is writable does not guarantee\n"
"-- that it will remain writable and therefore it is no replacement\n"
"-- for checking the result of subsequent operations.\n"
"-- @param file string: filename to test\n"
"-- @return boolean: true if file exists, false otherwise.\n"
"function fs_lua.is_writable(file)\n"
"   assert(file)\n"
"   file = dir.normalize(file)\n"
"   local result\n"
"   if fs.is_dir(file) then\n"
"      local file2 = dir.path(file, '.tmpluarockstestwritable')\n"
"      local fh = io.open(file2, 'wb')\n"
"      result = fh ~= nil\n"
"      if fh then fh:close() end\n"
"      os.remove(file2)\n"
"   else\n"
"      local fh = io.open(file, 'r+b')\n"
"      result = fh ~= nil\n"
"      if fh then fh:close() end\n"
"   end\n"
"   return result\n"
"end\n"
"\n"
"local function quote_args(command, ...)\n"
"   local out = { command }\n"
"   for _, arg in ipairs({...}) do\n"
"      assert(type(arg) == \"string\")\n"
"      out[#out+1] = fs.Q(arg)\n"
"   end\n"
"   return table.concat(out, \" \")\n"
"end\n"
"\n"
"--- Run the given command, quoting its arguments.\n"
"-- The command is executed in the current directory in the dir stack.\n"
"-- @param command string: The command to be executed. No quoting/escaping\n"
"-- is applied.\n"
"-- @param ... Strings containing additional arguments, which are quoted.\n"
"-- @return boolean: true if command succeeds (status code 0), false\n"
"-- otherwise.\n"
"function fs_lua.execute(command, ...)\n"
"   assert(type(command) == \"string\")\n"
"   return fs.execute_string(quote_args(command, ...))\n"
"end\n"
"\n"
"--- Run the given command, quoting its arguments, silencing its output.\n"
"-- The command is executed in the current directory in the dir stack.\n"
"-- Silencing is omitted if 'verbose' mode is enabled.\n"
"-- @param command string: The command to be executed. No quoting/escaping\n"
"-- is applied.\n"
"-- @param ... Strings containing additional arguments, which will be quoted.\n"
"-- @return boolean: true if command succeeds (status code 0), false\n"
"-- otherwise.\n"
"function fs_lua.execute_quiet(command, ...)\n"
"   assert(type(command) == \"string\")\n"
"   if cfg.verbose then -- omit silencing output\n"
"      return fs.execute_string(quote_args(command, ...))\n"
"   else\n"
"      return fs.execute_string(fs.quiet(quote_args(command, ...)))\n"
"   end\n"
"end\n"
"\n"
"function fs.execute_env(env, command, ...)\n"
"   assert(type(command) == \"string\")\n"
"   local envstr = {}\n"
"   for var, val in pairs(env) do\n"
"      table.insert(envstr, fs.export_cmd(var, val))\n"
"   end\n"
"   return fs.execute_string(table.concat(envstr, \"\\n\") .. \"\\n\" .. quote_args(command, ...))\n"
"end\n"
"\n"
"--- Checks if the given tool is available.\n"
"-- The tool is executed using a flag, usually just to ask its version.\n"
"-- @param tool_cmd string: The command to be used to check the tool's presence (e.g. hg in case of Mercurial)\n"
"-- @param tool_name string: The actual name of the tool (e.g. Mercurial)\n"
"-- @param arg string: The flag to pass to the tool. '--version' by default.\n"
"function fs_lua.is_tool_available(tool_cmd, tool_name, arg)\n"
"   assert(type(tool_cmd) == \"string\")\n"
"   assert(type(tool_name) == \"string\")\n"
"\n"
"   arg = arg or \"--version\"\n"
"   assert(type(arg) == \"string\")\n"
"\n"
"   if not fs.execute_quiet(tool_cmd, arg) then\n"
"      local msg = \"'%s' program not found. Make sure %s is installed and is available in your PATH \" ..\n"
"                  \"(or you may want to edit the 'variables.%s' value in file '%s')\"\n"
"      return nil, msg:format(tool_cmd, tool_name, tool_name:upper(), cfg.config_files.nearest)\n"
"   else\n"
"      return true\n"
"   end\n"
"end\n"
"\n"
"--- Check the MD5 checksum for a file.\n"
"-- @param file string: The file to be checked.\n"
"-- @param md5sum string: The string with the expected MD5 checksum.\n"
"-- @return boolean: true if the MD5 checksum for 'file' equals 'md5sum', false + msg if not\n"
"-- or if it could not perform the check for any reason.\n"
"function fs_lua.check_md5(file, md5sum)\n"
"   file = dir.normalize(file)\n"
"   local computed, msg = fs.get_md5(file)\n"
"   if not computed then\n"
"      return false, msg\n"
"   end\n"
"   if computed:match(\"^\"..md5sum) then\n"
"      return true\n"
"   else\n"
"      return false, \"Mismatch MD5 hash for file \"..file\n"
"   end\n"
"end\n"
"\n"
"--- List the contents of a directory.\n"
"-- @param at string or nil: directory to list (will be the current\n"
"-- directory if none is given).\n"
"-- @return table: an array of strings with the filenames representing\n"
"-- the contents of a directory.\n"
"function fs_lua.list_dir(at)\n"
"   local result = {}\n"
"   for file in fs.dir(at) do\n"
"      result[#result+1] = file\n"
"   end\n"
"   return result\n"
"end\n"
"\n"
"--- Iterate over the contents of a directory.\n"
"-- @param at string or nil: directory to list (will be the current\n"
"-- directory if none is given).\n"
"-- @return function: an iterator function suitable for use with\n"
"-- the for statement.\n"
"function fs_lua.dir(at)\n"
"   if not at then\n"
"      at = fs.current_dir()\n"
"   end\n"
"   at = dir.normalize(at)\n"
"   if not fs.is_dir(at) then\n"
"      return function() end\n"
"   end\n"
"   return coroutine.wrap(function() fs.dir_iterator(at) end)\n"
"end\n"
"\n"
"--- List the Lua modules at a specific require path.\n"
"-- eg. `modules(\"luarocks.cmd\")` would return a list of all LuaRocks command\n"
"-- modules, in the current Lua path.\n"
"function fs_lua.modules(at)\n"
"   at = at or \"\"\n"
"   if #at > 0 then\n"
"      -- turn require path into file path\n"
"      at = at:gsub(\"%.\", package.config:sub(1,1)) .. package.config:sub(1,1)\n"
"   end\n"
"\n"
"   local path = package.path:sub(-1, -1) == \";\" and package.path or package.path .. \";\"\n"
"   local paths = {}\n"
"   for location in path:gmatch(\"(.-);\") do\n"
"      if location:lower() == \"\?.lua\" then\n"
"         location = \"./\?.lua\"\n"
"      end\n"
"      local _, q_count = location:gsub(\"%\?\", \"\") -- only use the ones with a single '\?'\n"
"      if location:match(\"%\?%.[lL][uU][aA]$\") and q_count == 1 then  -- only use when ending with \"\?.lua\"\n"
"         location = location:gsub(\"%\?%.[lL][uU][aA]$\", at)\n"
"         table.insert(paths, location)\n"
"      end\n"
"   end\n"
"\n"
"   if #paths == 0 then\n"
"      return {}\n"
"   end\n"
"\n"
"   local modules = {}\n"
"   local is_duplicate = {}\n"
"   for _, path in ipairs(paths) do\n"
"      local files = fs.list_dir(path)\n"
"      for _, filename in ipairs(files or {}) do\n"
"         if filename:match(\"%.[lL][uU][aA]$\") then\n"
"           filename = filename:sub(1,-5) -- drop the extension\n"
"           if not is_duplicate[filename] then\n"
"              is_duplicate[filename] = true\n"
"              table.insert(modules, filename)\n"
"           end\n"
"         end\n"
"      end\n"
"   end\n"
"\n"
"   return modules\n"
"end\n"
"\n"
"function fs_lua.filter_file(fn, input_filename, output_filename)\n"
"   local fd, err = io.open(input_filename, \"rb\")\n"
"   if not fd then\n"
"      return nil, err\n"
"   end\n"
"\n"
"   local input, err = fd:read(\"*a\")\n"
"   fd:close()\n"
"   if not input then\n"
"      return nil, err\n"
"   end\n"
"\n"
"   local output, err = fn(input)\n"
"   if not output then\n"
"      return nil, err\n"
"   end\n"
"\n"
"   fd, err = io.open(output_filename, \"wb\")\n"
"   if not fd then\n"
"      return nil, err\n"
"   end\n"
"\n"
"   local ok, err = fd:write(output)\n"
"   fd:close()\n"
"   if not ok then\n"
"      return nil, err\n"
"   end\n"
"\n"
"   return true\n"
"end\n"
"\n"
"function fs_lua.system_temp_dir()\n"
"   return os.getenv(\"TMPDIR\") or os.getenv(\"TEMP\") or \"/tmp\"\n"
"end\n"
"\n"
"---------------------------------------------------------------------\n"
"-- LuaFileSystem functions\n"
"---------------------------------------------------------------------\n"
"\n"
"if lfs_ok then\n"
"\n"
"--- Run the given command.\n"
"-- The command is executed in the current directory in the dir stack.\n"
"-- @param cmd string: No quoting/escaping is applied to the command.\n"
"-- @return boolean: true if command succeeds (status code 0), false\n"
"-- otherwise.\n"
"function fs_lua.execute_string(cmd)\n"
"   local code = os.execute(cmd)\n"
"   return (code == 0 or code == true)\n"
"end\n"
"\n"
"--- Obtain current directory.\n"
"-- Uses the module's internal dir stack.\n"
"-- @return string: the absolute pathname of the current directory.\n"
"function fs_lua.current_dir()\n"
"   return lfs.currentdir()\n"
"end\n"
"\n"
"--- Change the current directory.\n"
"-- Uses the module's internal dir stack. This does not have exact\n"
"-- semantics of chdir, as it does not handle errors the same way,\n"
"-- but works well for our purposes for now.\n"
"-- @param d string: The directory to switch to.\n"
"function fs_lua.change_dir(d)\n"
"   table.insert(dir_stack, lfs.currentdir())\n"
"   d = dir.normalize(d)\n"
"   return lfs.chdir(d)\n"
"end\n"
"\n"
"--- Change directory to root.\n"
"-- Allows leaving a directory (e.g. for deleting it) in\n"
"-- a crossplatform way.\n"
"function fs_lua.change_dir_to_root()\n"
"   local current = lfs.currentdir()\n"
"   if not current or current == \"\" then\n"
"      return false\n"
"   end\n"
"   table.insert(dir_stack, current)\n"
"   lfs.chdir(\"/\") -- works on Windows too\n"
"   return true\n"
"end\n"
"\n"
"--- Change working directory to the previous in the dir stack.\n"
"-- @return true if a pop occurred, false if the stack was empty.\n"
"function fs_lua.pop_dir()\n"
"   local d = table.remove(dir_stack)\n"
"   if d then\n"
"      lfs.chdir(d)\n"
"      return true\n"
"   else\n"
"      return false\n"
"   end\n"
"end\n"
"\n"
"--- Create a directory if it does not already exist.\n"
"-- If any of the higher levels in the path name do not exist\n"
"-- too, they are created as well.\n"
"-- @param directory string: pathname of directory to create.\n"
"-- @return boolean or (boolean, string): true on success or (false, error message) on failure.\n"
"function fs_lua.make_dir(directory)\n"
"   assert(type(directory) == \"string\")\n"
"   directory = dir.normalize(directory)\n"
"   local path = nil\n"
"   if directory:sub(2, 2) == \":\" then\n"
"     path = directory:sub(1, 2)\n"
"     directory = directory:sub(4)\n"
"   else\n"
"     if directory:match(\"^/\") then\n"
"        path = \"\"\n"
"     end\n"
"   end\n"
"   for d in directory:gmatch(\"([^/]+)/*\") do\n"
"      path = path and path .. \"/\" .. d or d\n"
"      local mode = lfs.attributes(path, \"mode\")\n"
"      if not mode then\n"
"         local ok, err = lfs.mkdir(path)\n"
"         if not ok then\n"
"            return false, err\n"
"         end\n"
"         ok, err = fs.set_permissions(path, \"exec\", \"all\")\n"
"         if not ok then\n"
"            return false, err\n"
"         end\n"
"      elseif mode ~= \"directory\" then\n"
"         return false, path..\" is not a directory\"\n"
"      end\n"
"   end\n"
"   return true\n"
"end\n"
"\n"
"--- Remove a directory if it is empty.\n"
"-- Does not return errors (for example, if directory is not empty or\n"
"-- if already does not exist)\n"
"-- @param d string: pathname of directory to remove.\n"
"function fs_lua.remove_dir_if_empty(d)\n"
"   assert(d)\n"
"   d = dir.normalize(d)\n"
"   lfs.rmdir(d)\n"
"end\n"
"\n"
"--- Remove a directory if it is empty.\n"
"-- Does not return errors (for example, if directory is not empty or\n"
"-- if already does not exist)\n"
"-- @param d string: pathname of directory to remove.\n"
"function fs_lua.remove_dir_tree_if_empty(d)\n"
"   assert(d)\n"
"   d = dir.normalize(d)\n"
"   for i=1,10 do\n"
"      lfs.rmdir(d)\n"
"      d = dir.dir_name(d)\n"
"   end\n"
"end\n"
"\n"
"--- Copy a file.\n"
"-- @param src string: Pathname of source\n"
"-- @param dest string: Pathname of destination\n"
"-- @param perms string (\"read\" or \"exec\") or nil: Permissions for destination\n"
"-- file or nil to use the source file permissions\n"
"-- @return boolean or (boolean, string): true on success, false on failure,\n"
"-- plus an error message.\n"
"function fs_lua.copy(src, dest, perms)\n"
"   assert(src and dest)\n"
"   src = dir.normalize(src)\n"
"   dest = dir.normalize(dest)\n"
"   local destmode = lfs.attributes(dest, \"mode\")\n"
"   if destmode == \"directory\" then\n"
"      dest = dir.path(dest, dir.base_name(src))\n"
"   end\n"
"   if src == dest or (cfg.is_platform(\"unix\") and lfs.attributes(src, \"ino\") == lfs.attributes(dest, \"ino\")) then\n"
"      return nil, \"The source and destination are the same files\"\n"
"   end\n"
"   local src_h, err = io.open(src, \"rb\")\n"
"   if not src_h then return nil, err end\n"
"   local dest_h, err = io.open(dest, \"w+b\")\n"
"   if not dest_h then src_h:close() return nil, err end\n"
"   while true do\n"
"      local block = src_h:read(8192)\n"
"      if not block then break end\n"
"      dest_h:write(block)\n"
"   end\n"
"   src_h:close()\n"
"   dest_h:close()\n"
"\n"
"   local fullattrs\n"
"   if not perms then\n"
"      fullattrs = lfs.attributes(src, \"permissions\")\n"
"   end\n"
"   if fullattrs and posix_ok then\n"
"      return posix.chmod(dest, fullattrs)\n"
"   else\n"
"      if not perms then\n"
"         perms = fullattrs:match(\"x\") and \"exec\" or \"read\"\n"
"      end\n"
"      return fs.set_permissions(dest, perms, \"all\")\n"
"   end\n"
"end\n"
"\n"
"--- Implementation function for recursive copy of directory contents.\n"
"-- Assumes paths are normalized.\n"
"-- @param src string: Pathname of source\n"
"-- @param dest string: Pathname of destination\n"
"-- @param perms string (\"read\" or \"exec\") or nil: Optional permissions.\n"
"-- If not given, permissions of the source are copied over to the destination.\n"
"-- @return boolean or (boolean, string): true on success, false on failure\n"
"local function recursive_copy(src, dest, perms)\n"
"   local srcmode = lfs.attributes(src, \"mode\")\n"
"\n"
"   if srcmode == \"file\" then\n"
"      local ok = fs.copy(src, dest, perms)\n"
"      if not ok then return false end\n"
"   elseif srcmode == \"directory\" then\n"
"      local subdir = dir.path(dest, dir.base_name(src))\n"
"      local ok, err = fs.make_dir(subdir)\n"
"      if not ok then return nil, err end\n"
"      if pcall(lfs.dir, src) == false then\n"
"         return false\n"
"      end\n"
"      for file in lfs.dir(src) do\n"
"         if file ~= \".\" and file ~= \"..\" then\n"
"            local ok = recursive_copy(dir.path(src, file), subdir, perms)\n"
"            if not ok then return false end\n"
"         end\n"
"      end\n"
"   end\n"
"   return true\n"
"end\n"
"\n"
"--- Recursively copy the contents of a directory.\n"
"-- @param src string: Pathname of source\n"
"-- @param dest string: Pathname of destination\n"
"-- @param perms string (\"read\" or \"exec\") or nil: Optional permissions.\n"
"-- @return boolean or (boolean, string): true on success, false on failure,\n"
"-- plus an error message.\n"
"function fs_lua.copy_contents(src, dest, perms)\n"
"   assert(src and dest)\n"
"   src = dir.normalize(src)\n"
"   dest = dir.normalize(dest)\n"
"   if not fs.is_dir(src) then\n"
"      return false, src .. \" is not a directory\"\n"
"   end\n"
"   if pcall(lfs.dir, src) == false then\n"
"      return false, \"Permission denied\"\n"
"   end\n"
"   for file in lfs.dir(src) do\n"
"      if file ~= \".\" and file ~= \"..\" then\n"
"         local ok = recursive_copy(dir.path(src, file), dest, perms)\n"
"         if not ok then\n"
"            return false, \"Failed copying \"..src..\" to \"..dest\n"
"         end\n"
"      end\n"
"   end\n"
"   return true\n"
"end\n"
"\n"
"--- Implementation function for recursive removal of directories.\n"
"-- Assumes paths are normalized.\n"
"-- @param name string: Pathname of file\n"
"-- @return boolean or (boolean, string): true on success,\n"
"-- or nil and an error message on failure.\n"
"local function recursive_delete(name)\n"
"   local ok = os.remove(name)\n"
"   if ok then return true end\n"
"   local pok, ok, err = pcall(function()\n"
"      for file in lfs.dir(name) do\n"
"         if file ~= \".\" and file ~= \"..\" then\n"
"            local ok, err = recursive_delete(dir.path(name, file))\n"
"            if not ok then return nil, err end\n"
"         end\n"
"      end\n"
"      local ok, err = lfs.rmdir(name)\n"
"      return ok, (not ok) and err\n"
"   end)\n"
"   if pok then\n"
"      return ok, err\n"
"   else\n"
"      return pok, ok\n"
"   end\n"
"end\n"
"\n"
"--- Delete a file or a directory and all its contents.\n"
"-- @param name string: Pathname of source\n"
"-- @return nil\n"
"function fs_lua.delete(name)\n"
"   name = dir.normalize(name)\n"
"   recursive_delete(name)\n"
"end\n"
"\n"
"--- Internal implementation function for fs.dir.\n"
"-- Yields a filename on each iteration.\n"
"-- @param at string: directory to list\n"
"-- @return nil or (nil and string): an error message on failure \n"
"function fs_lua.dir_iterator(at)\n"
"   local pok, iter, arg = pcall(lfs.dir, at)\n"
"   if not pok then\n"
"      return nil, iter\n"
"   end\n"
"   for file in iter, arg do\n"
"      if file ~= \".\" and file ~= \"..\" then\n"
"         coroutine.yield(file)\n"
"      end\n"
"   end\n"
"end\n"
"\n"
"--- Implementation function for recursive find.\n"
"-- Assumes paths are normalized.\n"
"-- @param cwd string: Current working directory in recursion.\n"
"-- @param prefix string: Auxiliary prefix string to form pathname.\n"
"-- @param result table: Array of strings where results are collected.\n"
"local function recursive_find(cwd, prefix, result)\n"
"   local pok, iter, arg = pcall(lfs.dir, cwd)\n"
"   if not pok then\n"
"      return nil\n"
"   end\n"
"   for file in iter, arg do\n"
"      if file ~= \".\" and file ~= \"..\" then\n"
"         local item = prefix .. file\n"
"         table.insert(result, item)\n"
"         local pathname = dir.path(cwd, file)\n"
"         if lfs.attributes(pathname, \"mode\") == \"directory\" then\n"
"            recursive_find(pathname, item..\"/\", result)\n"
"         end\n"
"      end\n"
"   end\n"
"end\n"
"\n"
"--- Recursively scan the contents of a directory.\n"
"-- @param at string or nil: directory to scan (will be the current\n"
"-- directory if none is given).\n"
"-- @return table: an array of strings with the filenames representing\n"
"-- the contents of a directory.\n"
"function fs_lua.find(at)\n"
"   assert(type(at) == \"string\" or not at)\n"
"   if not at then\n"
"      at = fs.current_dir()\n"
"   end\n"
"   at = dir.normalize(at)\n"
"   local result = {}\n"
"   recursive_find(at, \"\", result)\n"
"   return result\n"
"end\n"
"\n"
"--- Test for existence of a file.\n"
"-- @param file string: filename to test\n"
"-- @return boolean: true if file exists, false otherwise.\n"
"function fs_lua.exists(file)\n"
"   assert(file)\n"
"   file = dir.normalize(file)\n"
"   return type(lfs.attributes(file)) == \"table\"\n"
"end\n"
"\n"
"--- Test is pathname is a directory.\n"
"-- @param file string: pathname to test\n"
"-- @return boolean: true if it is a directory, false otherwise.\n"
"function fs_lua.is_dir(file)\n"
"   assert(file)\n"
"   file = dir.normalize(file)\n"
"   return lfs.attributes(file, \"mode\") == \"directory\"\n"
"end\n"
"\n"
"--- Test is pathname is a regular file.\n"
"-- @param file string: pathname to test\n"
"-- @return boolean: true if it is a file, false otherwise.\n"
"function fs_lua.is_file(file)\n"
"   assert(file)\n"
"   file = dir.normalize(file)\n"
"   return lfs.attributes(file, \"mode\") == \"file\"\n"
"end\n"
"\n"
"-- Set access and modification times for a file.\n"
"-- @param filename File to set access and modification times for.\n"
"-- @param time may be a number containing the format returned\n"
"-- by os.time, or a table ready to be processed via os.time; if\n"
"-- nil, current time is assumed.\n"
"function fs_lua.set_time(file, time)\n"
"   assert(time == nil or type(time) == \"table\" or type(time) == \"number\")\n"
"   file = dir.normalize(file)\n"
"   if type(time) == \"table\" then\n"
"      time = os.time(time)\n"
"   end\n"
"   return lfs.touch(file, time)\n"
"end\n"
"\n"
"else -- if not lfs_ok\n"
"\n"
"function fs_lua.exists(file)\n"
"   assert(file)\n"
"   file = dir.normalize(fs.absolute_name(file))\n"
"   -- check if file exists by attempting to open it\n"
"   return util.exists(file)\n"
"end\n"
"\n"
"end\n"
"\n"
"---------------------------------------------------------------------\n"
"-- lua-bz2 functions\n"
"---------------------------------------------------------------------\n"
"\n"
"if bz2_ok then\n"
"\n"
"local function bunzip2_string(data)\n"
"   local decompressor = bz2.initDecompress()\n"
"   local output, err = decompressor:update(data)\n"
"   if not output then\n"
"      return nil, err\n"
"   end\n"
"   decompressor:close()\n"
"   return output\n"
"end\n"
"\n"
"--- Uncompresses a .bz2 file.\n"
"-- @param infile string: pathname of .bz2 file to be extracted.\n"
"-- @param outfile string or nil: pathname of output file to be produced.\n"
"-- If not given, name is derived from input file.\n"
"-- @return boolean: true on success; nil and error message on failure.\n"
"function fs_lua.bunzip2(infile, outfile)\n"
"   assert(type(infile) == \"string\")\n"
"   assert(outfile == nil or type(outfile) == \"string\")\n"
"   if not outfile then\n"
"      outfile = infile:gsub(\"%.bz2$\", \"\")\n"
"   end\n"
"\n"
"   return fs.filter_file(bunzip2_string, infile, outfile)\n"
"end\n"
"\n"
"end\n"
"\n"
"---------------------------------------------------------------------\n"
"-- luarocks.tools.zip functions\n"
"---------------------------------------------------------------------\n"
"\n"
"if zip_ok then\n"
"\n"
"function fs_lua.zip(zipfile, ...)\n"
"   return zip.zip(zipfile, ...)\n"
"end\n"
"\n"
"function fs_lua.unzip(zipfile)\n"
"   return zip.unzip(zipfile)\n"
"end\n"
"\n"
"function fs_lua.gunzip(infile, outfile)\n"
"   return zip.gunzip(infile, outfile)\n"
"end\n"
"\n"
"end\n"
"\n"
"---------------------------------------------------------------------\n"
"-- LuaSocket functions\n"
"---------------------------------------------------------------------\n"
"\n"
"if socket_ok then\n"
"\n"
"local ltn12 = require(\"ltn12\")\n"
"local luasec_ok, https = pcall(require, \"ssl.https\")\n"
"\n"
"local redirect_protocols = {\n"
"   http = http,\n"
"   https = luasec_ok and https,\n"
"}\n"
"\n"
"local function request(url, method, http, loop_control)\n"
"   local result = {}\n"
"   \n"
"   if cfg.verbose then\n"
"      print(method, url)\n"
"   end\n"
"\n"
"   local proxy = os.getenv(\"http_proxy\")\n"
"   if type(proxy) ~= \"string\" then proxy = nil end\n"
"   -- LuaSocket's http.request crashes when given URLs missing the scheme part.\n"
"   if proxy and not proxy:find(\"://\") then\n"
"      proxy = \"http://\" .. proxy\n"
"   end\n"
"\n"
"   if cfg.show_downloads then\n"
"      io.write(method..\" \"..url..\" ...\\n\")\n"
"   end\n"
"   local dots = 0\n"
"   if cfg.connection_timeout and cfg.connection_timeout > 0 then\n"
"      http.TIMEOUT = cfg.connection_timeout\n"
"   end\n"
"   local res, status, headers, err = http.request {\n"
"      url = url,\n"
"      proxy = proxy,\n"
"      method = method,\n"
"      redirect = false,\n"
"      sink = ltn12.sink.table(result),\n"
"      step = cfg.show_downloads and function(...)\n"
"         io.write(\".\")\n"
"         io.flush()\n"
"         dots = dots + 1\n"
"         if dots == 70 then\n"
"            io.write(\"\\n\")\n"
"            dots = 0\n"
"         end\n"
"         return ltn12.pump.step(...)\n"
"      end,\n"
"      headers = {\n"
"         [\"user-agent\"] = cfg.user_agent..\" via LuaSocket\"\n"
"      },\n"
"   }\n"
"   if cfg.show_downloads then\n"
"      io.write(\"\\n\")\n"
"   end\n"
"   if not res then\n"
"      return nil, status\n"
"   elseif status == 301 or status == 302 then\n"
"      local location = headers.location\n"
"      if location then\n"
"         local protocol, rest = dir.split_url(location)\n"
"         if redirect_protocols[protocol] then\n"
"            if not loop_control then\n"
"               loop_control = {}\n"
"            elseif loop_control[location] then\n"
"               return nil, \"Redirection loop -- broken URL\?\"\n"
"            end\n"
"            loop_control[url] = true\n"
"            return request(location, method, redirect_protocols[protocol], loop_control)\n"
"         else\n"
"            return nil, \"URL redirected to unsupported protocol - install luasec to get HTTPS support.\", \"https\"\n"
"         end\n"
"      end\n"
"      return nil, err\n"
"   elseif status ~= 200 then\n"
"      return nil, err\n"
"   else\n"
"      return result, status, headers, err\n"
"   end\n"
"end\n"
"\n"
"local function write_timestamp(filename, data)\n"
"   local fd = io.open(filename, \"w\")\n"
"   if fd then\n"
"      fd:write(data)\n"
"      fd:close()\n"
"   end\n"
"end\n"
"\n"
"local function read_timestamp(filename)\n"
"   local fd = io.open(filename, \"r\")\n"
"   if fd then\n"
"      local data = fd:read(\"*a\")\n"
"      fd:close()\n"
"      return data\n"
"   end\n"
"end\n"
"\n"
"local function fail_with_status(filename, status, headers)\n"
"   write_timestamp(filename .. \".unixtime\", os.time())\n"
"   write_timestamp(filename .. \".status\", status)\n"
"   return nil, status, headers\n"
"end\n"
"\n"
"-- @param url string: URL to fetch.\n"
"-- @param filename string: local filename of the file to fetch.\n"
"-- @param http table: The library to use (http from LuaSocket or LuaSec)\n"
"-- @param cache boolean: Whether to use a `.timestamp` file to check\n"
"-- via the HTTP Last-Modified header if the full download is needed.\n"
"-- @return (boolean | (nil, string, string\?)): True if successful, or\n"
"-- nil, error message and optionally HTTPS error in case of errors.\n"
"local function http_request(url, filename, http, cache)\n"
"   if cache then\n"
"      local status = read_timestamp(filename..\".status\")\n"
"      local timestamp = read_timestamp(filename..\".timestamp\")\n"
"      if status or timestamp then\n"
"         local unixtime = read_timestamp(filename..\".unixtime\")\n"
"         if unixtime then\n"
"            local diff = os.time() - tonumber(unixtime)\n"
"            if status then\n"
"               if diff < cfg.cache_fail_timeout then\n"
"                  return nil, status, {}\n"
"               end\n"
"            else\n"
"               if diff < cfg.cache_timeout then\n"
"                  return true, nil, nil, true\n"
"               end\n"
"            end\n"
"         end\n"
"\n"
"         local result, status, headers, err = request(url, \"HEAD\", http)\n"
"         if not result then\n"
"            return fail_with_status(filename, status, headers)\n"
"         end\n"
"         if status == 200 and headers[\"last-modified\"] == timestamp then\n"
"            write_timestamp(filename .. \".unixtime\", os.time())\n"
"            return true, nil, nil, true\n"
"         end\n"
"      end\n"
"   end\n"
"   local result, status, headers, err = request(url, \"GET\", http)\n"
"   if not result then\n"
"      if status then\n"
"         return fail_with_status(filename, status, headers)\n"
"      end\n"
"   end\n"
"   if cache and headers[\"last-modified\"] then\n"
"      write_timestamp(filename .. \".timestamp\", headers[\"last-modified\"])\n"
"      write_timestamp(filename .. \".unixtime\", os.time())\n"
"   end\n"
"   local file = io.open(filename, \"wb\")\n"
"   if not file then return nil, 0, {} end\n"
"   for _, data in ipairs(result) do\n"
"      file:write(data)\n"
"   end\n"
"   file:close()\n"
"   return true\n"
"end\n"
"\n"
"local function ftp_request(url, filename)\n"
"   local content, err = ftp.get(url)\n"
"   if not content then\n"
"      return false, err\n"
"   end\n"
"   local file = io.open(filename, \"wb\")\n"
"   if not file then return false, err end\n"
"   file:write(content)\n"
"   file:close()\n"
"   return true\n"
"end\n"
"\n"
"local downloader_warning = false\n"
"\n"
"--- Download a remote file.\n"
"-- @param url string: URL to be fetched.\n"
"-- @param filename string or nil: this function attempts to detect the\n"
"-- resulting local filename of the remote file as the basename of the URL;\n"
"-- if that is not correct (due to a redirection, for example), the local\n"
"-- filename can be given explicitly as this second argument.\n"
"-- @return (boolean, string, boolean):\n"
"-- In case of success:\n"
"-- * true\n"
"-- * a string with the filename\n"
"-- * true if the file was retrieved from local cache\n"
"-- In case of failure:\n"
"-- * false\n"
"-- * error message\n"
"function fs_lua.download(url, filename, cache)\n"
"   assert(type(url) == \"string\")\n"
"   assert(type(filename) == \"string\" or not filename)\n"
"\n"
"   filename = fs.absolute_name(filename or dir.base_name(url))\n"
"\n"
"   -- delegate to the configured downloader so we don't have to deal with whitelists\n"
"   if os.getenv(\"no_proxy\") then\n"
"      return fs.use_downloader(url, filename, cache)\n"
"   end\n"
"\n"
"   local ok, err, https_err, from_cache\n"
"   if util.starts_with(url, \"http:\") then\n"
"      ok, err, https_err, from_cache = http_request(url, filename, http, cache)\n"
"   elseif util.starts_with(url, \"ftp:\") then\n"
"      ok, err = ftp_request(url, filename)\n"
"   elseif util.starts_with(url, \"https:\") then\n"
"      -- skip LuaSec when proxy is enabled since it is not supported\n"
"      if luasec_ok and not os.getenv(\"https_proxy\") then\n"
"         local _\n"
"         ok, err, _, from_cache = http_request(url, filename, https, cache)\n"
"      else\n"
"         https_err = true\n"
"      end\n"
"   else\n"
"      err = \"Unsupported protocol\"\n"
"   end\n"
"   if https_err then\n"
"      if not downloader_warning then\n"
"         local downloader = fs.which_tool(\"downloader\")\n"
"         util.warning(\"falling back to \"..downloader..\" - install luasec to get native HTTPS support\")\n"
"         downloader_warning = true\n"
"      end\n"
"      return fs.use_downloader(url, filename, cache)\n"
"   elseif not ok then\n"
"      return nil, err\n"
"   end\n"
"   return true, filename, from_cache\n"
"end\n"
"\n"
"else --...if socket_ok == false then\n"
"\n"
"function fs_lua.download(url, filename, cache)\n"
"   return fs.use_downloader(url, filename, cache)\n"
"end\n"
"\n"
"end\n"
"---------------------------------------------------------------------\n"
"-- MD5 functions\n"
"---------------------------------------------------------------------\n"
"\n"
"local md5_hex = nil\n"
"\n"
"if digest_ok then\n"
"   md5_hex = digest.md5_hex\n"
"elseif md5_ok then\n"
"   md5_hex = md5.sumhexa\n"
"   --- Support the interface of lmd5 by lhf in addition to md5 by Roberto\n"
"   --- and the keplerproject.\n"
"   if not md5_hex and md5.digest then\n"
"      md5_hex = md5.digest\n"
"   end\n"
"end\n"
"\n"
"if md5_hex then\n"
"\n"
"--- Get the MD5 checksum for a file.\n"
"-- @param file string: The file to be computed.\n"
"-- @return string: The MD5 checksum or nil + error\n"
"function fs_lua.get_md5(file)\n"
"   file = fs.absolute_name(file)\n"
"   local file_handler = io.open(file, \"rb\")\n"
"   if not file_handler then return nil, \"Failed to open file for reading: \"..file end\n"
"   local computed = md5_hex(file_handler:read(\"*a\"))\n"
"   file_handler:close()\n"
"   if computed then return computed end\n"
"   return nil, \"Failed to compute MD5 hash for file \"..file\n"
"end\n"
"\n"
"end\n"
"\n"
"---------------------------------------------------------------------\n"
"-- POSIX functions\n"
"---------------------------------------------------------------------\n"
"\n"
"function fs_lua._unix_rwx_to_number(rwx, neg)\n"
"   local num = 0\n"
"   neg = neg or false\n"
"   for i = 1, 9 do\n"
"      local c = rwx:sub(10 - i, 10 - i) == \"-\"\n"
"      if neg == c then\n"
"         num = num + 2^(i-1)\n"
"      end\n"
"   end\n"
"   return math.floor(num)\n"
"end\n"
"\n"
"if posix_ok then\n"
"\n"
"local octal_to_rwx = {\n"
"   [\"0\"] = \"---\",\n"
"   [\"1\"] = \"--x\",\n"
"   [\"2\"] = \"-w-\",\n"
"   [\"3\"] = \"-wx\",\n"
"   [\"4\"] = \"r--\",\n"
"   [\"5\"] = \"r-x\",\n"
"   [\"6\"] = \"rw-\",\n"
"   [\"7\"] = \"rwx\",\n"
"}\n"
"\n"
"do\n"
"   local umask_cache\n"
"   function fs_lua._unix_umask()\n"
"      if umask_cache then\n"
"         return umask_cache\n"
"      end\n"
"      -- LuaPosix (as of 34.0.4) only returns the umask as rwx\n"
"      local rwx = posix.umask()\n"
"      local num = fs_lua._unix_rwx_to_number(rwx, true)\n"
"      umask_cache = (\"%03o\"):format(num)\n"
"      return umask_cache\n"
"   end\n"
"end\n"
"\n"
"function fs_lua.set_permissions(filename, mode, scope)\n"
"   local perms\n"
"   if mode == \"read\" and scope == \"user\" then\n"
"      perms = fs._unix_moderate_permissions(\"600\")\n"
"   elseif mode == \"exec\" and scope == \"user\" then\n"
"      perms = fs._unix_moderate_permissions(\"700\")\n"
"   elseif mode == \"read\" and scope == \"all\" then\n"
"      perms = fs._unix_moderate_permissions(\"644\")\n"
"   elseif mode == \"exec\" and scope == \"all\" then\n"
"      perms = fs._unix_moderate_permissions(\"755\")\n"
"   else\n"
"      return false, \"Invalid permission \" .. mode .. \" for \" .. scope\n"
"   end\n"
"\n"
"   -- LuaPosix (as of 5.1.15) does not support octal notation...\n"
"   local new_perms = {}\n"
"   for c in perms:sub(-3):gmatch(\".\") do\n"
"      table.insert(new_perms, octal_to_rwx[c])\n"
"   end\n"
"   perms = table.concat(new_perms)\n"
"   local err = posix.chmod(filename, perms)\n"
"   return err == 0\n"
"end\n"
"\n"
"function fs_lua.current_user()\n"
"   return posix.getpwuid(posix.geteuid()).pw_name\n"
"end\n"
"\n"
"-- This call is not available on all systems, see #677\n"
"if posix.mkdtemp then\n"
"\n"
"--- Create a temporary directory.\n"
"-- @param name_pattern string: name pattern to use for avoiding conflicts\n"
"-- when creating temporary directory.\n"
"-- @return string or (nil, string): name of temporary directory or (nil, error message) on failure.\n"
"function fs_lua.make_temp_dir(name_pattern)\n"
"   assert(type(name_pattern) == \"string\")\n"
"   name_pattern = dir.normalize(name_pattern)\n"
"\n"
"   return posix.mkdtemp(fs.system_temp_dir() .. \"/luarocks_\" .. name_pattern:gsub(\"/\", \"_\") .. \"-XXXXXX\")\n"
"end\n"
"\n"
"end -- if posix.mkdtemp\n"
"\n"
"end\n"
"\n"
"---------------------------------------------------------------------\n"
"-- Other functions\n"
"---------------------------------------------------------------------\n"
"\n"
"if lfs_ok and not fs_lua.make_temp_dir then\n"
"\n"
"function fs_lua.make_temp_dir(name_pattern)\n"
"   assert(type(name_pattern) == \"string\")\n"
"   name_pattern = dir.normalize(name_pattern)\n"
"\n"
"   local pattern = fs.system_temp_dir() .. \"/luarocks_\" .. name_pattern:gsub(\"/\", \"_\") .. \"-\"\n"
"\n"
"   while true do\n"
"      local name = pattern .. tostring(math.random(10000000))\n"
"      if lfs.mkdir(name) then\n"
"         return name\n"
"      end\n"
"   end\n"
"end\n"
"\n"
"end\n"
"\n"
"--- Apply a patch.\n"
"-- @param patchname string: The filename of the patch.\n"
"-- @param patchdata string or nil: The actual patch as a string.\n"
"-- @param create_delete boolean: Support creating and deleting files in a patch.\n"
"function fs_lua.apply_patch(patchname, patchdata, create_delete)\n"
"   local p, all_ok = patch.read_patch(patchname, patchdata)\n"
"   if not all_ok then\n"
"      return nil, \"Failed reading patch \"..patchname\n"
"   end\n"
"   if p then\n"
"      return patch.apply_patch(p, 1, create_delete)\n"
"   end\n"
"end\n"
"\n"
"--- Move a file.\n"
"-- @param src string: Pathname of source\n"
"-- @param dest string: Pathname of destination\n"
"-- @param perms string (\"read\" or \"exec\") or nil: Permissions for destination\n"
"-- file or nil to use the source file permissions.\n"
"-- @return boolean or (boolean, string): true on success, false on failure,\n"
"-- plus an error message.\n"
"function fs_lua.move(src, dest, perms)\n"
"   assert(src and dest)\n"
"   if fs.exists(dest) and not fs.is_dir(dest) then\n"
"      return false, \"File already exists: \"..dest\n"
"   end\n"
"   local ok, err = fs.copy(src, dest, perms)\n"
"   if not ok then\n"
"      return false, err\n"
"   end\n"
"   fs.delete(src)\n"
"   if fs.exists(src) then\n"
"      return false, \"Failed move: could not delete \"..src..\" after copy.\"\n"
"   end\n"
"   return true\n"
"end\n"
"\n"
"--- Check if user has write permissions for the command.\n"
"-- Assumes the configuration variables under cfg have been previously set up.\n"
"-- @param flags table: the flags table passed to run() drivers.\n"
"-- @return boolean or (boolean, string): true on success, false on failure,\n"
"-- plus an error message.\n"
"function fs_lua.check_command_permissions(flags)\n"
"   local ok = true\n"
"   local err = \"\"\n"
"   for _, directory in ipairs { cfg.rocks_dir, cfg.deploy_lua_dir, cfg.deploy_bin_dir, cfg.deploy_lua_dir } do\n"
"      if fs.exists(directory) then\n"
"         if not fs.is_writable(directory) then\n"
"            ok = false\n"
"            err = \"Your user does not have write permissions in \" .. directory\n"
"            break\n"
"         end\n"
"      else\n"
"         local root = fs.root_of(directory)\n"
"         local parent = directory\n"
"         repeat\n"
"            parent = dir.dir_name(parent)\n"
"            if parent == \"\" then\n"
"               parent = root\n"
"            end\n"
"         until parent == root or fs.exists(parent)\n"
"         if not fs.is_writable(parent) then\n"
"            ok = false\n"
"            err = directory..\" does not exist and your user does not have write permissions in \" .. parent\n"
"            break\n"
"         end\n"
"      end\n"
"   end\n"
"   if ok then\n"
"      return true\n"
"   else\n"
"      if flags[\"local\"] or cfg.local_by_default then\n"
"         err = err .. \" \\n-- please check your permissions.\"\n"
"      else\n"
"         err = err .. \" \\n-- you may want to run as a privileged user or use your local tree with --local.\"\n"
"      end\n"
"      return nil, err\n"
"   end\n"
"end\n"
"\n"
"--- Check whether a file is a Lua script\n"
"-- When the file can be successfully compiled by the configured\n"
"-- Lua interpreter, it's considered to be a valid Lua file.\n"
"-- @param filename filename of file to check\n"
"-- @return boolean true, if it is a Lua script, false otherwise\n"
"function fs_lua.is_lua(filename)\n"
"  filename = filename:gsub([[%\\]],\"/\")   -- normalize on fw slash to prevent escaping issues\n"
"  local lua = fs.Q(dir.path(cfg.variables[\"LUA_BINDIR\"], cfg.lua_interpreter))  -- get lua interpreter configured\n"
"  -- execute on configured interpreter, might not be the same as the interpreter LR is run on\n"
"  local result = fs.execute_string(lua..[[ -e \"if loadfile(']]..filename..[[') then os.exit(0) else os.exit(1) end\"]])\n"
"  return (result == true)\n"
"end\n"
"\n"
"--- Unpack an archive.\n"
"-- Extract the contents of an archive, detecting its format by\n"
"-- filename extension.\n"
"-- @param archive string: Filename of archive.\n"
"-- @return boolean or (boolean, string): true on success, false and an error message on failure.\n"
"function fs_lua.unpack_archive(archive)\n"
"   assert(type(archive) == \"string\")\n"
"\n"
"   local ok, err\n"
"   archive = fs.absolute_name(archive)\n"
"   if archive:match(\"%.tar%.gz$\") then\n"
"      local tar_filename = archive:gsub(\"%.gz$\", \"\")\n"
"      ok, err = fs.gunzip(archive, tar_filename)\n"
"      if ok then\n"
"         ok, err = tar.untar(tar_filename, \".\")\n"
"      end\n"
"   elseif archive:match(\"%.tgz$\") then\n"
"      local tar_filename = archive:gsub(\"%.tgz$\", \".tar\")\n"
"      ok, err = fs.gunzip(archive, tar_filename)\n"
"      if ok then\n"
"         ok, err = tar.untar(tar_filename, \".\")\n"
"      end\n"
"   elseif archive:match(\"%.tar%.bz2$\") then\n"
"      local tar_filename = archive:gsub(\"%.bz2$\", \"\")\n"
"      ok, err = fs.bunzip2(archive, tar_filename)\n"
"      if ok then\n"
"         ok, err = tar.untar(tar_filename, \".\")\n"
"      end\n"
"   elseif archive:match(\"%.zip$\") then\n"
"      ok, err = fs.unzip(archive)\n"
"   elseif archive:match(\"%.lua$\") or archive:match(\"%.c$\") then\n"
"      -- Ignore .lua and .c files; they don't need to be extracted.\n"
"      return true\n"
"   else\n"
"      return false, \"Couldn't extract archive \"..archive..\": unrecognized filename extension\"\n"
"   end\n"
"   if not ok then\n"
"      return false, \"Failed extracting \"..archive..\": \"..err\n"
"   end\n"
"   return true\n"
"end\n"
"\n"
"return fs_lua\n"
""
;
