const char luarocks_fs_unix_lua[] =
"\n"
"--- Unix implementation of filesystem and platform abstractions.\n"
"local unix = {}\n"
"\n"
"local fs = require(\"luarocks.fs\")\n"
"\n"
"local cfg = require(\"luarocks.core.cfg\")\n"
"local dir = require(\"luarocks.dir\")\n"
"local path = require(\"luarocks.path\")\n"
"local util = require(\"luarocks.util\")\n"
"\n"
"--- Annotate command string for quiet execution.\n"
"-- @param cmd string: A command-line string.\n"
"-- @return string: The command-line, with silencing annotation.\n"
"function unix.quiet(cmd)\n"
"   return cmd..\" 1> /dev/null 2> /dev/null\"\n"
"end\n"
"\n"
"--- Annotate command string for execution with quiet stderr.\n"
"-- @param cmd string: A command-line string.\n"
"-- @return string: The command-line, with stderr silencing annotation.\n"
"function unix.quiet_stderr(cmd)\n"
"   return cmd..\" 2> /dev/null\"\n"
"end\n"
"\n"
"--- Quote argument for shell processing.\n"
"-- Adds single quotes and escapes.\n"
"-- @param arg string: Unquoted argument.\n"
"-- @return string: Quoted argument.\n"
"function unix.Q(arg)\n"
"   assert(type(arg) == \"string\")\n"
"   return \"'\" .. arg:gsub(\"'\", \"'\\\\''\") .. \"'\"\n"
"end\n"
"\n"
"--- Return an absolute pathname from a potentially relative one.\n"
"-- @param pathname string: pathname to convert.\n"
"-- @param relative_to string or nil: path to prepend when making\n"
"-- pathname absolute, or the current dir in the dir stack if\n"
"-- not given.\n"
"-- @return string: The pathname converted to absolute.\n"
"function unix.absolute_name(pathname, relative_to)\n"
"   assert(type(pathname) == \"string\")\n"
"   assert(type(relative_to) == \"string\" or not relative_to)\n"
"\n"
"   relative_to = relative_to or fs.current_dir()\n"
"   if pathname:sub(1,1) == \"/\" then\n"
"      return pathname\n"
"   else\n"
"      return relative_to .. \"/\" .. pathname\n"
"   end\n"
"end\n"
"\n"
"--- Return the root directory for the given path.\n"
"-- In Unix, root is always \"/\".\n"
"-- @param pathname string: pathname to use.\n"
"-- @return string: The root of the given pathname.\n"
"function unix.root_of(_)\n"
"   return \"/\"\n"
"end\n"
"\n"
"--- Create a wrapper to make a script executable from the command-line.\n"
"-- @param script string: Pathname of script to be made executable.\n"
"-- @param target string: wrapper target pathname (without wrapper suffix).\n"
"-- @param name string: rock name to be used in loader context.\n"
"-- @param version string: rock version to be used in loader context.\n"
"-- @return boolean or (nil, string): True if succeeded, or nil and\n"
"-- an error message.\n"
"function unix.wrap_script(script, target, deps_mode, name, version, ...)\n"
"   assert(type(script) == \"string\" or not script)\n"
"   assert(type(target) == \"string\")\n"
"   assert(type(deps_mode) == \"string\")\n"
"   assert(type(name) == \"string\" or not name)\n"
"   assert(type(version) == \"string\" or not version)\n"
"   \n"
"   local wrapper = io.open(target, \"w\")\n"
"   if not wrapper then\n"
"      return nil, \"Could not open \"..target..\" for writing.\"\n"
"   end\n"
"\n"
"   local lpath, lcpath = path.package_paths(deps_mode)\n"
"\n"
"   local luainit = {\n"
"      \"package.path=\"..util.LQ(lpath..\";\")..\"..package.path\",\n"
"      \"package.cpath=\"..util.LQ(lcpath..\";\")..\"..package.cpath\",\n"
"   }\n"
"   if target == \"luarocks\" or target == \"luarocks-admin\" then\n"
"      luainit = {\n"
"         \"package.path=\"..util.LQ(package.path),\n"
"         \"package.cpath=\"..util.LQ(package.cpath),\n"
"      }\n"
"   end\n"
"   if name and version then\n"
"      local addctx = \"local k,l,_=pcall(require,\"..util.LQ(\"luarocks.loader\")..\") _=k \" ..\n"
"                     \"and l.add_context(\"..util.LQ(name)..\",\"..util.LQ(version)..\")\"\n"
"      table.insert(luainit, addctx)\n"
"   end\n"
"\n"
"   local argv = {\n"
"      fs.Q(dir.path(cfg.variables[\"LUA_BINDIR\"], cfg.lua_interpreter)),\n"
"      \"-e\",\n"
"      fs.Q(table.concat(luainit, \";\")),\n"
"      script and fs.Q(script) or \"\",\n"
"      ...\n"
"   }\n"
"\n"
"   wrapper:write(\"#!/bin/sh\\n\\n\")\n"
"   wrapper:write(\"LUAROCKS_SYSCONFDIR=\"..fs.Q(cfg.sysconfdir) .. \" \")\n"
"   wrapper:write(\"exec \"..table.concat(argv, \" \")..' \"$@\"\\n')\n"
"   wrapper:close()\n"
"\n"
"   if fs.set_permissions(target, \"exec\", \"all\") then\n"
"      return true\n"
"   else\n"
"      return nil, \"Could not make \"..target..\" executable.\"\n"
"   end\n"
"end\n"
"\n"
"--- Check if a file (typically inside path.bin_dir) is an actual binary\n"
"-- or a Lua wrapper.\n"
"-- @param filename string: the file name with full path.\n"
"-- @return boolean: returns true if file is an actual binary\n"
"-- (or if it couldn't check) or false if it is a Lua wrapper.\n"
"function unix.is_actual_binary(filename)\n"
"   if filename:match(\"%.lua$\") then\n"
"      return false\n"
"   end\n"
"   local file = io.open(filename)\n"
"   if not file then\n"
"      return true\n"
"   end\n"
"   local first = file:read(2)\n"
"   file:close()\n"
"   if not first then\n"
"      util.warning(\"could not read \"..filename)\n"
"      return true\n"
"   end\n"
"   return first ~= \"#!\"\n"
"end\n"
"\n"
"function unix.copy_binary(filename, dest) \n"
"   return fs.copy(filename, dest, \"exec\")\n"
"end\n"
"\n"
"--- Move a file on top of the other.\n"
"-- The new file ceases to exist under its original name,\n"
"-- and takes over the name of the old file.\n"
"-- On Unix this is done through a single rename operation.\n"
"-- @param old_file The name of the original file,\n"
"-- which will be the new name of new_file.\n"
"-- @param new_file The name of the new file,\n"
"-- which will replace old_file.\n"
"-- @return boolean or (nil, string): True if succeeded, or nil and\n"
"-- an error message.\n"
"function unix.replace_file(old_file, new_file)\n"
"   return os.rename(new_file, old_file)\n"
"end\n"
"\n"
"function unix.tmpname()\n"
"   return os.tmpname()\n"
"end\n"
"\n"
"function unix.current_user()\n"
"   return os.getenv(\"USER\")\n"
"end\n"
"\n"
"function unix.export_cmd(var, val)\n"
"   return (\"export %s='%s'\"):format(var, val)\n"
"end\n"
"\n"
"local octal_to_rwx = {\n"
"   [\"0\"] = \"---\",\n"
"   [\"1\"] = \"--x\",\n"
"   [\"2\"] = \"-w-\",\n"
"   [\"3\"] = \"-wx\",\n"
"   [\"4\"] = \"r--\",\n"
"   [\"5\"] = \"r-x\",\n"
"   [\"6\"] = \"rw-\",\n"
"   [\"7\"] = \"rwx\",\n"
"}\n"
"local rwx_to_octal = {}\n"
"for octal, rwx in pairs(octal_to_rwx) do\n"
"   rwx_to_octal[rwx] = octal\n"
"end\n"
"--- Moderate the given permissions based on the local umask\n"
"-- @param perms string: permissions to moderate\n"
"-- @return string: the moderated permissions\n"
"function unix._unix_moderate_permissions(perms)\n"
"   local umask = fs._unix_umask()\n"
"\n"
"   local moderated_perms = \"\"\n"
"   for i = 1, 3 do\n"
"      local p_rwx = octal_to_rwx[perms:sub(i, i)]\n"
"      local u_rwx = octal_to_rwx[umask:sub(i, i)]\n"
"      local new_perm = \"\"\n"
"      for j = 1, 3 do\n"
"         local p_val = p_rwx:sub(j, j)\n"
"         local u_val = u_rwx:sub(j, j)\n"
"         if p_val == u_val then\n"
"            new_perm = new_perm .. \"-\"\n"
"         else\n"
"            new_perm = new_perm .. p_val\n"
"         end\n"
"      end\n"
"      moderated_perms = moderated_perms .. rwx_to_octal[new_perm]\n"
"   end\n"
"   return moderated_perms\n"
"end\n"
"\n"
"function unix.is_dir(file)\n"
"   file = fs.absolute_name(file)\n"
"   file = dir.normalize(file) .. \"/\"\n"
"   local fd, _, code = io.open(file, \"r\")\n"
"   if code == 2 then -- \"No such file or directory\"\n"
"      return false\n"
"   end\n"
"   if code == 20 then -- \"Not a directory\", regardless of permissions\n"
"      return false\n"
"   end\n"
"   if code == 13 then -- \"Permission denied\", but is a directory\n"
"      return true\n"
"   end\n"
"   if fd then\n"
"      fd:close()\n"
"      return true\n"
"   end\n"
"   return false\n"
"end\n"
"\n"
"function unix.is_file(file)\n"
"   file = fs.absolute_name(file)\n"
"   if fs.is_dir(file) then\n"
"      return false\n"
"   end\n"
"   file = dir.normalize(file)\n"
"   local fd, _, code = io.open(file, \"r\")\n"
"   if code == 2 then -- \"No such file or directory\"\n"
"      return false\n"
"   end\n"
"   if code == 13 then -- \"Permission denied\", but it exists\n"
"      return true\n"
"   end\n"
"   if fd then\n"
"      fd:close()\n"
"      return true\n"
"   end\n"
"   return false\n"
"end\n"
"\n"
"function unix.system_cache_dir()\n"
"   if fs.is_dir(\"/var/cache\") then\n"
"      return \"/var/cache\"\n"
"   end\n"
"   return dir.path(fs.system_temp_dir(), \"cache\")\n"
"end\n"
"\n"
"return unix\n"
""
;
