const char luarocks_fs_unix_tools_lua[] =
"\n"
"--- fs operations implemented with third-party tools for Unix platform abstractions.\n"
"local tools = {}\n"
"\n"
"local fs = require(\"luarocks.fs\")\n"
"local dir = require(\"luarocks.dir\")\n"
"local cfg = require(\"luarocks.core.cfg\")\n"
"\n"
"local vars = setmetatable({}, { __index = function(_,k) return cfg.variables[k] end })\n"
"\n"
"--- Adds prefix to command to make it run from a directory.\n"
"-- @param directory string: Path to a directory.\n"
"-- @param cmd string: A command-line string.\n"
"-- @return string: The command-line with prefix.\n"
"function tools.command_at(directory, cmd)\n"
"   return \"cd \" .. fs.Q(fs.absolute_name(directory)) .. \" && \" .. cmd\n"
"end\n"
"\n"
"--- Create a directory if it does not already exist.\n"
"-- If any of the higher levels in the path name does not exist\n"
"-- too, they are created as well.\n"
"-- @param directory string: pathname of directory to create.\n"
"-- @return boolean: true on success, false on failure.\n"
"function tools.make_dir(directory)\n"
"   assert(directory)\n"
"   local ok, err = fs.execute(vars.MKDIR..\" -p\", directory)\n"
"   if not ok then\n"
"      err = \"failed making directory \"..directory\n"
"   end\n"
"   return ok, err\n"
"end\n"
"\n"
"--- Remove a directory if it is empty.\n"
"-- Does not return errors (for example, if directory is not empty or\n"
"-- if already does not exist)\n"
"-- @param directory string: pathname of directory to remove.\n"
"function tools.remove_dir_if_empty(directory)\n"
"   assert(directory)\n"
"   fs.execute_quiet(vars.RMDIR, directory)\n"
"end\n"
"\n"
"--- Remove a directory if it is empty.\n"
"-- Does not return errors (for example, if directory is not empty or\n"
"-- if already does not exist)\n"
"-- @param directory string: pathname of directory to remove.\n"
"function tools.remove_dir_tree_if_empty(directory)\n"
"   assert(directory)\n"
"   fs.execute_quiet(vars.RMDIR, \"-p\", directory)\n"
"end\n"
"\n"
"--- Copy a file.\n"
"-- @param src string: Pathname of source\n"
"-- @param dest string: Pathname of destination\n"
"-- @param perm string (\"read\" or \"exec\") or nil: Permissions for destination \n"
"-- file or nil to use the source permissions\n"
"-- @return boolean or (boolean, string): true on success, false on failure,\n"
"-- plus an error message.\n"
"function tools.copy(src, dest, perm)\n"
"   assert(src and dest)\n"
"   if fs.execute(vars.CP, src, dest) then\n"
"      if perm then\n"
"         if fs.is_dir(dest) then\n"
"            dest = dir.path(dest, dir.base_name(src))\n"
"         end\n"
"         if fs.set_permissions(dest, perm, \"all\") then\n"
"            return true\n"
"         else\n"
"            return false, \"Failed setting permissions of \"..dest\n"
"         end\n"
"      end\n"
"      return true\n"
"   else\n"
"      return false, \"Failed copying \"..src..\" to \"..dest\n"
"   end\n"
"end\n"
"\n"
"--- Recursively copy the contents of a directory.\n"
"-- @param src string: Pathname of source\n"
"-- @param dest string: Pathname of destination\n"
"-- @return boolean or (boolean, string): true on success, false on failure,\n"
"-- plus an error message.\n"
"function tools.copy_contents(src, dest)\n"
"   assert(src and dest)\n"
"   if fs.execute_quiet(vars.CP..\" -pPR \"..fs.Q(src)..\"/* \"..fs.Q(dest)) then\n"
"      return true\n"
"   else\n"
"      return false, \"Failed copying \"..src..\" to \"..dest\n"
"   end\n"
"end\n"
"--- Delete a file or a directory and all its contents.\n"
"-- For safety, this only accepts absolute paths.\n"
"-- @param arg string: Pathname of source\n"
"-- @return nil\n"
"function tools.delete(arg)\n"
"   assert(arg)\n"
"   assert(arg:sub(1,1) == \"/\")\n"
"   fs.execute_quiet(vars.RM, \"-rf\", arg)\n"
"end\n"
"\n"
"--- Recursively scan the contents of a directory.\n"
"-- @param at string or nil: directory to scan (will be the current\n"
"-- directory if none is given).\n"
"-- @return table: an array of strings with the filenames representing\n"
"-- the contents of a directory.\n"
"function tools.find(at)\n"
"   assert(type(at) == \"string\" or not at)\n"
"   if not at then\n"
"      at = fs.current_dir()\n"
"   end\n"
"   if not fs.is_dir(at) then\n"
"      return {}\n"
"   end\n"
"   local result = {}\n"
"   local pipe = io.popen(fs.command_at(at, fs.quiet_stderr(vars.FIND..\" *\")))\n"
"   for file in pipe:lines() do\n"
"      table.insert(result, file)\n"
"   end\n"
"   pipe:close()\n"
"   return result\n"
"end\n"
"\n"
"--- Compress files in a .zip archive.\n"
"-- @param zipfile string: pathname of .zip archive to be created.\n"
"-- @param ... Filenames to be stored in the archive are given as\n"
"-- additional arguments.\n"
"-- @return boolean: true on success, nil and error message on failure.\n"
"function tools.zip(zipfile, ...)\n"
"   local ok, err = fs.is_tool_available(vars.ZIP, \"zip\")\n"
"   if not ok then\n"
"      return nil, err\n"
"   end\n"
"   if fs.execute_quiet(vars.ZIP..\" -r\", zipfile, ...) then\n"
"      return true\n"
"   else\n"
"      return nil, \"failed compressing \" .. zipfile\n"
"   end\n"
"end\n"
"\n"
"--- Uncompress files from a .zip archive.\n"
"-- @param zipfile string: pathname of .zip archive to be extracted.\n"
"-- @return boolean: true on success, nil and error message on failure.\n"
"function tools.unzip(zipfile)\n"
"   assert(zipfile)\n"
"   local ok, err = fs.is_tool_available(vars.UNZIP, \"unzip\", \"-h\")\n"
"   if not ok then\n"
"      return nil, err\n"
"   end\n"
"   if fs.execute_quiet(vars.UNZIP, zipfile) then\n"
"      return true\n"
"   else\n"
"      return nil, \"failed extracting \" .. zipfile\n"
"   end\n"
"end\n"
"\n"
"local function uncompress(default_ext, program, infile, outfile)\n"
"   assert(type(infile) == \"string\")\n"
"   assert(outfile == nil or type(outfile) == \"string\")\n"
"   if not outfile then\n"
"      outfile = infile:gsub(\"%.\"..default_ext..\"$\", \"\")\n"
"   end\n"
"   if fs.execute(fs.Q(program)..\" -c \"..fs.Q(infile)..\" > \"..fs.Q(outfile)) then\n"
"      return true\n"
"   else\n"
"      return nil, \"failed extracting \" .. infile\n"
"   end\n"
"end\n"
"\n"
"--- Uncompresses a .gz file.\n"
"-- @param infile string: pathname of .gz file to be extracted.\n"
"-- @param outfile string or nil: pathname of output file to be produced.\n"
"-- If not given, name is derived from input file.\n"
"-- @return boolean: true on success; nil and error message on failure.\n"
"function tools.gunzip(infile, outfile)\n"
"   return uncompress(\"gz\", \"gunzip\", infile, outfile)\n"
"end\n"
"\n"
"--- Uncompresses a .bz2 file.\n"
"-- @param infile string: pathname of .bz2 file to be extracted.\n"
"-- @param outfile string or nil: pathname of output file to be produced.\n"
"-- If not given, name is derived from input file.\n"
"-- @return boolean: true on success; nil and error message on failure.\n"
"function tools.bunzip2(infile, outfile)\n"
"   return uncompress(\"bz2\", \"bunzip2\", infile, outfile)\n"
"end\n"
"\n"
"do\n"
"   local function rwx_to_octal(rwx)\n"
"      return (rwx:match \"r\" and 4 or 0)\n"
"         + (rwx:match \"w\" and 2 or 0)\n"
"         + (rwx:match \"x\" and 1 or 0)\n"
"   end\n"
"   local umask_cache\n"
"   function tools._unix_umask()\n"
"      if umask_cache then\n"
"         return umask_cache\n"
"      end\n"
"      local fd = assert(io.popen(\"umask -S\"))\n"
"      local umask = assert(fd:read(\"*a\"))\n"
"      fd:close()\n"
"      local u, g, o = umask:match(\"u=([rwx]*),g=([rwx]*),o=([rwx]*)\")\n"
"      if not u then\n"
"         error(\"invalid umask result\")\n"
"      end\n"
"      umask_cache = string.format(\"%d%d%d\",\n"
"         7 - rwx_to_octal(u),\n"
"         7 - rwx_to_octal(g),\n"
"         7 - rwx_to_octal(o))\n"
"      return umask_cache\n"
"   end\n"
"end\n"
"\n"
"--- Set permissions for file or directory\n"
"-- @param filename string: filename whose permissions are to be modified\n"
"-- @param mode string (\"read\" or \"exec\"): permissions to set\n"
"-- @param scope string (\"user\" or \"all\"): the user(s) to whom the permission applies\n"
"-- @return boolean or (boolean, string): true on success, false on failure,\n"
"-- plus an error message\n"
"function tools.set_permissions(filename, mode, scope)\n"
"   assert(filename and mode and scope)\n"
"\n"
"   local perms\n"
"   if mode == \"read\" and scope == \"user\" then\n"
"      perms = fs._unix_moderate_permissions(\"600\")\n"
"   elseif mode == \"exec\" and scope == \"user\" then\n"
"      perms = fs._unix_moderate_permissions(\"700\")\n"
"   elseif mode == \"read\" and scope == \"all\" then\n"
"      perms = fs._unix_moderate_permissions(\"644\")\n"
"   elseif mode == \"exec\" and scope == \"all\" then\n"
"      perms = fs._unix_moderate_permissions(\"755\")\n"
"   else\n"
"      return false, \"Invalid permission \" .. mode .. \" for \" .. scope\n"
"   end\n"
"   return fs.execute(vars.CHMOD, perms, filename)\n"
"end\n"
"\n"
"function tools.browser(url)\n"
"   return fs.execute(cfg.web_browser, url)\n"
"end\n"
"\n"
"-- Set access and modification times for a file.\n"
"-- @param filename File to set access and modification times for.\n"
"-- @param time may be a string or number containing the format returned\n"
"-- by os.time, or a table ready to be processed via os.time; if\n"
"-- nil, current time is assumed.\n"
"function tools.set_time(file, time)\n"
"   assert(time == nil or type(time) == \"table\" or type(time) == \"number\")\n"
"   file = dir.normalize(file)\n"
"   local flag = \"\"\n"
"   if type(time) == \"number\" then\n"
"      time = os.date(\"*t\", time)\n"
"   end\n"
"   if type(time) == \"table\" then\n"
"      flag = (\"-t %04d%02d%02d%02d%02d.%02d\"):format(time.year, time.month, time.day, time.hour, time.min, time.sec)\n"
"   end\n"
"   return fs.execute(vars.TOUCH .. \" \" .. flag, file)\n"
"end\n"
"\n"
"--- Create a temporary directory.\n"
"-- @param name_pattern string: name pattern to use for avoiding conflicts\n"
"-- when creating temporary directory.\n"
"-- @return string or (nil, string): name of temporary directory or (nil, error message) on failure.\n"
"function tools.make_temp_dir(name_pattern)\n"
"   assert(type(name_pattern) == \"string\")\n"
"   name_pattern = dir.normalize(name_pattern)\n"
"\n"
"   local template = (os.getenv(\"TMPDIR\") or \"/tmp\") .. \"/luarocks_\" .. name_pattern:gsub(\"/\", \"_\") .. \"-XXXXXX\"\n"
"   local pipe = io.popen(vars.MKTEMP..\" -d \"..fs.Q(template))\n"
"   local dirname = pipe:read(\"*l\")\n"
"   pipe:close()\n"
"   if dirname and dirname:match(\"^/\") then\n"
"      return dirname\n"
"   end\n"
"   return nil, \"Failed to create temporary directory \"..tostring(dirname)\n"
"end\n"
"\n"
"return tools\n"
""
;
