const char luarocks_loader_lua[] =
"\n"
"--- A module which installs a Lua package loader that is LuaRocks-aware.\n"
"-- This loader uses dependency information from the LuaRocks tree to load\n"
"-- correct versions of modules. It does this by constructing a \"context\"\n"
"-- table in the environment, which records which versions of packages were\n"
"-- used to load previous modules, so that the loader chooses versions\n"
"-- that are declared to be compatible with the ones loaded earlier.\n"
"local loaders = package.loaders or package.searchers\n"
"local require, ipairs, table, type, next, tostring, error =\n"
"      require, ipairs, table, type, next, tostring, error\n"
"local unpack = unpack or table.unpack\n"
"\n"
"local loader = {}\n"
"\n"
"local is_clean = not package.loaded[\"luarocks.core.cfg\"]\n"
"\n"
"-- This loader module depends only on core modules.\n"
"local cfg = require(\"luarocks.core.cfg\")\n"
"local cfg_ok, err = cfg.init()\n"
"if cfg_ok then\n"
"   cfg.init_package_paths()\n"
"end\n"
"\n"
"local path = require(\"luarocks.core.path\")\n"
"local manif = require(\"luarocks.core.manif\")\n"
"local vers = require(\"luarocks.core.vers\")\n"
"local util = require(\"luarocks.core.util\")\n"
"local require = nil\n"
"--------------------------------------------------------------------------------\n"
"\n"
"local temporary_global = false\n"
"\n"
"loader.context = {}\n"
"\n"
"-- Contains a table when rocks trees are loaded,\n"
"-- or 'false' to indicate rocks trees failed to load.\n"
"-- 'nil' indicates rocks trees were not attempted to be loaded yet.\n"
"loader.rocks_trees = nil\n"
"\n"
"function loader.load_rocks_trees()\n"
"   local any_ok = false\n"
"   local trees = {}\n"
"   for _, tree in ipairs(cfg.rocks_trees or {}) do\n"
"      local manifest, err = manif.fast_load_local_manifest(path.rocks_dir(tree))\n"
"      if manifest then\n"
"         any_ok = true\n"
"         table.insert(trees, {tree=tree, manifest=manifest})\n"
"      end\n"
"   end\n"
"   if not any_ok then\n"
"      loader.rocks_trees = false\n"
"      return false\n"
"   end\n"
"   loader.rocks_trees = trees\n"
"   return true\n"
"end\n"
"\n"
"--- Process the dependencies of a package to determine its dependency\n"
"-- chain for loading modules.\n"
"-- @param name string: The name of an installed rock.\n"
"-- @param version string: The version of the rock, in string format\n"
"function loader.add_context(name, version)\n"
"   -- assert(type(name) == \"string\")\n"
"   -- assert(type(version) == \"string\")\n"
"\n"
"   if temporary_global then\n"
"      -- The first thing a wrapper does is to call add_context.\n"
"      -- From here on, it's safe to clean the global environment.\n"
"      luarocks = nil\n"
"      temporary_global = false\n"
"   end\n"
"\n"
"   if loader.context[name] then\n"
"      return\n"
"   end\n"
"   loader.context[name] = version\n"
"\n"
"   if not loader.rocks_trees and not loader.load_rocks_trees() then\n"
"      return nil\n"
"   end\n"
"\n"
"   for _, tree in ipairs(loader.rocks_trees) do\n"
"      local manifest = tree.manifest\n"
"\n"
"      local pkgdeps\n"
"      if manifest.dependencies and manifest.dependencies[name] then\n"
"         pkgdeps = manifest.dependencies[name][version]\n"
"      end\n"
"      if not pkgdeps then\n"
"         return nil\n"
"      end\n"
"      for _, dep in ipairs(pkgdeps) do\n"
"         local pkg, constraints = dep.name, dep.constraints\n"
"   \n"
"         for _, tree in ipairs(loader.rocks_trees) do\n"
"            local entries = tree.manifest.repository[pkg]\n"
"            if entries then\n"
"               for ver, pkgs in util.sortedpairs(entries, vers.compare_versions) do\n"
"                  if (not constraints) or vers.match_constraints(vers.parse_version(ver), constraints) then\n"
"                     loader.add_context(pkg, version)\n"
"                  end\n"
"               end\n"
"            end\n"
"         end\n"
"      end\n"
"   end\n"
"end\n"
"\n"
"--- Internal sorting function.\n"
"-- @param a table: A provider table.\n"
"-- @param b table: Another provider table.\n"
"-- @return boolean: True if the version of a is greater than that of b.\n"
"local function sort_versions(a,b)\n"
"   return a.version > b.version\n"
"end\n"
"\n"
"--- Request module to be loaded through other loaders,\n"
"-- once the proper name of the module has been determined.\n"
"-- For example, in case the module \"socket.core\" has been requested\n"
"-- to the LuaRocks loader and it determined based on context that\n"
"-- the version 2.0.2 needs to be loaded and it is not the current\n"
"-- version, the module requested for the other loaders will be\n"
"-- \"socket.core_2_0_2\".\n"
"-- @param module The module name requested by the user, such as \"socket.core\"\n"
"-- @param name The rock name, such as \"luasocket\"\n"
"-- @param version The rock version, such as \"2.0.2-1\"\n"
"-- @param module_name The actual module name, such as \"socket.core\" or \"socket.core_2_0_2\".\n"
"-- @return table or (nil, string): The module table as returned by some other loader,\n"
"-- or nil followed by an error message if no other loader managed to load the module.\n"
"local function call_other_loaders(module, name, version, module_name)\n"
"   for _, a_loader in ipairs(loaders) do\n"
"      if a_loader ~= loader.luarocks_loader then\n"
"         local results = { a_loader(module_name) }\n"
"         if type(results[1]) == \"function\" then\n"
"            return unpack(results)\n"
"         end\n"
"      end\n"
"   end\n"
"   return \"Failed loading module \"..module..\" in LuaRocks rock \"..name..\" \"..version\n"
"end\n"
"\n"
"local function add_providers(providers, entries, tree, module, filter_file_name)\n"
"   for i, entry in ipairs(entries) do\n"
"      local name, version = entry:match(\"^([^/]*)/(.*)$\")\n"
"      local file_name = tree.manifest.repository[name][version][1].modules[module]\n"
"      if type(file_name) ~= \"string\" then\n"
"         error(\"Invalid data in manifest file for module \"..tostring(module)..\" (invalid data for \"..tostring(name)..\" \"..tostring(version)..\")\")\n"
"      end\n"
"      file_name = filter_file_name(file_name, name, version, tree.tree, i)\n"
"      if loader.context[name] == version then\n"
"         return name, version, file_name\n"
"      end\n"
"      version = vers.parse_version(version)\n"
"      table.insert(providers, {name = name, version = version, module_name = file_name, tree = tree})\n"
"   end\n"
"end\n"
"\n"
"--- Search for a module in the rocks trees\n"
"-- @param module string: module name (eg. \"socket.core\")\n"
"-- @param filter_file_name function(string, string, string, string, number):\n"
"-- a function that takes the module file name (eg \"socket/core.so\"), the rock name\n"
"-- (eg \"luasocket\"), the version (eg \"2.0.2-1\"), the path of the rocks tree\n"
"-- (eg \"/usr/local\"), and the numeric index of the matching entry, so the\n"
"-- filter function can know if the matching module was the first entry or not.\n"
"-- @return string, string, string, (string or table):\n"
"-- * name of the rock containing the module (eg. \"luasocket\")\n"
"-- * version of the rock (eg. \"2.0.2-1\")\n"
"-- * return value of filter_file_name\n"
"-- * tree of the module (string or table in `rocks_trees` format)\n"
"local function select_module(module, filter_file_name)\n"
"   --assert(type(module) == \"string\")\n"
"   --assert(type(filter_module_name) == \"function\")\n"
"\n"
"   if not loader.rocks_trees and not loader.load_rocks_trees() then\n"
"      return nil\n"
"   end\n"
"\n"
"   local providers = {}\n"
"   local initmodule\n"
"   for _, tree in ipairs(loader.rocks_trees) do\n"
"      local entries = tree.manifest.modules[module]\n"
"      if entries then\n"
"         add_providers(providers, entries, tree, module, filter_file_name)\n"
"      else\n"
"         initmodule = initmodule or module .. \".init\"\n"
"         entries = tree.manifest.modules[initmodule]\n"
"         if entries then\n"
"            add_providers(providers, entries, tree, initmodule, filter_file_name)\n"
"         end\n"
"      end\n"
"   end\n"
"\n"
"   if next(providers) then\n"
"      table.sort(providers, sort_versions)\n"
"      local first = providers[1]\n"
"      return first.name, first.version.string, first.module_name, first.tree\n"
"   end\n"
"end\n"
"\n"
"--- Search for a module\n"
"-- @param module string: module name (eg. \"socket.core\")\n"
"-- @return string, string, string, (string or table):\n"
"-- * name of the rock containing the module (eg. \"luasocket\")\n"
"-- * version of the rock (eg. \"2.0.2-1\")\n"
"-- * name of the module (eg. \"socket.core\", or \"socket.core_2_0_2\" if file is stored versioned).\n"
"-- * tree of the module (string or table in `rocks_trees` format)\n"
"local function pick_module(module)\n"
"   return\n"
"      select_module(module, function(file_name, name, version, tree, i)\n"
"         if i > 1 then\n"
"            file_name = path.versioned_name(file_name, \"\", name, version)\n"
"         end\n"
"         return path.path_to_module(file_name)\n"
"      end)\n"
"end\n"
"\n"
"--- Return the pathname of the file that would be loaded for a module.\n"
"-- @param module string: module name (eg. \"socket.core\")\n"
"-- @return filename of the module (eg. \"/usr/local/lib/lua/5.1/socket/core.so\"),\n"
"-- the rock name and the rock version.\n"
"function loader.which(module)\n"
"   local rock_name, rock_version, file_name = select_module(module, path.which_i)\n"
"   return file_name, rock_name, rock_version\n"
"end\n"
"\n"
"--- Package loader for LuaRocks support.\n"
"-- A module is searched in installed rocks that match the\n"
"-- current LuaRocks context. If module is not part of the\n"
"-- context, or if a context has not yet been set, the module\n"
"-- in the package with the highest version is used.\n"
"-- @param module string: The module name, like in plain require().\n"
"-- @return table: The module table (typically), like in plain\n"
"-- require(). See <a href=\"http://www.lua.org/manual/5.1/manual.html#pdf-require\">require()</a>\n"
"-- in the Lua reference manual for details.\n"
"function loader.luarocks_loader(module)\n"
"   local name, version, module_name = pick_module(module)\n"
"   if not name then\n"
"      return \"No LuaRocks module found for \"..module\n"
"   else\n"
"      loader.add_context(name, version)\n"
"      return call_other_loaders(module, name, version, module_name)\n"
"   end\n"
"end\n"
"\n"
"table.insert(loaders, 1, loader.luarocks_loader)\n"
"\n"
"if is_clean then\n"
"   for modname, _ in pairs(package.loaded) do\n"
"      if modname:match(\"^luarocks%.\") then\n"
"         package.loaded[modname] = nil\n"
"      end\n"
"   end\n"
"end\n"
"\n"
"return loader\n"
""
;
