const char luarocks_manif_lua[] =
"--- Module for handling manifest files and tables.\n"
"-- Manifest files describe the contents of a LuaRocks tree or server.\n"
"-- They are loaded into manifest tables, which are then used for\n"
"-- performing searches, matching dependencies, etc.\n"
"local manif = {}\n"
"\n"
"local core = require(\"luarocks.core.manif\")\n"
"local persist = require(\"luarocks.persist\")\n"
"local fetch = require(\"luarocks.fetch\")\n"
"local dir = require(\"luarocks.dir\")\n"
"local fs = require(\"luarocks.fs\")\n"
"local cfg = require(\"luarocks.core.cfg\")\n"
"local path = require(\"luarocks.path\")\n"
"local util = require(\"luarocks.util\")\n"
"local queries = require(\"luarocks.queries\")\n"
"local type_manifest = require(\"luarocks.type.manifest\")\n"
"\n"
"manif.cache_manifest = core.cache_manifest\n"
"\n"
"manif.rock_manifest_cache = {}\n"
"\n"
"local function check_manifest(repo_url, manifest, globals)\n"
"   local ok, err = type_manifest.check(manifest, globals)\n"
"   if not ok then\n"
"      core.cache_manifest(repo_url, cfg.lua_version, nil)\n"
"      return nil, \"Error checking manifest: \"..err, \"type\"\n"
"   end\n"
"   return manifest\n"
"end\n"
"\n"
"local postprocess_dependencies\n"
"do\n"
"   local postprocess_check = setmetatable({}, { __mode = \"k\" })\n"
"   postprocess_dependencies = function(manifest)\n"
"      if postprocess_check[manifest] then\n"
"         return\n"
"      end\n"
"      if manifest.dependencies then\n"
"         for name, versions in pairs(manifest.dependencies) do\n"
"            for version, entries in pairs(versions) do\n"
"               for k, v in pairs(entries) do\n"
"                  entries[k] = queries.from_persisted_table(v)\n"
"               end\n"
"            end\n"
"         end\n"
"      end\n"
"      postprocess_check[manifest] = true\n"
"   end\n"
"end\n"
"\n"
"function manif.load_rock_manifest(name, version, root)\n"
"   assert(type(name) == \"string\" and not name:match(\"/\"))\n"
"   assert(type(version) == \"string\")\n"
"\n"
"   local name_version = name..\"/\"..version\n"
"   if manif.rock_manifest_cache[name_version] then\n"
"      return manif.rock_manifest_cache[name_version].rock_manifest\n"
"   end\n"
"   local pathname = path.rock_manifest_file(name, version, root)\n"
"   local rock_manifest = persist.load_into_table(pathname)\n"
"   if not rock_manifest then\n"
"      return nil, \"rock_manifest file not found for \"..name..\" \"..version..\" - not a LuaRocks tree\?\"\n"
"   end\n"
"   manif.rock_manifest_cache[name_version] = rock_manifest\n"
"   return rock_manifest.rock_manifest\n"
"end\n"
"\n"
"\n"
"local function fetch_manifest_from(repo_url, filename)\n"
"   local url = dir.path(repo_url, filename)\n"
"   local name = repo_url:gsub(\"[/:]\",\"_\")\n"
"   local cache_dir = dir.path(cfg.local_cache, name)\n"
"   local ok = fs.make_dir(cache_dir)\n"
"   if not ok then\n"
"      cfg.local_cache = fs.make_temp_dir(\"local_cache\")\n"
"      cache_dir = dir.path(cfg.local_cache, name)\n"
"      ok = fs.make_dir(cache_dir)\n"
"      if not ok then\n"
"         return nil, \"Failed creating temporary cache directory \"..cache_dir\n"
"      end\n"
"   end\n"
"   local file, err, errcode, from_cache = fetch.fetch_url(url, dir.path(cache_dir, filename), true)\n"
"   if not file then\n"
"      return nil, \"Failed fetching manifest for \"..repo_url..(err and \" - \"..err or \"\"), errcode\n"
"   end\n"
"   return file, nil, nil, from_cache\n"
"end\n"
"\n"
"--- Load a local or remote manifest describing a repository.\n"
"-- All functions that use manifest tables assume they were obtained\n"
"-- through this function.\n"
"-- @param repo_url string: URL or pathname for the repository.\n"
"-- @param lua_version string: Lua version in \"5.x\" format, defaults to installed version.\n"
"-- @return table or (nil, string, [string]): A table representing the manifest,\n"
"-- or nil followed by an error message and an optional error code.\n"
"function manif.load_manifest(repo_url, lua_version)\n"
"   assert(type(repo_url) == \"string\")\n"
"   assert(type(lua_version) == \"string\" or not lua_version)\n"
"   lua_version = lua_version or cfg.lua_version\n"
"\n"
"   local cached_manifest = core.get_cached_manifest(repo_url, lua_version)\n"
"   if cached_manifest then\n"
"      postprocess_dependencies(cached_manifest)\n"
"      return cached_manifest\n"
"   end\n"
"\n"
"   local filenames = {\n"
"      \"manifest-\"..lua_version..\".zip\",\n"
"      \"manifest-\"..lua_version,\n"
"      \"manifest\",\n"
"   }\n"
"\n"
"   local protocol, repodir = dir.split_url(repo_url)\n"
"   local pathname, from_cache\n"
"   if protocol == \"file\" then\n"
"      for _, filename in ipairs(filenames) do\n"
"         pathname = dir.path(repodir, filename)\n"
"         if fs.exists(pathname) then\n"
"            break\n"
"         end\n"
"      end\n"
"   else\n"
"      local err, errcode\n"
"      for _, filename in ipairs(filenames) do\n"
"         pathname, err, errcode, from_cache = fetch_manifest_from(repo_url, filename)\n"
"         if pathname then\n"
"            break\n"
"         end\n"
"      end\n"
"      if not pathname then \n"
"         return nil, err, errcode\n"
"      end\n"
"   end\n"
"   if pathname:match(\".*%.zip$\") then\n"
"      pathname = fs.absolute_name(pathname)\n"
"      local dirname = dir.dir_name(pathname)\n"
"      fs.change_dir(dirname)\n"
"      local nozip = pathname:match(\"(.*)%.zip$\")\n"
"      if not from_cache then\n"
"         fs.delete(nozip)\n"
"         local ok, err = fs.unzip(pathname)\n"
"         fs.pop_dir()\n"
"         if not ok then\n"
"            fs.delete(pathname)\n"
"            fs.delete(pathname..\".timestamp\")\n"
"            return nil, \"Failed extracting manifest file: \" .. err\n"
"         end\n"
"      end\n"
"      pathname = nozip\n"
"   end\n"
"   local manifest, err, errcode = core.manifest_loader(pathname, repo_url, lua_version)\n"
"   if not manifest then\n"
"      return nil, err, errcode\n"
"   end\n"
"\n"
"   postprocess_dependencies(manifest)\n"
"   return check_manifest(repo_url, manifest, err)\n"
"end\n"
"\n"
"--- Get type and name of an item (a module or a command) provided by a file.\n"
"-- @param deploy_type string: rock manifest subtree the file comes from (\"bin\", \"lua\", or \"lib\").\n"
"-- @param file_path string: path to the file relatively to deploy_type subdirectory.\n"
"-- @return (string, string): item type (\"module\" or \"command\") and name.\n"
"function manif.get_provided_item(deploy_type, file_path)\n"
"   assert(type(deploy_type) == \"string\")\n"
"   assert(type(file_path) == \"string\")\n"
"   local item_type = deploy_type == \"bin\" and \"command\" or \"module\"\n"
"   local item_name = item_type == \"command\" and file_path or path.path_to_module(file_path)\n"
"   return item_type, item_name\n"
"end\n"
"\n"
"local function get_providers(item_type, item_name, repo)\n"
"   assert(type(item_type) == \"string\")\n"
"   assert(type(item_name) == \"string\")\n"
"   local rocks_dir = path.rocks_dir(repo or cfg.root_dir)\n"
"   local manifest = manif.load_manifest(rocks_dir)\n"
"   return manifest and manifest[item_type .. \"s\"][item_name]\n"
"end\n"
"\n"
"--- Given a name of a module or a command, figure out which rock name and version\n"
"-- correspond to it in the rock tree manifest.\n"
"-- @param item_type string: \"module\" or \"command\".\n"
"-- @param item_name string: module or command name.\n"
"-- @param root string or nil: A local root dir for a rocks tree. If not given, the default is used.\n"
"-- @return (string, string) or nil: name and version of the provider rock or nil if there\n"
"-- is no provider.\n"
"function manif.get_current_provider(item_type, item_name, repo)\n"
"   local providers = get_providers(item_type, item_name, repo)\n"
"   if providers then\n"
"      return providers[1]:match(\"([^/]*)/([^/]*)\")\n"
"   end\n"
"end\n"
"\n"
"function manif.get_next_provider(item_type, item_name, repo)\n"
"   local providers = get_providers(item_type, item_name, repo)\n"
"   if providers and providers[2] then\n"
"      return providers[2]:match(\"([^/]*)/([^/]*)\")\n"
"   end\n"
"end\n"
"\n"
"--- Given a name of a module or a command provided by a package, figure out\n"
"-- which file provides it.\n"
"-- @param name string: package name.\n"
"-- @param version string: package version.\n"
"-- @param item_type string: \"module\" or \"command\".\n"
"-- @param item_name string: module or command name.\n"
"-- @param root string or nil: A local root dir for a rocks tree. If not given, the default is used.\n"
"-- @return (string, string): rock manifest subtree the file comes from (\"bin\", \"lua\", or \"lib\")\n"
"-- and path to the providing file relatively to that subtree.\n"
"function manif.get_providing_file(name, version, item_type, item_name, repo)\n"
"   local rocks_dir = path.rocks_dir(repo or cfg.root_dir)\n"
"   local manifest = manif.load_manifest(rocks_dir)\n"
"\n"
"   local entry_table = manifest.repository[name][version][1]\n"
"   local file_path = entry_table[item_type .. \"s\"][item_name]\n"
"\n"
"   if item_type == \"command\" then\n"
"      return \"bin\", file_path\n"
"   end\n"
"\n"
"   -- A module can be in \"lua\" or \"lib\". Decide based on extension first:\n"
"   -- most likely Lua modules are in \"lua/\" and C modules are in \"lib/\".\n"
"   if file_path:match(\"%.\" .. cfg.lua_extension .. \"$\") then\n"
"      return \"lua\", file_path\n"
"   elseif file_path:match(\"%.\" .. cfg.lib_extension .. \"$\") then\n"
"      return \"lib\", file_path\n"
"   end\n"
"\n"
"   -- Fallback to rock manifest scanning.\n"
"   local rock_manifest = manif.load_rock_manifest(name, version, repo and repo.root)\n"
"   local subtree = rock_manifest.lib\n"
"\n"
"   for path_part in file_path:gmatch(\"[^/]+\") do\n"
"      if type(subtree) == \"table\" then\n"
"         subtree = subtree[path_part]\n"
"      else\n"
"         -- Assume it's in \"lua/\" if it's not in \"lib/\".\n"
"         return \"lua\", file_path\n"
"      end\n"
"   end\n"
"\n"
"   return type(subtree) == \"string\" and \"lib\" or \"lua\", file_path\n"
"end\n"
"\n"
"--- Get all versions of a package listed in a manifest file.\n"
"-- @param name string: a package name.\n"
"-- @param deps_mode string: \"one\", to use only the currently\n"
"-- configured tree; \"order\" to select trees based on order\n"
"-- (use the current tree and all trees below it on the list)\n"
"-- or \"all\", to use all trees.\n"
"-- @return table: An array of strings listing installed\n"
"-- versions of a package, and a table indicating where they are found.\n"
"function manif.get_versions(dep, deps_mode)\n"
"   assert(type(dep) == \"table\")\n"
"   assert(type(deps_mode) == \"string\")\n"
"   \n"
"   local name = dep.name\n"
"   local namespace = dep.namespace\n"
"\n"
"   local version_set = {}\n"
"   path.map_trees(deps_mode, function(tree)\n"
"      local manifest = manif.load_manifest(path.rocks_dir(tree))\n"
"\n"
"      if manifest and manifest.repository[name] then\n"
"         for version in pairs(manifest.repository[name]) do\n"
"            if dep.namespace then\n"
"               local ns_file = path.rock_namespace_file(name, version, tree)\n"
"               local fd = io.open(ns_file, \"r\")\n"
"               if fd then\n"
"                  local ns = fd:read(\"*a\")\n"
"                  fd:close()\n"
"                  if ns == namespace then\n"
"                     version_set[version] = tree\n"
"                  end\n"
"               end\n"
"            else\n"
"               version_set[version] = tree\n"
"            end\n"
"         end\n"
"      end\n"
"   end)\n"
"\n"
"   return util.keys(version_set), version_set\n"
"end\n"
"\n"
"return manif\n"
""
;
