const char luarocks_pack_lua[] =
"\n"
"-- Create rock files, packing sources or binaries.\n"
"local pack = {}\n"
"\n"
"local unpack = unpack or table.unpack\n"
"\n"
"local queries = require(\"luarocks.queries\")\n"
"local path = require(\"luarocks.path\")\n"
"local repos = require(\"luarocks.repos\")\n"
"local fetch = require(\"luarocks.fetch\")\n"
"local fs = require(\"luarocks.fs\")\n"
"local cfg = require(\"luarocks.core.cfg\")\n"
"local util = require(\"luarocks.util\")\n"
"local dir = require(\"luarocks.dir\")\n"
"local manif = require(\"luarocks.manif\")\n"
"local search = require(\"luarocks.search\")\n"
"local signing = require(\"luarocks.signing\")\n"
"\n"
"--- Create a source rock.\n"
"-- Packages a rockspec and its required source files in a rock\n"
"-- file with the .src.rock extension, which can later be built and\n"
"-- installed with the \"build\" command.\n"
"-- @param rockspec_file string: An URL or pathname for a rockspec file.\n"
"-- @return string or (nil, string): The filename of the resulting\n"
"-- .src.rock file; or nil and an error message.\n"
"function pack.pack_source_rock(rockspec_file)\n"
"   assert(type(rockspec_file) == \"string\")\n"
"\n"
"   local rockspec, err = fetch.load_rockspec(rockspec_file)\n"
"   if err then\n"
"      return nil, \"Error loading rockspec: \"..err\n"
"   end\n"
"   rockspec_file = rockspec.local_abs_filename\n"
"\n"
"   local name_version = rockspec.name .. \"-\" .. rockspec.version\n"
"   local rock_file = fs.absolute_name(name_version .. \".src.rock\")\n"
"\n"
"   local source_file, source_dir = fetch.fetch_sources(rockspec, false)\n"
"   if not source_file then\n"
"      return nil, source_dir\n"
"   end\n"
"   local ok, err = fs.change_dir(source_dir)\n"
"   if not ok then return nil, err end\n"
"\n"
"   fs.delete(rock_file)\n"
"   fs.copy(rockspec_file, source_dir, \"read\")\n"
"   ok, err = fs.zip(rock_file, dir.base_name(rockspec_file), dir.base_name(source_file))\n"
"   if not ok then\n"
"      return nil, \"Failed packing \"..rock_file..\" - \"..err\n"
"   end\n"
"   fs.pop_dir()\n"
"\n"
"   return rock_file\n"
"end\n"
"\n"
"local function copy_back_files(name, version, file_tree, deploy_dir, pack_dir, perms)\n"
"   local ok, err = fs.make_dir(pack_dir)\n"
"   if not ok then return nil, err end\n"
"   for file, sub in pairs(file_tree) do\n"
"      local source = dir.path(deploy_dir, file)\n"
"      local target = dir.path(pack_dir, file)\n"
"      if type(sub) == \"table\" then\n"
"         local ok, err = copy_back_files(name, version, sub, source, target)\n"
"         if not ok then return nil, err end\n"
"      else\n"
"         local versioned = path.versioned_name(source, deploy_dir, name, version)\n"
"         if fs.exists(versioned) then\n"
"            fs.copy(versioned, target, perms)\n"
"         else\n"
"            fs.copy(source, target, perms)\n"
"         end\n"
"      end\n"
"   end\n"
"   return true\n"
"end\n"
"\n"
"-- @param name string: Name of package to pack.\n"
"-- @param version string or nil: A version number may also be passed.\n"
"-- @param tree string or nil: An optional tree to pick the package from.\n"
"-- @return string or (nil, string): The filename of the resulting\n"
"-- .src.rock file; or nil and an error message.\n"
"function pack.pack_installed_rock(query, tree)\n"
"\n"
"   local name, version, repo, repo_url = search.pick_installed_rock(query, tree)\n"
"   if not name then\n"
"      return nil, version\n"
"   end\n"
"\n"
"   local root = path.root_from_rocks_dir(repo_url)\n"
"   local prefix = path.install_dir(name, version, root)\n"
"   if not fs.exists(prefix) then\n"
"      return nil, \"'\"..name..\" \"..version..\"' does not seem to be an installed rock.\"\n"
"   end\n"
"   \n"
"   local rock_manifest, err = manif.load_rock_manifest(name, version, root)\n"
"   if not rock_manifest then return nil, err end\n"
"\n"
"   local name_version = name .. \"-\" .. version\n"
"   local rock_file = fs.absolute_name(name_version .. \".\"..cfg.arch..\".rock\")\n"
"   \n"
"   local temp_dir = fs.make_temp_dir(\"pack\")\n"
"   fs.copy_contents(prefix, temp_dir)\n"
"\n"
"   local is_binary = false\n"
"   if rock_manifest.lib then\n"
"      local ok, err = copy_back_files(name, version, rock_manifest.lib, path.deploy_lib_dir(repo), dir.path(temp_dir, \"lib\"), \"exec\")\n"
"      if not ok then return nil, \"Failed copying back files: \" .. err end\n"
"      is_binary = true\n"
"   end\n"
"   if rock_manifest.lua then\n"
"      local ok, err = copy_back_files(name, version, rock_manifest.lua, path.deploy_lua_dir(repo), dir.path(temp_dir, \"lua\"), \"read\")\n"
"      if not ok then return nil, \"Failed copying back files: \" .. err end\n"
"   end\n"
"   \n"
"   local ok, err = fs.change_dir(temp_dir)\n"
"   if not ok then return nil, err end\n"
"   if not is_binary and not repos.has_binaries(name, version) then\n"
"      rock_file = rock_file:gsub(\"%.\"..cfg.arch:gsub(\"%-\",\"%%-\")..\"%.\", \".all.\")\n"
"   end\n"
"   fs.delete(rock_file)\n"
"   if not fs.zip(rock_file, unpack(fs.list_dir())) then\n"
"      return nil, \"Failed packing \"..rock_file\n"
"   end\n"
"   fs.pop_dir()\n"
"   fs.delete(temp_dir)\n"
"   return rock_file\n"
"end\n"
"\n"
"function pack.report_and_sign_local_file(file, err, sign)\n"
"   if err then\n"
"      return nil, err\n"
"   end\n"
"   local sigfile\n"
"   if sign then\n"
"      sigfile, err = signing.sign_file(file)\n"
"      util.printout()\n"
"   end\n"
"   util.printout(\"Packed: \"..file)\n"
"   if sigfile then\n"
"      util.printout(\"Sigature stored in: \"..sigfile)\n"
"   end\n"
"   if err then\n"
"      return nil, err\n"
"   end\n"
"   return true\n"
"end\n"
"\n"
"function pack.pack_binary_rock(name, version, sign, cmd)\n"
"\n"
"   -- The --pack-binary-rock option for \"luarocks build\" basically performs\n"
"   -- \"luarocks build\" on a temporary tree and then \"luarocks pack\". The\n"
"   -- alternative would require refactoring parts of luarocks.build and\n"
"   -- luarocks.pack, which would save a few file operations: the idea would be\n"
"   -- to shave off the final deploy steps from the build phase and the initial\n"
"   -- collect steps from the pack phase.\n"
"\n"
"   local temp_dir, err = fs.make_temp_dir(\"luarocks-build-pack-\"..dir.base_name(name))\n"
"   if not temp_dir then\n"
"      return nil, \"Failed creating temporary directory: \"..err\n"
"   end\n"
"   util.schedule_function(fs.delete, temp_dir)\n"
"\n"
"   path.use_tree(temp_dir)\n"
"   local ok, err = cmd()\n"
"   if not ok then\n"
"      return nil, err\n"
"   end\n"
"   local rname, rversion = path.parse_name(name)\n"
"   if not rname then\n"
"      rname, rversion = name, version\n"
"   end\n"
"   local query = queries.new(rname, rversion)\n"
"   local file, err = pack.pack_installed_rock(query, temp_dir)\n"
"   return pack.report_and_sign_local_file(file, err, sign)\n"
"end\n"
"\n"
"return pack\n"
""
;
