const char luarocks_persist_lua[] =
"\n"
"--- Utility module for loading files into tables and\n"
"-- saving tables into files.\n"
"local persist = {}\n"
"\n"
"local core = require(\"luarocks.core.persist\")\n"
"local util = require(\"luarocks.util\")\n"
"local dir = require(\"luarocks.dir\")\n"
"local fs = require(\"luarocks.fs\")\n"
"\n"
"persist.load_into_table = core.load_into_table\n"
"\n"
"local write_table\n"
"\n"
"--- Write a value as Lua code.\n"
"-- This function handles only numbers and strings, invoking write_table\n"
"-- to write tables.\n"
"-- @param out table or userdata: a writer object supporting :write() method.\n"
"-- @param v: the value to be written.\n"
"-- @param level number: the indentation level\n"
"-- @param sub_order table: optional prioritization table\n"
"-- @see write_table\n"
"function persist.write_value(out, v, level, sub_order)\n"
"   if type(v) == \"table\" then\n"
"      level = level or 0\n"
"      write_table(out, v, level + 1, sub_order)\n"
"   elseif type(v) == \"string\" then\n"
"      if v:match(\"[\\r\\n]\") then\n"
"         local open, close = \"[[\", \"]]\"\n"
"         local equals = 0\n"
"         local v_with_bracket = v..\"]\"\n"
"         while v_with_bracket:find(close, 1, true) do\n"
"            equals = equals + 1\n"
"            local eqs = (\"=\"):rep(equals)\n"
"            open, close = \"[\"..eqs..\"[\", \"]\"..eqs..\"]\"\n"
"         end\n"
"         out:write(open..\"\\n\"..v..close)\n"
"      else\n"
"         out:write((\"%q\"):format(v))\n"
"      end\n"
"   else\n"
"      out:write(tostring(v))\n"
"   end\n"
"end\n"
"\n"
"local is_valid_plain_key\n"
"do\n"
"   local keywords = {\n"
"      [\"and\"] = true,\n"
"      [\"break\"] = true,\n"
"      [\"do\"] = true,\n"
"      [\"else\"] = true,\n"
"      [\"elseif\"] = true,\n"
"      [\"end\"] = true,\n"
"      [\"false\"] = true,\n"
"      [\"for\"] = true,\n"
"      [\"function\"] = true,\n"
"      [\"goto\"] = true,\n"
"      [\"if\"] = true,\n"
"      [\"in\"] = true,\n"
"      [\"local\"] = true,\n"
"      [\"nil\"] = true,\n"
"      [\"not\"] = true,\n"
"      [\"or\"] = true,\n"
"      [\"repeat\"] = true,\n"
"      [\"return\"] = true,\n"
"      [\"then\"] = true,\n"
"      [\"true\"] = true,\n"
"      [\"until\"] = true,\n"
"      [\"while\"] = true,\n"
"   }\n"
"   function is_valid_plain_key(k)\n"
"      return type(k) == \"string\"\n"
"             and k:match(\"^[a-zA-Z_][a-zA-Z0-9_]*$\")\n"
"             and not keywords[k]\n"
"   end\n"
"end\n"
"\n"
"local function write_table_key_assignment(out, k, level)\n"
"   if is_valid_plain_key(k) then\n"
"      out:write(k)\n"
"   else\n"
"      out:write(\"[\")\n"
"      persist.write_value(out, k, level)\n"
"      out:write(\"]\")\n"
"   end\n"
"   \n"
"   out:write(\" = \")\n"
"end\n"
"\n"
"--- Write a table as Lua code in curly brackets notation to a writer object.\n"
"-- Only numbers, strings and tables (containing numbers, strings\n"
"-- or other recursively processed tables) are supported.\n"
"-- @param out table or userdata: a writer object supporting :write() method.\n"
"-- @param tbl table: the table to be written.\n"
"-- @param level number: the indentation level\n"
"-- @param field_order table: optional prioritization table\n"
"write_table = function(out, tbl, level, field_order)\n"
"   out:write(\"{\")\n"
"   local sep = \"\\n\"\n"
"   local indentation = \"   \"\n"
"   local indent = true\n"
"   local i = 1\n"
"   for k, v, sub_order in util.sortedpairs(tbl, field_order) do\n"
"      out:write(sep)\n"
"      if indent then\n"
"         for _ = 1, level do out:write(indentation) end\n"
"      end\n"
"\n"
"      if k == i then\n"
"         i = i + 1\n"
"      else\n"
"         write_table_key_assignment(out, k, level)\n"
"      end\n"
"\n"
"      persist.write_value(out, v, level, sub_order)\n"
"      if type(v) == \"number\" then\n"
"         sep = \", \"\n"
"         indent = false\n"
"      else\n"
"         sep = \",\\n\"\n"
"         indent = true\n"
"      end\n"
"   end\n"
"   if sep ~= \"\\n\" then\n"
"      out:write(\"\\n\")\n"
"      for _ = 1, level - 1 do out:write(indentation) end\n"
"   end\n"
"   out:write(\"}\")\n"
"end\n"
"\n"
"--- Write a table as series of assignments to a writer object.\n"
"-- @param out table or userdata: a writer object supporting :write() method.\n"
"-- @param tbl table: the table to be written.\n"
"-- @param field_order table: optional prioritization table\n"
"-- @return true if successful; nil and error message if failed.\n"
"local function write_table_as_assignments(out, tbl, field_order)\n"
"   for k, v, sub_order in util.sortedpairs(tbl, field_order) do\n"
"      if not is_valid_plain_key(k) then\n"
"         return nil, \"cannot store '\"..tostring(k)..\"' as a plain key.\"\n"
"      end\n"
"      out:write(k..\" = \")\n"
"      persist.write_value(out, v, 0, sub_order)\n"
"      out:write(\"\\n\")\n"
"   end\n"
"   return true\n"
"end\n"
"\n"
"--- Write a table using Lua table syntax to a writer object.\n"
"-- @param out table or userdata: a writer object supporting :write() method.\n"
"-- @param tbl table: the table to be written.\n"
"local function write_table_as_table(out, tbl)\n"
"   out:write(\"return {\\n\")\n"
"   for k, v, sub_order in util.sortedpairs(tbl) do\n"
"      out:write(\"   \")\n"
"      write_table_key_assignment(out, k, 1)\n"
"      persist.write_value(out, v, 1, sub_order)\n"
"      out:write(\",\\n\")\n"
"   end\n"
"   out:write(\"}\\n\")\n"
"end\n"
"\n"
"--- Save the contents of a table to a string.\n"
"-- Each element of the table is saved as a global assignment.\n"
"-- Only numbers, strings and tables (containing numbers, strings\n"
"-- or other recursively processed tables) are supported.\n"
"-- @param tbl table: the table containing the data to be written\n"
"-- @param field_order table: an optional array indicating the order of top-level fields.\n"
"-- @return persisted data as string; or nil and an error message\n"
"function persist.save_from_table_to_string(tbl, field_order)\n"
"   local out = {buffer = {}}\n"
"   function out:write(data) table.insert(self.buffer, data) end\n"
"   local ok, err = write_table_as_assignments(out, tbl, field_order)\n"
"   if not ok then\n"
"      return nil, err\n"
"   end\n"
"   return table.concat(out.buffer)\n"
"end\n"
"\n"
"--- Save the contents of a table in a file.\n"
"-- Each element of the table is saved as a global assignment.\n"
"-- Only numbers, strings and tables (containing numbers, strings\n"
"-- or other recursively processed tables) are supported.\n"
"-- @param filename string: the output filename\n"
"-- @param tbl table: the table containing the data to be written\n"
"-- @param field_order table: an optional array indicating the order of top-level fields.\n"
"-- @return boolean or (nil, string): true if successful, or nil and a\n"
"-- message in case of errors.\n"
"function persist.save_from_table(filename, tbl, field_order)\n"
"   local out = io.open(filename, \"w\")\n"
"   if not out then\n"
"      return nil, \"Cannot create file at \"..filename\n"
"   end\n"
"   local ok, err = write_table_as_assignments(out, tbl, field_order)\n"
"   out:close()\n"
"   if not ok then\n"
"      return nil, err\n"
"   end\n"
"   return true\n"
"end\n"
"\n"
"--- Save the contents of a table as a module.\n"
"-- Each element of the table is saved as a global assignment.\n"
"-- Only numbers, strings and tables (containing numbers, strings\n"
"-- or other recursively processed tables) are supported.\n"
"-- @param filename string: the output filename\n"
"-- @param tbl table: the table containing the data to be written\n"
"-- @return boolean or (nil, string): true if successful, or nil and a\n"
"-- message in case of errors.\n"
"function persist.save_as_module(filename, tbl)\n"
"   local out = io.open(filename, \"w\")\n"
"   if not out then\n"
"      return nil, \"Cannot create file at \"..filename\n"
"   end\n"
"   write_table_as_table(out, tbl)\n"
"   out:close()\n"
"   return true\n"
"end\n"
"\n"
"function persist.load_config_file_if_basic(filename, cfg)\n"
"   local env = {\n"
"      home = cfg.home\n"
"   }\n"
"   local result, err, errcode = persist.load_into_table(filename, env)\n"
"   if errcode == \"load\" or errcode == \"run\" then\n"
"      -- bad config file or depends on env, so error out\n"
"      return nil, \"Could not read existing config file \" .. filename\n"
"   end\n"
"\n"
"   local tbl\n"
"   if errcode == \"open\" then\n"
"      -- could not open, maybe file does not exist\n"
"      tbl = {}\n"
"   else\n"
"      tbl = result\n"
"      tbl.home = nil\n"
"   end\n"
"\n"
"   return tbl\n"
"end\n"
"\n"
"function persist.save_default_lua_version(prefix, lua_version)\n"
"   local ok, err = fs.make_dir(prefix)\n"
"   if not ok then\n"
"      return nil, err\n"
"   end\n"
"   local fd, err = io.open(dir.path(prefix, \"default-lua-version.lua\"), \"w\")\n"
"   if not fd then\n"
"      return nil, err\n"
"   end\n"
"   fd:write('return \"' .. lua_version .. '\"\\n')\n"
"   fd:close()\n"
"   return true\n"
"end\n"
"\n"
"return persist\n"
""
;
