const char luarocks_repos_lua[] =
"\n"
"--- Functions for managing the repository on disk.\n"
"local repos = {}\n"
"\n"
"local fs = require(\"luarocks.fs\")\n"
"local path = require(\"luarocks.path\")\n"
"local cfg = require(\"luarocks.core.cfg\")\n"
"local util = require(\"luarocks.util\")\n"
"local dir = require(\"luarocks.dir\")\n"
"local manif = require(\"luarocks.manif\")\n"
"local vers = require(\"luarocks.core.vers\")\n"
"local E = {}\n"
"\n"
"local unpack = unpack or table.unpack\n"
"\n"
"--- Get type and name of an item (a module or a command) provided by a file.\n"
"-- @param deploy_type string: rock manifest subtree the file comes from (\"bin\", \"lua\", or \"lib\").\n"
"-- @param file_path string: path to the file relatively to deploy_type subdirectory.\n"
"-- @return (string, string): item type (\"module\" or \"command\") and name.\n"
"local function get_provided_item(deploy_type, file_path)\n"
"   assert(type(deploy_type) == \"string\")\n"
"   assert(type(file_path) == \"string\")\n"
"   local item_type = deploy_type == \"bin\" and \"command\" or \"module\"\n"
"   local item_name = item_type == \"command\" and file_path or path.path_to_module(file_path)\n"
"   return item_type, item_name\n"
"end\n"
"\n"
"-- Tree of files installed by a package are stored\n"
"-- in its rock manifest. Some of these files have to\n"
"-- be deployed to locations where Lua can load them as\n"
"-- modules or where they can be used as commands.\n"
"-- These files are characterised by pair\n"
"-- (deploy_type, file_path), where deploy_type is the first\n"
"-- component of the file path and file_path is the rest of the\n"
"-- path. Only files with deploy_type in {\"lua\", \"lib\", \"bin\"}\n"
"-- are deployed somewhere.\n"
"-- Each deployed file provides an \"item\". An item is\n"
"-- characterised by pair (item_type, item_name).\n"
"-- item_type is \"command\" for files with deploy_type\n"
"-- \"bin\" and \"module\" for deploy_type in {\"lua\", \"lib\"}.\n"
"-- item_name is same as file_path for commands\n"
"-- and is produced using path.path_to_module(file_path)\n"
"-- for modules.\n"
"\n"
"--- Get all installed versions of a package.\n"
"-- @param name string: a package name.\n"
"-- @return table or nil: An array of strings listing installed\n"
"-- versions of a package, or nil if none is available.\n"
"local function get_installed_versions(name)\n"
"   assert(type(name) == \"string\" and not name:match(\"/\"))\n"
"   \n"
"   local dirs = fs.list_dir(path.versions_dir(name))\n"
"   return (dirs and #dirs > 0) and dirs or nil\n"
"end\n"
"\n"
"--- Check if a package exists in a local repository.\n"
"-- Version numbers are compared as exact string comparison.\n"
"-- @param name string: name of package\n"
"-- @param version string: package version in string format\n"
"-- @return boolean: true if a package is installed,\n"
"-- false otherwise.\n"
"function repos.is_installed(name, version)\n"
"   assert(type(name) == \"string\" and not name:match(\"/\"))\n"
"   assert(type(version) == \"string\")\n"
"      \n"
"   return fs.is_dir(path.install_dir(name, version))\n"
"end\n"
"\n"
"function repos.recurse_rock_manifest_entry(entry, action)\n"
"   assert(type(action) == \"function\")\n"
"\n"
"   if entry == nil then\n"
"      return true\n"
"   end\n"
"\n"
"   local function do_recurse_rock_manifest_entry(tree, parent_path)\n"
"\n"
"      for file, sub in pairs(tree) do\n"
"         local sub_path = (parent_path and (parent_path .. \"/\") or \"\") .. file\n"
"         local ok, err\n"
"\n"
"         if type(sub) == \"table\" then\n"
"            ok, err = do_recurse_rock_manifest_entry(sub, sub_path)\n"
"         else\n"
"            ok, err = action(sub_path)\n"
"         end\n"
"\n"
"         if err then return nil, err end\n"
"      end\n"
"      return true\n"
"   end\n"
"   return do_recurse_rock_manifest_entry(entry)\n"
"end\n"
"\n"
"local function store_package_data(result, rock_manifest, deploy_type)\n"
"   if rock_manifest[deploy_type] then\n"
"      repos.recurse_rock_manifest_entry(rock_manifest[deploy_type], function(file_path)\n"
"         local _, item_name = get_provided_item(deploy_type, file_path)\n"
"         result[item_name] = file_path\n"
"         return true\n"
"      end)\n"
"   end\n"
"end\n"
"\n"
"--- Obtain a table of modules within an installed package.\n"
"-- @param name string: The package name; for example \"luasocket\"\n"
"-- @param version string: The exact version number including revision;\n"
"-- for example \"2.0.1-1\".\n"
"-- @return table: A table of modules where keys are module names\n"
"-- and values are file paths of files providing modules\n"
"-- relative to \"lib\" or \"lua\" rock manifest subtree.\n"
"-- If no modules are found or if package name or version\n"
"-- are invalid, an empty table is returned.\n"
"function repos.package_modules(name, version)\n"
"   assert(type(name) == \"string\" and not name:match(\"/\"))\n"
"   assert(type(version) == \"string\")\n"
"\n"
"   local result = {}\n"
"   local rock_manifest = manif.load_rock_manifest(name, version)\n"
"   if not rock_manifest then return result end\n"
"   store_package_data(result, rock_manifest, \"lib\")\n"
"   store_package_data(result, rock_manifest, \"lua\")\n"
"   return result\n"
"end\n"
"\n"
"--- Obtain a table of command-line scripts within an installed package.\n"
"-- @param name string: The package name; for example \"luasocket\"\n"
"-- @param version string: The exact version number including revision;\n"
"-- for example \"2.0.1-1\".\n"
"-- @return table: A table of commands where keys and values are command names\n"
"-- as strings - file paths of files providing commands\n"
"-- relative to \"bin\" rock manifest subtree.\n"
"-- If no commands are found or if package name or version\n"
"-- are invalid, an empty table is returned.\n"
"function repos.package_commands(name, version)\n"
"   assert(type(name) == \"string\" and not name:match(\"/\"))\n"
"   assert(type(version) == \"string\")\n"
"\n"
"   local result = {}\n"
"   local rock_manifest = manif.load_rock_manifest(name, version)\n"
"   if not rock_manifest then return result end\n"
"   store_package_data(result, rock_manifest, \"bin\")\n"
"   return result\n"
"end\n"
"\n"
"\n"
"--- Check if a rock contains binary executables.\n"
"-- @param name string: name of an installed rock\n"
"-- @param version string: version of an installed rock\n"
"-- @return boolean: returns true if rock contains platform-specific\n"
"-- binary executables, or false if it is a pure-Lua rock.\n"
"function repos.has_binaries(name, version)\n"
"   assert(type(name) == \"string\" and not name:match(\"/\"))\n"
"   assert(type(version) == \"string\")\n"
"\n"
"   local rock_manifest = manif.load_rock_manifest(name, version)\n"
"   if rock_manifest and rock_manifest.bin then\n"
"      for bin_name, md5 in pairs(rock_manifest.bin) do\n"
"         -- TODO verify that it is the same file. If it isn't, find the actual command.\n"
"         if fs.is_actual_binary(dir.path(cfg.deploy_bin_dir, bin_name)) then\n"
"            return true\n"
"         end\n"
"      end\n"
"   end\n"
"   return false\n"
"end\n"
"\n"
"function repos.run_hook(rockspec, hook_name)\n"
"   assert(rockspec:type() == \"rockspec\")\n"
"   assert(type(hook_name) == \"string\")\n"
"\n"
"   local hooks = rockspec.hooks\n"
"   if not hooks then\n"
"      return true\n"
"   end\n"
"   \n"
"   if cfg.hooks_enabled == false then\n"
"      return nil, \"This rockspec contains hooks, which are blocked by the 'hooks_enabled' setting in your LuaRocks configuration.\"\n"
"   end\n"
"   \n"
"   if not hooks.substituted_variables then\n"
"      util.variable_substitutions(hooks, rockspec.variables)\n"
"      hooks.substituted_variables = true\n"
"   end\n"
"   local hook = hooks[hook_name]\n"
"   if hook then\n"
"      util.printout(hook)\n"
"      if not fs.execute(hook) then\n"
"         return nil, \"Failed running \"..hook_name..\" hook.\"\n"
"      end\n"
"   end\n"
"   return true\n"
"end\n"
"\n"
"function repos.should_wrap_bin_scripts(rockspec)\n"
"   assert(rockspec:type() == \"rockspec\")\n"
"\n"
"   if cfg.wrap_bin_scripts ~= nil then\n"
"      return cfg.wrap_bin_scripts\n"
"   end\n"
"   if rockspec.deploy and rockspec.deploy.wrap_bin_scripts == false then\n"
"      return false\n"
"   end\n"
"   return true\n"
"end\n"
"\n"
"local function find_suffixed(file, suffix)\n"
"   local filenames = {file}\n"
"   if suffix and suffix ~= \"\" then\n"
"      table.insert(filenames, 1, file .. suffix)\n"
"   end\n"
"\n"
"   for _, filename in ipairs(filenames) do\n"
"      if fs.exists(filename) then\n"
"         return filename\n"
"      end\n"
"   end\n"
"\n"
"   return nil, table.concat(filenames, \", \") .. \" not found\"\n"
"end\n"
"\n"
"local function check_suffix(filename, suffix)\n"
"   local suffixed_filename, err = find_suffixed(filename, suffix)\n"
"   if not suffixed_filename then\n"
"      return \"\"\n"
"   end\n"
"   return suffixed_filename:sub(#filename + 1)\n"
"end\n"
"\n"
"-- Files can be deployed using versioned and non-versioned names.\n"
"-- Several items with same type and name can exist if they are\n"
"-- provided by different packages or versions. In any case\n"
"-- item from the newest version of lexicographically smallest package\n"
"-- is deployed using non-versioned name and others use versioned names.\n"
"\n"
"local function get_deploy_paths(name, version, deploy_type, file_path, repo)\n"
"   assert(type(name) == \"string\")\n"
"   assert(type(version) == \"string\")\n"
"   assert(type(deploy_type) == \"string\")\n"
"   assert(type(file_path) == \"string\")\n"
"\n"
"   repo = repo or cfg.root_dir\n"
"   local deploy_dir = path[\"deploy_\" .. deploy_type .. \"_dir\"](repo)\n"
"   local non_versioned = dir.path(deploy_dir, file_path)\n"
"   local versioned = path.versioned_name(non_versioned, deploy_dir, name, version)\n"
"   return { nv = non_versioned, v = versioned }\n"
"end\n"
"\n"
"local function check_spot_if_available(name, version, deploy_type, file_path)\n"
"   local item_type, item_name = get_provided_item(deploy_type, file_path)\n"
"   local cur_name, cur_version = manif.get_current_provider(item_type, item_name)\n"
"   if (not cur_name)\n"
"      or (name < cur_name)\n"
"      or (name == cur_name and (version == cur_version\n"
"                                or vers.compare_versions(version, cur_version))) then\n"
"      return \"nv\", cur_name, cur_version, item_name\n"
"   else\n"
"      -- Existing version has priority, deploy new version using versioned name.\n"
"      return \"v\", cur_name, cur_version, item_name\n"
"   end\n"
"end\n"
"\n"
"local function backup_existing(should_backup, target)\n"
"   if not should_backup then\n"
"      fs.delete(target)\n"
"      return\n"
"   end\n"
"   if fs.exists(target) then\n"
"      local backup = target\n"
"      repeat\n"
"         backup = backup..\"~\"\n"
"      until not fs.exists(backup) -- Slight race condition here, but shouldn't be a problem.\n"
"   \n"
"      util.warning(target..\" is not tracked by this installation of LuaRocks. Moving it to \"..backup)\n"
"      local move_ok, move_err = fs.move(target, backup)\n"
"      if not move_ok then\n"
"         return nil, move_err\n"
"      end\n"
"   end\n"
"end\n"
"\n"
"local function op_install(op)\n"
"   local ok, err = fs.make_dir(dir.dir_name(op.dst))\n"
"   if not ok then\n"
"      return nil, err\n"
"   end\n"
"\n"
"   ok, err = op.fn(op.src, op.dst, op.backup)\n"
"   if not ok then\n"
"      return nil, err\n"
"   end\n"
"\n"
"   fs.remove_dir_tree_if_empty(dir.dir_name(op.src))\n"
"end\n"
"\n"
"local function op_rename(op)\n"
"   if op.suffix then\n"
"      local suffix = check_suffix(op.src, op.suffix)\n"
"      op.src = op.src .. suffix\n"
"      op.dst = op.dst .. suffix\n"
"   end\n"
"\n"
"   if fs.exists(op.src) then\n"
"      fs.make_dir(dir.dir_name(op.dst))\n"
"      fs.delete(op.dst)\n"
"      local ok, err = fs.move(op.src, op.dst)\n"
"      fs.remove_dir_tree_if_empty(dir.dir_name(op.src))\n"
"      return ok, err\n"
"   else\n"
"      return true\n"
"   end\n"
"end\n"
"\n"
"local function op_delete(op)\n"
"   if op.suffix then\n"
"      local suffix = check_suffix(op.name, op.suffix)\n"
"      op.name = op.name .. suffix\n"
"   end\n"
"\n"
"   local ok, err = fs.delete(op.name)\n"
"   fs.remove_dir_tree_if_empty(dir.dir_name(op.name))\n"
"   return ok, err\n"
"end\n"
"\n"
"--- Deploy a package from the rocks subdirectory.\n"
"-- @param name string: name of package\n"
"-- @param version string: exact package version in string format\n"
"-- @param wrap_bin_scripts bool: whether commands written in Lua should be wrapped.\n"
"-- @param deps_mode: string: Which trees to check dependencies for:\n"
"-- \"one\" for the current default tree, \"all\" for all trees,\n"
"-- \"order\" for all trees with priority >= the current default, \"none\" for no trees.\n"
"function repos.deploy_files(name, version, wrap_bin_scripts, deps_mode)\n"
"   assert(type(name) == \"string\" and not name:match(\"/\"))\n"
"   assert(type(version) == \"string\")\n"
"   assert(type(wrap_bin_scripts) == \"boolean\")\n"
"\n"
"   local rock_manifest, load_err = manif.load_rock_manifest(name, version)\n"
"   if not rock_manifest then return nil, load_err end\n"
"   \n"
"   local repo = cfg.root_dir\n"
"   local renames = {}\n"
"   local installs = {}\n"
"\n"
"   local function install_binary(source, target, should_backup)\n"
"      if wrap_bin_scripts and fs.is_lua(source) then\n"
"         backup_existing(should_backup, target .. (cfg.wrapper_suffix or \"\"))\n"
"         return fs.wrap_script(source, target, deps_mode, name, version)\n"
"      else\n"
"         backup_existing(should_backup, target)\n"
"         return fs.copy_binary(source, target)\n"
"      end\n"
"   end\n"
"\n"
"   local function move_lua(source, target, should_backup)\n"
"      backup_existing(should_backup, target)\n"
"      return fs.move(source, target, \"read\")\n"
"   end\n"
"\n"
"   local function move_lib(source, target, should_backup)\n"
"      backup_existing(should_backup, target)\n"
"      return fs.move(source, target, \"exec\")\n"
"   end\n"
"\n"
"   if rock_manifest.bin then\n"
"      local source_dir = path.bin_dir(name, version)\n"
"      repos.recurse_rock_manifest_entry(rock_manifest.bin, function(file_path)\n"
"         local source = dir.path(source_dir, file_path)\n"
"         local paths = get_deploy_paths(name, version, \"bin\", file_path, repo)\n"
"         local mode, cur_name, cur_version = check_spot_if_available(name, version, \"bin\", file_path)\n"
"\n"
"         if mode == \"nv\" and cur_name then\n"
"            local cur_paths = get_deploy_paths(cur_name, cur_version, \"bin\", file_path, repo)\n"
"            table.insert(renames, { src = cur_paths.nv, dst = cur_paths.v, suffix = cfg.wrapper_suffix })\n"
"         end\n"
"         local backup = name ~= cur_name or version ~= cur_version\n"
"         table.insert(installs, { fn = install_binary, src = source, dst = mode == \"nv\" and paths.nv or paths.v, backup = backup })\n"
"      end)\n"
"   end\n"
"\n"
"   if rock_manifest.lua then\n"
"      local source_dir = path.lua_dir(name, version)\n"
"      repos.recurse_rock_manifest_entry(rock_manifest.lua, function(file_path)\n"
"         local source = dir.path(source_dir, file_path)\n"
"         local paths = get_deploy_paths(name, version, \"lua\", file_path, repo)\n"
"         local mode, cur_name, cur_version = check_spot_if_available(name, version, \"lua\", file_path)\n"
"\n"
"         if mode == \"nv\" and cur_name then\n"
"            local cur_paths = get_deploy_paths(cur_name, cur_version, \"lua\", file_path, repo)\n"
"            table.insert(renames, { src = cur_paths.nv, dst = cur_paths.v })\n"
"            cur_paths = get_deploy_paths(cur_name, cur_version, \"lib\", file_path:gsub(\"%.lua$\", \".\" .. cfg.lib_extension), repo)\n"
"            table.insert(renames, { src = cur_paths.nv, dst = cur_paths.v })\n"
"         end\n"
"         local backup = name ~= cur_name or version ~= cur_version\n"
"         table.insert(installs, { fn = move_lua, src = source, dst = mode == \"nv\" and paths.nv or paths.v, backup = backup })\n"
"      end)\n"
"   end\n"
"\n"
"   if rock_manifest.lib then\n"
"      local source_dir = path.lib_dir(name, version)\n"
"      repos.recurse_rock_manifest_entry(rock_manifest.lib, function(file_path)\n"
"         local source = dir.path(source_dir, file_path)\n"
"         local paths = get_deploy_paths(name, version, \"lib\", file_path, repo)\n"
"         local mode, cur_name, cur_version = check_spot_if_available(name, version, \"lib\", file_path)\n"
"\n"
"         if mode == \"nv\" and cur_name then\n"
"            local cur_paths = get_deploy_paths(cur_name, cur_version, \"lua\", file_path:gsub(\"%.[^.]+$\", \".lua\"), repo)\n"
"            table.insert(renames, { src = cur_paths.nv, dst = cur_paths.v })\n"
"            cur_paths = get_deploy_paths(cur_name, cur_version, \"lib\", file_path, repo)\n"
"            table.insert(renames, { src = cur_paths.nv, dst = cur_paths.v })\n"
"         end\n"
"         local backup = name ~= cur_name or version ~= cur_version\n"
"         table.insert(installs, { fn = move_lib, src = source, dst = mode == \"nv\" and paths.nv or paths.v, backup = backup })\n"
"      end)\n"
"   end\n"
"\n"
"   for _, op in ipairs(renames) do\n"
"      op_rename(op)\n"
"   end\n"
"   for _, op in ipairs(installs) do\n"
"      op_install(op)\n"
"   end\n"
"\n"
"   local writer = require(\"luarocks.manif.writer\")\n"
"   return writer.add_to_manifest(name, version, nil, deps_mode)\n"
"end\n"
"\n"
"--- Delete a package from the local repository.\n"
"-- @param name string: name of package\n"
"-- @param version string: exact package version in string format\n"
"-- @param deps_mode: string: Which trees to check dependencies for:\n"
"-- \"one\" for the current default tree, \"all\" for all trees,\n"
"-- \"order\" for all trees with priority >= the current default, \"none\" for no trees.\n"
"-- @param quick boolean: do not try to fix the versioned name\n"
"-- of another version that provides the same module that\n"
"-- was deleted. This is used during 'purge', as every module\n"
"-- will be eventually deleted.\n"
"function repos.delete_version(name, version, deps_mode, quick)\n"
"   assert(type(name) == \"string\" and not name:match(\"/\"))\n"
"   assert(type(version) == \"string\")\n"
"   assert(type(deps_mode) == \"string\")\n"
"\n"
"   local rock_manifest, load_err = manif.load_rock_manifest(name, version)\n"
"   if not rock_manifest then return nil, load_err end\n"
"\n"
"   local repo = cfg.root_dir\n"
"   local renames = {}\n"
"   local deletes = {}\n"
"\n"
"   if rock_manifest.bin then\n"
"      repos.recurse_rock_manifest_entry(rock_manifest.bin, function(file_path)\n"
"         local paths = get_deploy_paths(name, version, \"bin\", file_path, repo)\n"
"         local mode, cur_name, cur_version, item_name = check_spot_if_available(name, version, \"bin\", file_path)\n"
"         if mode == \"v\" then\n"
"            table.insert(deletes, { name = paths.v, suffix = cfg.wrapper_suffix })\n"
"         else\n"
"            table.insert(deletes, { name = paths.nv, suffix = cfg.wrapper_suffix })\n"
"\n"
"            local next_name, next_version = manif.get_next_provider(\"command\", item_name)\n"
"            if next_name then\n"
"               local next_paths = get_deploy_paths(next_name, next_version, \"lua\", file_path, repo)\n"
"               table.insert(renames, { src = next_paths.v, dst = next_paths.nv, suffix = cfg.wrapper_suffix })\n"
"            end\n"
"         end\n"
"      end)\n"
"   end\n"
"\n"
"   if rock_manifest.lua then\n"
"      repos.recurse_rock_manifest_entry(rock_manifest.lua, function(file_path)\n"
"         local paths = get_deploy_paths(name, version, \"lua\", file_path, repo)\n"
"         local mode, cur_name, cur_version, item_name = check_spot_if_available(name, version, \"lua\", file_path)\n"
"         if mode == \"v\" then\n"
"            table.insert(deletes, { name = paths.v })\n"
"         else\n"
"            table.insert(deletes, { name = paths.nv })\n"
"\n"
"            local next_name, next_version = manif.get_next_provider(\"module\", item_name)\n"
"            if next_name then\n"
"               local next_lua_paths = get_deploy_paths(next_name, next_version, \"lua\", file_path, repo)\n"
"               table.insert(renames, { src = next_lua_paths.v, dst = next_lua_paths.nv })\n"
"               local next_lib_paths = get_deploy_paths(next_name, next_version, \"lib\", file_path:gsub(\"%.[^.]+$\", \".lua\"), repo)\n"
"               table.insert(renames, { src = next_lib_paths.v, dst = next_lib_paths.nv })\n"
"            end\n"
"         end\n"
"      end)\n"
"   end\n"
"\n"
"   if rock_manifest.lib then\n"
"      repos.recurse_rock_manifest_entry(rock_manifest.lib, function(file_path)\n"
"         local paths = get_deploy_paths(name, version, \"lib\", file_path, repo)\n"
"         local mode, cur_name, cur_version, item_name = check_spot_if_available(name, version, \"lib\", file_path)\n"
"         if mode == \"v\" then\n"
"            table.insert(deletes, { name = paths.v })\n"
"         else\n"
"            table.insert(deletes, { name = paths.nv })\n"
"\n"
"            local next_name, next_version = manif.get_next_provider(\"module\", item_name)\n"
"            if next_name then\n"
"               local next_lua_paths = get_deploy_paths(next_name, next_version, \"lua\", file_path:gsub(\"%.[^.]+$\", \".lua\"), repo)\n"
"               table.insert(renames, { src = next_lua_paths.v, dst = next_lua_paths.nv })\n"
"               local next_lib_paths = get_deploy_paths(next_name, next_version, \"lib\", file_path, repo)\n"
"               table.insert(renames, { src = next_lib_paths.v, dst = next_lib_paths.nv })\n"
"            end\n"
"         end\n"
"      end)\n"
"   end\n"
"\n"
"   for _, op in ipairs(deletes) do\n"
"      op_delete(op)\n"
"   end\n"
"   if not quick then\n"
"      for _, op in ipairs(renames) do\n"
"         op_rename(op)\n"
"      end\n"
"   end\n"
"\n"
"   fs.delete(path.install_dir(name, version))\n"
"   if not get_installed_versions(name) then\n"
"      fs.delete(dir.path(cfg.rocks_dir, name))\n"
"   end\n"
"\n"
"   if quick then\n"
"      return true\n"
"   end\n"
"\n"
"   local writer = require(\"luarocks.manif.writer\")\n"
"   return writer.remove_from_manifest(name, version, nil, deps_mode)\n"
"end\n"
"\n"
"return repos\n"
""
;
