const char luarocks_util_lua[] =
"\n"
"--- Assorted utilities for managing tables, plus a scheduler for rollback functions.\n"
"-- Does not requires modules directly (only as locals\n"
"-- inside specific functions) to avoid interdependencies,\n"
"-- as this is used in the bootstrapping stage of luarocks.core.cfg.\n"
"\n"
"local util = {}\n"
"\n"
"local core = require(\"luarocks.core.util\")\n"
"\n"
"util.cleanup_path = core.cleanup_path\n"
"util.split_string = core.split_string\n"
"util.sortedpairs = core.sortedpairs\n"
"util.deep_merge = core.deep_merge\n"
"util.deep_merge_under = core.deep_merge_under\n"
"util.popen_read = core.popen_read\n"
"util.show_table = core.show_table\n"
"util.printerr = core.printerr\n"
"util.warning = core.warning\n"
"util.keys = core.keys\n"
"\n"
"local unpack = unpack or table.unpack\n"
"\n"
"local scheduled_functions = {}\n"
"local debug = require(\"debug\")\n"
"\n"
"--- Schedule a function to be executed upon program termination.\n"
"-- This is useful for actions such as deleting temporary directories\n"
"-- or failure rollbacks.\n"
"-- @param f function: Function to be executed.\n"
"-- @param ... arguments to be passed to function.\n"
"-- @return table: A token representing the scheduled execution,\n"
"-- which can be used to remove the item later from the list.\n"
"function util.schedule_function(f, ...)\n"
"   assert(type(f) == \"function\")\n"
"   \n"
"   local item = { fn = f, args = {...} }\n"
"   table.insert(scheduled_functions, item)\n"
"   return item\n"
"end\n"
"\n"
"--- Unschedule a function.\n"
"-- This is useful for cancelling a rollback of a completed operation.\n"
"-- @param item table: The token representing the scheduled function that was\n"
"-- returned from the schedule_function call.\n"
"function util.remove_scheduled_function(item)\n"
"   for k, v in pairs(scheduled_functions) do\n"
"      if v == item then\n"
"         table.remove(scheduled_functions, k)\n"
"         return\n"
"      end\n"
"   end\n"
"end\n"
"\n"
"--- Execute scheduled functions.\n"
"-- Some calls create temporary files and/or directories and register\n"
"-- corresponding cleanup functions. Calling this function will run\n"
"-- these function, erasing temporaries.\n"
"-- Functions are executed in the inverse order they were scheduled.\n"
"function util.run_scheduled_functions()\n"
"   local fs = require(\"luarocks.fs\")\n"
"   if fs.change_dir_to_root then\n"
"      fs.change_dir_to_root()\n"
"   end\n"
"   for i = #scheduled_functions, 1, -1 do\n"
"      local item = scheduled_functions[i]\n"
"      item.fn(unpack(item.args))\n"
"   end\n"
"end\n"
"\n"
"--- Produce a Lua pattern that matches precisely the given string\n"
"-- (this is suitable to be concatenating to other patterns,\n"
"-- so it does not include beginning- and end-of-string markers (^$)\n"
"-- @param s string: The input string\n"
"-- @return string: The equivalent pattern\n"
"function util.matchquote(s)\n"
"   return (s:gsub(\"[\?%-+*%[%].%%()$^]\",\"%%%1\"))\n"
"end\n"
"\n"
"--- List of supported arguments.\n"
"-- Arguments that take no parameters are marked with the boolean true.\n"
"-- Arguments that take a parameter are marked with a descriptive string.\n"
"-- Arguments that may take an empty string are described in quotes,\n"
"-- (as in the value for --detailed=\"<text>\").\n"
"-- For all other string values, it means the parameter is mandatory.\n"
"local supported_flags = {\n"
"   [\"all\"] = true,\n"
"   [\"api-key\"] = \"<key>\",\n"
"   [\"append\"] = true,\n"
"   [\"arch\"] = \"<arch>\",\n"
"   [\"bin\"] = true,\n"
"   [\"binary\"] = true,\n"
"   [\"branch\"] = \"<branch-name>\",\n"
"   [\"build-deps\"] = true,\n"
"   [\"chdir\"] = \"<path>\",\n"
"   [\"debug\"] = true,\n"
"   [\"deps\"] = true,\n"
"   [\"deps-mode\"] = \"<mode>\",\n"
"   [\"detailed\"] = \"\\\"<text>\\\"\",\n"
"   [\"dev\"] = true,\n"
"   [\"force\"] = true,\n"
"   [\"force-fast\"] = true,\n"
"   [\"from\"] = \"<server>\",\n"
"   [\"global\"] = true,\n"
"   [\"help\"] = true,\n"
"   [\"home\"] = true,\n"
"   [\"homepage\"] = \"\\\"<url>\\\"\",\n"
"   [\"index\"] = true,\n"
"   [\"issues\"] = true,\n"
"   [\"json\"] = true,\n"
"   [\"keep\"] = true,\n"
"   [\"labels\"] = true,\n"
"   [\"lib\"] = \"<library>\",\n"
"   [\"license\"] = \"\\\"<text>\\\"\",\n"
"   [\"list\"] = true,\n"
"   [\"local\"] = true,\n"
"   [\"local-tree\"] = true,\n"
"   [\"lr-bin\"] = true,\n"
"   [\"lr-cpath\"] = true,\n"
"   [\"lr-path\"] = true,\n"
"   [\"lua-dir\"] = \"<path>\",\n"
"   [\"lua-version\"] = \"<vers>\",\n"
"   [\"lua-versions\"] = \"<versions>\",\n"
"   [\"lua-ver\"] = true,\n"
"   [\"lua-incdir\"] = true,\n"
"   [\"lua-libdir\"] = true,\n"
"   [\"modules\"] = true,\n"
"   [\"mversion\"] = true,\n"
"   [\"namespace\"] = \"<namespace>\",\n"
"   [\"no-bin\"] = true,\n"
"   [\"no-doc\"] = true,\n"
"   [\"no-refresh\"] = true,\n"
"   [\"nodeps\"] = true,\n"
"   [\"old-versions\"] = true,\n"
"   [\"only-deps\"] = true,\n"
"   [\"only-from\"] = \"<server>\",\n"
"   [\"only-server\"] = \"<server>\",\n"
"   [\"only-sources\"] = \"<url>\",\n"
"   [\"only-sources-from\"] = \"<url>\",\n"
"   [\"outdated\"] = true,\n"
"   [\"output\"] = \"<file>\",\n"
"   [\"pack-binary-rock\"] = true,\n"
"   [\"porcelain\"] = true,\n"
"   [\"project-tree\"] = \"<tree>\",\n"
"   [\"quick\"] = true,\n"
"   [\"reset\"] = true,\n"
"   [\"rock-dir\"] = true,\n"
"   [\"rock-license\"] = true,\n"
"   [\"rock-namespace\"] = true,\n"
"   [\"rock-tree\"] = true,\n"
"   [\"rock-trees\"] = true,\n"
"   [\"rockspec\"] = true,\n"
"   [\"rockspec-format\"] = \"<ver>\",\n"
"   [\"scope\"] = \"<system|user|project>\",\n"
"   [\"server\"] = \"<server>\",\n"
"   [\"sign\"] = true,\n"
"   [\"skip-pack\"] = true,\n"
"   [\"source\"] = true,\n"
"   [\"summary\"] = \"\\\"<text>\\\"\",\n"
"   [\"system-config\"] = true,\n"
"   [\"tag\"] = \"<tag>\",\n"
"   [\"test-type\"] = \"<type>\",\n"
"   [\"temp-key\"] = \"<key>\",\n"
"   [\"timeout\"] = \"<seconds>\",\n"
"   [\"to\"] = \"<path>\",\n"
"   [\"tree\"] = \"<path>\",\n"
"   [\"unset\"] = true,\n"
"   [\"user-config\"] = true,\n"
"   [\"verbose\"] = true,\n"
"   [\"verify\"] = true,\n"
"   [\"version\"] = true,\n"
"}\n"
"\n"
"--- Extract flags from an arguments list.\n"
"-- Given string arguments, extract flag arguments into a flags set.\n"
"-- For example, given \"foo\", \"--tux=beep\", \"--bla\", \"bar\", \"--baz\",\n"
"-- it would return the following:\n"
"-- {[\"bla\"] = true, [\"tux\"] = \"beep\", [\"baz\"] = true}, \"foo\", \"bar\".\n"
"function util.parse_flags(...)\n"
"   local args = {...}\n"
"   local flags = {}\n"
"   local i = 1\n"
"   local out = {}\n"
"   local state = \"initial\"\n"
"   while i <= #args do\n"
"      local flag = args[i]:match(\"^%-%-(.*)\")\n"
"      if state == \"initial\" and flag == \"\" then\n"
"         state = \"ignore_flags\"\n"
"      elseif state == \"initial\" and flag then\n"
"         local var,val = flag:match(\"([a-z_%-]*)=(.*)\")\n"
"         if val then\n"
"            local vartype = supported_flags[var]\n"
"            if type(vartype) == \"string\" then\n"
"               if val == \"\" and vartype:sub(1,1) ~= '\"' then\n"
"                  return { ERROR = \"Invalid argument: parameter to flag --\"..var..\"=\"..vartype..\" cannot be empty.\" }\n"
"               end\n"
"               flags[var] = val\n"
"            else\n"
"               if vartype then\n"
"                  return { ERROR = \"Invalid argument: flag --\"..var..\" does not take an parameter.\" }\n"
"               else\n"
"                  return { ERROR = \"Invalid argument: unknown flag --\"..var..\".\" }\n"
"               end\n"
"            end\n"
"         else\n"
"            local var = flag\n"
"            local vartype = supported_flags[var]\n"
"            if type(vartype) == \"string\" then\n"
"               i = i + 1\n"
"               local val = args[i]\n"
"               if not val then\n"
"                  return { ERROR = \"Invalid argument: flag --\"..var..\"=\"..vartype..\" expects a parameter.\" }\n"
"               end\n"
"               if val:match(\"^%-%-.*\") then\n"
"                  return { ERROR = \"Invalid argument: flag --\"..var..\"=\"..vartype..\" expects a parameter (if you really want to pass \"..val..\" as an argument to --\"..var..\", use --\"..var..\"=\"..val..\").\" }\n"
"               else\n"
"                  if val == \"\" and vartype:sub(1,1) ~= '\"' then\n"
"                     return { ERROR = \"Invalid argument: parameter to flag --\"..var..\"=\"..vartype..\" cannot be empty.\" }\n"
"                  end\n"
"                  flags[var] = val\n"
"               end\n"
"            elseif vartype == true then\n"
"               flags[var] = true\n"
"            else\n"
"               return { ERROR = \"Invalid argument: unknown flag --\"..var..\".\" }\n"
"            end\n"
"         end\n"
"      elseif state == \"ignore_flags\" or (state == \"initial\" and not flag) then\n"
"         table.insert(out, args[i])\n"
"      end\n"
"      i = i + 1\n"
"   end\n"
"   return flags, unpack(out)\n"
"end\n"
"\n"
"local var_format_pattern = \"%$%((%a[%a%d_]+)%)\"\n"
"\n"
"-- Check if a set of needed variables are referenced\n"
"-- somewhere in a list of definitions, warning the user\n"
"-- about any unused ones. Each key in needed_set should\n"
"-- appear as a $(XYZ) variable at least once as a\n"
"-- substring of some value of var_defs.\n"
"-- @param var_defs: a table with string keys and string\n"
"-- values, containing variable definitions.\n"
"-- @param needed_set: a set where keys are the names of\n"
"-- needed variables.\n"
"-- @param msg string: the warning message to display.\n"
"function util.warn_if_not_used(var_defs, needed_set, msg)\n"
"   needed_set = core.make_shallow_copy(needed_set)\n"
"   for _, val in pairs(var_defs) do\n"
"      for used in val:gmatch(var_format_pattern) do\n"
"         needed_set[used] = nil\n"
"      end\n"
"   end\n"
"   for var, _ in pairs(needed_set) do\n"
"      util.warning(msg:format(var))\n"
"   end\n"
"end\n"
"\n"
"-- Output any entries that might remain in $(XYZ) format,\n"
"-- warning the user that substitutions have failed.\n"
"-- @param line string: the input string\n"
"local function warn_failed_matches(line)\n"
"   local any_failed = false\n"
"   if line:match(var_format_pattern) then\n"
"      for unmatched in line:gmatch(var_format_pattern) do\n"
"         util.warning(\"unmatched variable \" .. unmatched)\n"
"         any_failed = true\n"
"      end\n"
"   end\n"
"   return any_failed\n"
"end\n"
"\n"
"--- Perform make-style variable substitutions on string values of a table.\n"
"-- For every string value tbl.x which contains a substring of the format\n"
"-- \"$(XYZ)\" will have this substring replaced by vars[\"XYZ\"], if that field\n"
"-- exists in vars. Only string values are processed; this function\n"
"-- does not scan subtables recursively.\n"
"-- @param tbl table: Table to have its string values modified.\n"
"-- @param vars table: Table containing string-string key-value pairs \n"
"-- representing variables to replace in the strings values of tbl.\n"
"function util.variable_substitutions(tbl, vars)\n"
"   assert(type(tbl) == \"table\")\n"
"   assert(type(vars) == \"table\")\n"
"   \n"
"   local updated = {}\n"
"   for k, v in pairs(tbl) do\n"
"      if type(v) == \"string\" then\n"
"         updated[k] = v:gsub(var_format_pattern, vars)\n"
"         if warn_failed_matches(updated[k]) then\n"
"            updated[k] = updated[k]:gsub(var_format_pattern, \"\")\n"
"         end\n"
"      end\n"
"   end\n"
"   for k, v in pairs(updated) do\n"
"      tbl[k] = v\n"
"   end\n"
"end\n"
"\n"
"function util.lua_versions(sort)\n"
"   local versions = { \"5.1\", \"5.2\", \"5.3\", \"5.4\" }\n"
"   local i = 0\n"
"   if sort == \"descending\" then\n"
"      i = #versions + 1\n"
"      return function()\n"
"         i = i - 1\n"
"         return versions[i]\n"
"      end\n"
"   else\n"
"      return function()\n"
"         i = i + 1\n"
"         return versions[i]\n"
"      end\n"
"   end\n"
"end\n"
"\n"
"function util.lua_path_variables()\n"
"   local cfg = require(\"luarocks.core.cfg\")\n"
"   local lpath_var = \"LUA_PATH\"\n"
"   local lcpath_var = \"LUA_CPATH\"\n"
"\n"
"   local lv = cfg.lua_version:gsub(\"%.\", \"_\")\n"
"   if lv ~= \"5_1\" then\n"
"      if os.getenv(\"LUA_PATH_\" .. lv) then\n"
"         lpath_var = \"LUA_PATH_\" .. lv\n"
"      end\n"
"      if os.getenv(\"LUA_CPATH_\" .. lv) then\n"
"         lcpath_var = \"LUA_CPATH_\" .. lv\n"
"      end\n"
"   end\n"
"   return lpath_var, lcpath_var\n"
"end\n"
"\n"
"function util.starts_with(s, prefix)\n"
"   return s:sub(1,#prefix) == prefix\n"
"end\n"
"\n"
"--- Print a line to standard output\n"
"function util.printout(...)\n"
"   io.stdout:write(table.concat({...},\"\\t\"))\n"
"   io.stdout:write(\"\\n\")\n"
"end\n"
"\n"
"function util.title(msg, porcelain, underline)\n"
"   if porcelain then return end\n"
"   util.printout()\n"
"   util.printout(msg)\n"
"   util.printout((underline or \"-\"):rep(#msg))\n"
"   util.printout()\n"
"end\n"
"\n"
"function util.this_program(default)\n"
"   local i = 1\n"
"   local last, cur = default, default\n"
"   while i do\n"
"      local dbg = debug.getinfo(i,\"S\")\n"
"      if not dbg then break end\n"
"      last = cur\n"
"      cur = dbg.source\n"
"      i=i+1\n"
"   end\n"
"   local prog = last:sub(1,1) == \"@\" and last:sub(2) or last\n"
"\n"
"   -- Check if we found the true path of a script that has a wrapper\n"
"   local lrdir, binpath = prog:match(\"^(.*)/lib/luarocks/rocks%-[0-9.]*/[^/]+/[^/]+(/bin/[^/]+)$\")\n"
"   if lrdir then\n"
"      -- Return the wrapper instead\n"
"      return lrdir .. binpath\n"
"   end\n"
"\n"
"   return prog\n"
"end\n"
"\n"
"function util.deps_mode_help(program)\n"
"   local cfg = require(\"luarocks.core.cfg\")\n"
"   return [[\n"
"--deps-mode=<mode>  How to handle dependencies. Four modes are supported:\n"
"                    * all - use all trees from the rocks_trees list\n"
"                      for finding dependencies\n"
"                    * one - use only the current tree (possibly set\n"
"                      with --tree)\n"
"                    * order - use trees based on order (use the current\n"
"                      tree and all trees below it on the rocks_trees list)\n"
"                    * none - ignore dependencies altogether.\n"
"                    The default mode may be set with the deps_mode entry\n"
"                    in the configuration file.\n"
"                    The current default is \"]]..cfg.deps_mode..[[\".\n"
"                    Type ']]..util.this_program(program or \"luarocks\")..[[' with no arguments to see\n"
"                    your list of rocks trees.\n"
"]]\n"
"end\n"
"\n"
"function util.see_help(command, program)\n"
"   return \"See '\"..util.this_program(program or \"luarocks\")..' help'..(command and \" \"..command or \"\")..\"'.\"\n"
"end\n"
"\n"
"function util.announce_install(rockspec)\n"
"   local cfg = require(\"luarocks.core.cfg\")\n"
"   local path = require(\"luarocks.path\")\n"
"\n"
"   local suffix = \"\"\n"
"   if rockspec.description and rockspec.description.license then\n"
"      suffix = \" (license: \"..rockspec.description.license..\")\"\n"
"   end\n"
"\n"
"   util.printout(rockspec.name..\" \"..rockspec.version..\" is now installed in \"..path.root_dir(cfg.root_dir)..suffix)\n"
"   util.printout()\n"
"end\n"
"\n"
"--- Collect rockspecs located in a subdirectory.\n"
"-- @param versions table: A table mapping rock names to newest rockspec versions.\n"
"-- @param paths table: A table mapping rock names to newest rockspec paths.\n"
"-- @param unnamed_paths table: An array of rockspec paths that don't contain rock\n"
"-- name and version in regular format.\n"
"-- @param subdir string: path to subdirectory.\n"
"local function collect_rockspecs(versions, paths, unnamed_paths, subdir)\n"
"   local fs = require(\"luarocks.fs\")\n"
"   local dir = require(\"luarocks.dir\")\n"
"   local path = require(\"luarocks.path\")\n"
"   local vers = require(\"luarocks.core.vers\")\n"
"\n"
"   if fs.is_dir(subdir) then\n"
"      for file in fs.dir(subdir) do\n"
"         file = dir.path(subdir, file)\n"
"\n"
"         if file:match(\"rockspec$\") and fs.is_file(file) then\n"
"            local rock, version = path.parse_name(file)\n"
"\n"
"            if rock then\n"
"               if not versions[rock] or vers.compare_versions(version, versions[rock]) then\n"
"                  versions[rock] = version\n"
"                  paths[rock] = file\n"
"               end\n"
"            else\n"
"               table.insert(unnamed_paths, file)\n"
"            end\n"
"         end\n"
"      end\n"
"   end\n"
"end\n"
"\n"
"--- Get default rockspec name for commands that take optional rockspec name.\n"
"-- @return string or (nil, string): path to the rockspec or nil and error message.\n"
"function util.get_default_rockspec()\n"
"   local versions, paths, unnamed_paths = {}, {}, {}\n"
"   -- Look for rockspecs in some common locations.\n"
"   collect_rockspecs(versions, paths, unnamed_paths, \".\")\n"
"   collect_rockspecs(versions, paths, unnamed_paths, \"rockspec\")\n"
"   collect_rockspecs(versions, paths, unnamed_paths, \"rockspecs\")\n"
"\n"
"   if #unnamed_paths > 0 then\n"
"      -- There are rockspecs not following \"name-version.rockspec\" format.\n"
"      -- More than one are ambiguous.\n"
"      if #unnamed_paths > 1 then\n"
"         return nil, \"Please specify which rockspec file to use.\"\n"
"      else\n"
"         return unnamed_paths[1]\n"
"      end\n"
"   else\n"
"      local fs = require(\"luarocks.fs\")\n"
"      local dir = require(\"luarocks.dir\")\n"
"      local basename = dir.base_name(fs.current_dir())\n"
"\n"
"      if paths[basename] then\n"
"         return paths[basename]\n"
"      end\n"
"\n"
"      local rock = next(versions)\n"
"\n"
"      if rock then\n"
"         -- If there are rockspecs for multiple rocks it's ambiguous.\n"
"         if next(versions, rock) then\n"
"            return nil, \"Please specify which rockspec file to use.\"\n"
"         else\n"
"            return paths[rock]\n"
"         end\n"
"      else\n"
"         return nil, \"Argument missing: please specify a rockspec to use on current directory.\"\n"
"      end\n"
"   end\n"
"end\n"
"\n"
"-- Quote Lua string, analogous to fs.Q.\n"
"-- @param s A string, such as \"hello\"\n"
"-- @return string: A quoted string, such as '\"hello\"'\n"
"function util.LQ(s)\n"
"   return (\"%q\"):format(s)\n"
"end\n"
"\n"
"--- Normalize the --namespace flag and the user/rock syntax for namespaces.\n"
"-- If a namespace is given in user/rock syntax, update the --namespace flag;\n"
"-- If a namespace is given in --namespace flag, update the user/rock syntax.\n"
"-- In case of conflicts, the user/rock syntax takes precedence.\n"
"function util.adjust_name_and_namespace(ns_name, flags)\n"
"   assert(type(ns_name) == \"string\" or not ns_name)\n"
"   assert(type(flags) == \"table\")\n"
"\n"
"   if not ns_name then\n"
"      return\n"
"   elseif ns_name:match(\"%.rockspec$\") or ns_name:match(\"%.rock$\") then\n"
"      return ns_name\n"
"   end\n"
"\n"
"   local name, namespace = util.split_namespace(ns_name)\n"
"   if namespace then\n"
"      flags[\"namespace\"] = namespace\n"
"   end\n"
"   if flags[\"namespace\"] then\n"
"      name = flags[\"namespace\"] .. \"/\" .. name\n"
"   end\n"
"   return name:lower()\n"
"end\n"
"\n"
"-- Split name and namespace of a package name.\n"
"-- @param ns_name a name that may be in \"namespace/name\" format\n"
"-- @return string, string\? - name and optionally a namespace\n"
"function util.split_namespace(ns_name)\n"
"   local p1, p2 = ns_name:match(\"^([^/]+)/([^/]+)$\")\n"
"   if p1 then\n"
"      return p2, p1\n"
"   end\n"
"   return ns_name\n"
"end\n"
"\n"
"function util.deep_copy(tbl)\n"
"   local copy = {}\n"
"   for k, v in pairs(tbl) do\n"
"      if type(v) == \"table\" then\n"
"         copy[k] = util.deep_copy(v)\n"
"      else\n"
"         copy[k] = v\n"
"      end\n"
"   end\n"
"   return copy\n"
"end\n"
"\n"
"-- An ode to the multitude of JSON libraries out there...\n"
"function util.require_json()\n"
"   local list = { \"cjson\", \"dkjson\", \"json\" }\n"
"   for _, lib in ipairs(list) do\n"
"      local json_ok, json = pcall(require, lib)\n"
"      if json_ok then\n"
"         pcall(json.use_lpeg) -- optional feature in dkjson\n"
"         return json_ok, json\n"
"      end\n"
"   end\n"
"   local errmsg = \"Failed loading \"\n"
"   for i, name in ipairs(list) do\n"
"      if i == #list then\n"
"         errmsg = errmsg ..\"and '\"..name..\"'. Use 'luarocks search <partial-name>' to search for a library and 'luarocks install <name>' to install one.\"\n"
"      else\n"
"         errmsg = errmsg ..\"'\"..name..\"', \"\n"
"      end\n"
"   end\n"
"   return nil, errmsg\n"
"end\n"
"\n"
"-- A portable version of fs.exists that can be used at early startup,\n"
"-- before the platform has been determined and luarocks.fs has been\n"
"-- initialized.\n"
"function util.exists(file)\n"
"   local fd, _, code = io.open(file, \"r\")\n"
"   if code == 13 then\n"
"      -- code 13 means \"Permission denied\" on both Unix and Windows\n"
"      -- io.open on folders always fails with code 13 on Windows\n"
"      return true\n"
"   end\n"
"   if fd then\n"
"      fd:close()\n"
"      return true\n"
"   end\n"
"   return false\n"
"end\n"
"\n"
"do\n"
"   local function Q(pathname)\n"
"      if pathname:match(\"^.:\") then\n"
"         return pathname:sub(1, 2) .. '\"' .. pathname:sub(3) .. '\"'\n"
"      end\n"
"      return '\"' .. pathname .. '\"'\n"
"   end\n"
"\n"
"   function util.check_lua_version(lua_exe, luaver)\n"
"      if not util.exists(lua_exe) then\n"
"         return nil\n"
"      end\n"
"      local cmd = 'io.write(_VERSION:sub(5))'\n"
"      -- Tarantool falls into the interactive mode if no script is\n"
"      -- given even when -e option is passed.\n"
"      if lua_exe:find('tarantool') then\n"
"         cmd = cmd .. ' os.exit()'\n"
"      end\n"
"      local lv, err = util.popen_read(Q(lua_exe) .. (' -e \"%s\"'):format(cmd))\n"
"      if luaver and luaver ~= lv then\n"
"         return nil\n"
"      end\n"
"      local ljv\n"
"      if lv == \"5.1\" then\n"
"         local cmd = 'io.write(tostring(jit and jit.version:sub(8)))'\n"
"         -- Exit from the iteractive mode. See the comment above.\n"
"         if lua_exe:find('tarantool') then\n"
"            cmd = cmd .. ' os.exit()'\n"
"         end\n"
"         ljv = util.popen_read(Q(lua_exe) .. (' -e \"%s\"'):format(cmd))\n"
"         if ljv == \"nil\" then\n"
"            ljv = nil\n"
"         end\n"
"      end\n"
"      return lv, ljv\n"
"   end\n"
"\n"
"   local find_lua_bindir\n"
"   do\n"
"      local exe_suffix = (package.config:sub(1, 1) == \"\\\\\" and \".exe\" or \"\")\n"
"\n"
"      local function insert_lua_variants(names, luaver)\n"
"         local variants = {\n"
"            \"lua\" .. luaver .. exe_suffix,\n"
"            \"lua\" .. luaver:gsub(\"%.\", \"\") .. exe_suffix,\n"
"            \"lua-\" .. luaver .. exe_suffix,\n"
"            \"lua-\" .. luaver:gsub(\"%.\", \"\") .. exe_suffix,\n"
"         }\n"
"         for _, name in ipairs(variants) do\n"
"            names[name] = luaver\n"
"            table.insert(names, name)\n"
"         end\n"
"      end\n"
"\n"
"      find_lua_bindir = function(prefix, luaver)\n"
"         local names = {}\n"
"         if luaver then\n"
"            insert_lua_variants(names, luaver)\n"
"         else\n"
"            for v in util.lua_versions(\"descending\") do\n"
"               insert_lua_variants(names, v)\n"
"            end\n"
"         end\n"
"         if luaver == \"5.1\" or not luaver then\n"
"            table.insert(names, \"luajit\" .. exe_suffix)\n"
"            table.insert(names, \"tarantool\")\n"
"         end\n"
"         table.insert(names, \"lua\" .. exe_suffix)\n"
"\n"
"         local bindirs = { prefix .. \"/bin\", prefix }\n"
"         local tried = {}\n"
"         for _, d in ipairs(bindirs) do\n"
"            for _, name in ipairs(names) do\n"
"               local lua_exe = d .. \"/\" .. name\n"
"               local is_wrapper, err = util.lua_is_wrapper(lua_exe)\n"
"               if is_wrapper == false then\n"
"                  local lv, ljv = util.check_lua_version(lua_exe, luaver)\n"
"                  if lv then\n"
"                     return name, d, lv, ljv\n"
"                  end\n"
"               elseif is_wrapper == true or err == nil then\n"
"                  table.insert(tried, lua_exe)\n"
"               else\n"
"                  table.insert(tried, string.format(\"%-13s (%s)\", lua_exe, err))\n"
"               end\n"
"            end\n"
"         end\n"
"         local interp = luaver\n"
"                        and (\"Lua \" .. luaver .. \" interpreter\")\n"
"                        or  \"Lua interpreter\"\n"
"         return nil, interp .. \" not found at \" .. prefix .. \"\\n\" ..\n"
"                     \"Tried:\\t\" .. table.concat(tried, \"\\n\\t\")\n"
"      end\n"
"   end\n"
"\n"
"   function util.find_lua(prefix, luaver)\n"
"      local lua_interpreter, bindir, luajitver\n"
"      lua_interpreter, bindir, luaver, luajitver = find_lua_bindir(prefix, luaver)\n"
"      if not lua_interpreter then\n"
"         return nil, bindir\n"
"      end\n"
"\n"
"      return {\n"
"         lua_version = luaver,\n"
"         luajit_version = luajitver,\n"
"         lua_interpreter = lua_interpreter,\n"
"         lua_dir = prefix,\n"
"         lua_bindir = bindir,\n"
"      }\n"
"   end\n"
"end\n"
"\n"
"function util.lua_is_wrapper(interp)\n"
"   local fd, err = io.open(interp, \"r\")\n"
"   if not fd then\n"
"      return nil, err\n"
"   end\n"
"   local data, err = fd:read(1000)\n"
"   fd:close()\n"
"   if not data then\n"
"      return nil, err\n"
"   end\n"
"   return not not data:match(\"LUAROCKS_SYSCONFDIR\")\n"
"end\n"
"\n"
"function util.opts_table(type_name, valid_opts)\n"
"   local opts_mt = {}\n"
"   \n"
"   opts_mt.__index = opts_mt\n"
"   \n"
"   function opts_mt.type()\n"
"      return type_name\n"
"   end\n"
"\n"
"   return function(opts)\n"
"      for k, v in pairs(opts) do\n"
"         local tv = type(v)\n"
"         if not valid_opts[k] then\n"
"            error(\"invalid option: \"..k)\n"
"         end\n"
"         local vo, optional = valid_opts[k]:match(\"^(.-)(%\?\?)$\")\n"
"         if not (tv == vo or (optional == \"\?\" and tv == nil)) then\n"
"            error(\"invalid type option: \"..k..\" - got \"..tv..\", expected \"..vo)\n"
"         end\n"
"      end\n"
"      for k, v in pairs(valid_opts) do\n"
"         if (not v:find(\"\?\", 1, true)) and opts[k] == nil then\n"
"            error(\"missing option: \"..k)\n"
"         end\n"
"      end\n"
"      return setmetatable(opts, opts_mt)\n"
"   end\n"
"end\n"
"\n"
"return util\n"
"\n"
""
;
